## risk analyisis for present and future damages

#### SET RUN MODE
run<-"local" ## choose whether to run on (local) or (cluster)
test<-"no"   ##  run test data (yes) or real data (no)
MR_data<-"Rdata" ## read (nectdf) file or using R dataset (Rdata)
present<-"yes" ## run analysis for present period (yes-no)

if(run=='cluster'){
  args <- commandArgs(TRUE)
  modelRun <- as.character(args[1])     # model run to analyse
  mainDir <- as.character(args[2])     # main directory
  prot_par<- as.character(args[3]) # level of flood protection - 050: 50% of estimated average protection standards    
  library(sp,lib.loc="/home/dottofr/Rlib")       # needed to load rdgdal library
  Sys.sleep(1)
  library(raster,lib.loc="/home/dottofr/Rlib")
  Sys.sleep(1)
  library(rgdal,lib.loc="/home/dottofr/Rlib") 	# use to read .map raster format
} else if(run=='local'){
  modelRun <- "projection_dis_rcp45_CLMcom-CCLM4-8-17_BC_ICHEC-EC-EARTH_wuConst_statistics_ptId.nc"
  mainDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")
  prot_par<- "050"# level of flood protection     
  ## load libraries
  library(sp)       # needed to load rdgdal library
  library(raster)
  library(rgdal) 	# use to read .map raster format
}
workDir <-paste0(mainDir,"/PESETA_IV_data")
workDir2<-paste0(mainDir,"/PESETA_IV_data") # backup variable to be used if Rdata has different workDir 

#### SET PARAMETERS USED FOR THE SENSISTIVITY ANALYSIS (for sensitivity analysis)
ead_int<-c(-0.29,0.49) # 90% confidence interval to quantify the uncertainty in damage functions
ead_qnt<-c("q05","q50","q95") # quantiles to use to quantify the uncertainty in damage functions 
# prot_par<- "050"

#######################################
print('-----')
print("handle netcdf maps")
## NOTE: many commands are not necessary but are useful as examples for handling netcdf maps
#######################################

## READ and elaborate DISCHARGE DATA from NETCDF or RDATA files
if(MR_data=='netcdf'){
  
  library(ncdf4) ###reapaing and saving netcdf files 
  
  print ('Load map of return levels')
  rl_map <- nc_open(paste0(workDir,"/maps/modelRuns/",modelRun))
  print ('Load map of EFAS point ids')
  id_map <- nc_open(paste0(workDir,"/maps/idChan_5km.nc"))

  ## get relevant variables
  # idEP <- ncvar_get(id_map, "idChan_5km.map")
  v3 <- rl_map$var[['rl']]
  varsize <- v3$varsize
  ndims <- v3$ndims
  nt <- varsize[ndims] # Remember timelike dim is always the LAST dimension!
  print(paste("var size is ",varsize,sep=""))
  print(paste("ndims is ",ndims,sep=""))

  RP <-rl_map$dim['return_period']
  lRP <-RP$return_period$vals # get total number of RP
  print(paste(length(lRP)," return periods",sep=""))
  RY <-rl_map$dim['year']
  lRY <-RY$year$vals # get total number reference years
  print(paste(length(lRY)," reference years",sep=""))
  ID <-rl_map$dim['ptId']
  lID <-ID$year$vals # get total number reference years
  print(paste(length(lID)," points ",sep=""))

## CHECK: maybe the loop is not needed and data can directly be extracted!
  for(ii in 1:length(lRY)) {    # length(lRY)
    for(jj in 1:length(lRP)) {   # length(lRP)
#      print(paste("extract map for year ",lRY[ii]," and RP ",lRP[jj],sep=""))   
      data<- ncvar_get(rl_map, 'rl', count=c(ii,jj, -1))
    }
  }
  dis_f<-aperm(data,c(3,1,2))
  nc_close(rl_map)
  nc_close(id_map)
  rm(data,rl_map,id_map,v3,varsize,ndims,ID,RP,RY)
  
## activate to save data in R workspace
# save(list = ls(all = TRUE), file = paste0(workDir,"/Rdata/",modelRun,".RData"))
gc ()  

}  else if (MR_data=='Rdata'){
  run2<-run
  load(file = paste0(workDir,"/Rdata/",modelRun,".RData"))
  MR_data=='Rdata'
  if(run2=='local'){
    run='local'
  }
}
workDir <-workDir2

## interpolate RP 200y using 150y and 250y
dis_150_250 <- dis_f[,,c(14,15,15)]  
dis_150_250[,,3] <- (dis_150_250[,,1]+dis_150_250[,,2])/2    
## replace 250y wit interpolated 200y
dis_f[,,15] <- dis_150_250[,,3]
rm(dis_150_250)

## keep only used RPs (2,10,20,50,100,200,500,1000)
dis_f<-dis_f[,,c(2,7,9,11,13,15,17,19)]
#dis_f <- dis_f[,-c(1:4),] # delete years 1981-1995

dis_p <- dis_f[,8,] # year 2015 as reference to match discharge and impacts

## TEST: use synthetic discharge values instead of model runs
# dis_f <- array(0, dim = c(num_ep, LRY, nRP), dimnames = c('ID','year','RP'))
if (test=='yes'){
  dis_f <- array(100, dim = c(100, 25, 8), dimnames = c('ID','year','RP'))
# set base discharge  
  dis_f[,,2]<-500
  dis_f[,,3]<-700
  dis_f[,,4]<-800
  dis_f[,,5]<-900
  dis_f[,,6]<-950
  dis_f[,,7]<-1000
  dis_f[,,8]<-1050
  dis_p <- dis_f[,7,] # year 2010 as reference  
# simulate Q increase
  dis_f[,13:25,]<-dis_f[,13:25,]*1.05
  dis_f[,20:25,]<-dis_f[,20:25,]*1.1
}

dis_f[is.na(dis_f)] <- 0
dis_p[is.na(dis_p)] <- 0

print('reading of discharge data done')

#############################
print('---')
print('input data')

##########
## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)
## PERIOD: pf: present rescaled to future (p->f) fp: future actualized to present (f->p)

num_ep <-dim(dis_f)[1] # number of EFAS network cells 

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
## RP_freq = c(2y(0.5-0.1),10y(0.1-0.05),20y(0.05-0.02),50y(0.02-0.01),100y(0.01-0.005),200y(0.005-0.002),500y(0.002-0.001)
## integral is calculated as sum of trapezoids (x=prob-y=dam) RP_freq are the "heights" of each trapezoid
RP_freq<- c(0.4,0.05,0.03,0.01,0.005,0.003,0.001,0.001)
nRP<-length(RP_classes)
nRY<-length(RY_classes)

nRYT<-max(RY_classes)-min(RY_classes)+1
startY=min(RY_classes)
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
  startY=startY-1
} 
#########
## READ INPUT TABLES

## read tables of present impacts - POPULATION, DAMAGE, BUILT-UP
dam_p_t <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_Cost_EP_v4_all.csv", sep=""),row.name=1)
pop_p_t <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_PopExp_EP_luisa.csv", sep=""),row.name=1)
bld_p_t <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_Builtup_EP_ghsl.csv", sep=""),row.name=1)

## read present level of protection in each point (RP in years)
prot_RP_p_table <- read.csv(paste(workDir,"/tables/protection_levels/table_EP_protLev_2018_fp",prot_par,".csv", sep=""))

## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))

## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))

## read table of XY coordinates for EFAS points
table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))

#########
## FORMAT AND INITIALIZE  DATA

## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)


## format tables of present impacts 
pop_p<-array(0, dim= c(nrow(pop_p_t),nRP))
dam_p<-array(0, dim= c(nrow(dam_p_t),nRP))
bld_p<-array(0, dim= c(nrow(bld_p_t),nRP))
for (ii in 1:dim(dam_p_t)[2]) {
  pop_p[,ii+1]<-pop_p_t[,ii]
  dam_p[,ii+1]<-dam_p_t[,ii]
  bld_p[,ii+1]<-bld_p_t[,ii]   
}
pop_p[,8]<-pop_p[,7]
dam_p[,8]<-dam_p[,7]
bld_p[,8]<-bld_p[,7]
rm(pop_p_t,dam_p_t,bld_p_t)

if (test=='yes'){
  pop_p <- dis_p*1000
  dam_p <- dis_p*1000
  dam_p[,8] <- dam_p[,7]
  pop_p[,8] <- pop_p[,7]  
}

## read model run set up and corresponding warming levels
ind_MR<- which(WL_table[,"ModelRun"]==modelRun)
Pr_mod<-WL_table[ind_MR,"Model"]
Pr_rcp<-WL_table[ind_MR,"RCP"]
Pr_wu<-WL_table[ind_MR,"WU"]
ind_base<-WL_table[ind_MR,"Base"]-min(RY_classes)+1
ind_WL15<-WL_table[ind_MR,"X1.5C"]-min(RY_classes)+1
ind_WL20<-WL_table[ind_MR,"X2C"]-min(RY_classes)+1
ind_WL30<-WL_table[ind_MR,"X3C"]-min(RY_classes)+1

## format protection level tables and assign synthetic values (if requested)
prot_RP_p <- prot_RP_p_table[,2] 
rm(prot_RP_p_table)
if (test=='yes'){
  prot_RP_p <- numeric(num_ep)
  prot_RP_p[1:(0.5*num_ep)] <- 20
  prot_RP_p[(0.5*num_ep+1):num_ep] <- 1250  
}
prot_QP <- prot_RP_p*0

## initialize future RP of present protection standards")
prot_RP_pf <- array(0, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),lRY))
## initialize present RP (and associated damage ) of future discharges
# RP_fp <- array(0, dim = c(num_ep, nRY, nRP))
dam_fp <- array(0, dim = c(num_ep, nRY, nRP))
pop_fp <- array(0, dim = c(num_ep, nRY, nRP))
bld_fp <- array(0, dim = c(num_ep, nRY, nRP))

## initialize present and future EAD,EAPA (output variables)
ead_p <- array(0, dim = c(num_ep, 1))
eapa_p <- array(0, dim = c(num_ep, 1))
# eab_p <- array(0, dim = c(num_ep, 1))
ead_f <- array(0, dim = c(num_ep, nRY))
eapa_f <- array(0, dim = c(num_ep, nRY))
# eab_f <- array(0, dim = c(num_ep, nRY))
colnames(ead_f) <- RY_classes
# colnames(eab_f) <- RY_classes
colnames(eapa_f) <- RY_classes

ead_p_countries <- array(0, dim = c(nCY,2))
ead_p_nuts2 <- array(0, dim = c(nNU,2))
eapa_p_countries <- array(0, dim = c(nCY,2))
eapa_p_nuts2 <- array(0, dim = c(nNU,2))
# eab_p_countries <- array(0, dim = c(nCY,2))
# eab_p_nuts2 <- array(0, dim = c(nNU,2))

ead_f_nuts2 <- array(0, dim = c(nNU,nRY))
eapa_f_nuts2 <- array(0, dim = c(nNU,nRY))
# eab_f_nuts2 <- array(0, dim = c(nNU,nRY))
colnames(ead_f_nuts2) <- RY_classes
colnames(eapa_f_nuts2) <- RY_classes
# colnames(eab_f_nuts2) <- RY_classes

## create directories
dir.create( paste0(workDir,"/tables/EAI_Europe/ead_present"))
dir.create( paste0(workDir,"/tables/EAI_Europe/eapa_present"))   
# dir.create( paste0(workDir,"/tables/EAI_Europe/eab_present"))

dir.create( paste0(workDir,"/tables/EAI_Europe/ead_ep_clim_fp",prot_par))
dir.create( paste0(workDir,"/tables/EAI_Europe/eapa_ep_clim_fp",prot_par))
# dir.create( paste0(workDir,"/tables/EAI_Europe/eab_ep_clim_",prot_par))

dir.create( paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_clim_fp",prot_par))
dir.create( paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_clim_fp",prot_par))

###################
## POSSIBLE ALTERNATIVE METHOD with lapply functions
#  NOTE: functions should be used in loop and not be vectorized, othewise will give errors! 

# # create piecewise inear functions
# EDFun<-lapply(EDtable_2, approxfun, x=RP_freq)
# 
# #Integral of the population affected and damage curve, using the estimates for the selected frequencies freqs
# EP_ind<-which(prot_RP_p>0)
# eapa_p2<-list()
# eapa_p2[EP_ind]<-lapply(PAFun[EP_ind],integrate, lower=0, upper=(1/prot_RP_p[EP_ind]))
# 
# ExpDamYear<-lapply(EDFun,integrate, lower=0, upper=(1/prot_RP_p[])) 

#############################
print('---')
print('PERFORM RISK ANALYSIS FOR PRESENT SCENARIOS')
print('---')

######################
if(present=='yes'){
  print("calculate expected annual impacts for the present (step 1P)")
  
  ## ACTIVATE TO TEST WITH NO PROT
  # prot_RP_p <- prot_RP_p*0 
  
  print("- population affected")
  for (np in 1:num_ep) {
  #   print(paste("EP ",np,sep=""))
    if(prot_RP_p[np]>=RP_classes[8]){     ## residual risk
      eapa_p[np,1]<-pop_p[np,8]*(1/prot_RP_p[np])
    }else{
      unprot_RP_p<- which(prot_RP_p[np]< RP_classes[])
      popaff<-RP_classes*0
      cumul_p<-RP_freq    
      ## interpolate popaff between highest-protected and lowest-unprotected ref RP  
      if(min(unprot_RP_p)>1){
        int_ind<-min(unprot_RP_p)-1  
        popaff[int_ind]<-pop_p[np,int_ind]+ (pop_p[np,int_ind+1]-pop_p[np,int_ind])*(prot_RP_p[np]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
        popaff[int_ind]<-(popaff[int_ind]+pop_p[np,int_ind+1])/2
        cumul_p[int_ind]<-1/prot_RP_p[np]-1/RP_classes[int_ind+1]                                     
        eapa_p[np,1]<-popaff[int_ind]*cumul_p[int_ind]          
      }
      popaff[max(unprot_RP_p)]<-pop_p[np,max(unprot_RP_p)]    
      ## consider RP classes above local protection level    
      if(min(unprot_RP_p)<8){
        for (yy in min(unprot_RP_p):(max(unprot_RP_p)-1)) {
          popaff[yy]<-(pop_p[np,yy]+pop_p[np,yy+1])/2
        }                                    
      }
      eapa_p[np,1]<-eapa_p[np,1]+sum(popaff[unprot_RP_p]*cumul_p[unprot_RP_p])   
    }
  }
  
  print("- economic damage")
  for (np in 1:num_ep) {
  #  print(paste("EP ",np,sep=""))
    if(prot_RP_p[np]>=RP_classes[8]){     ## residual risk
      ead_p[np,1]<-dam_p[np,8]*(1/prot_RP_p[np])
    }else{
      unprot_RP_p<- which(prot_RP_p[np]< RP_classes[])
      damage<-RP_classes*0
      cumul_p<-RP_freq    
      ## interpolate damage between highest-protected and lowest-unprotected ref RP  
      if(min(unprot_RP_p)>1){
        int_ind<-min(unprot_RP_p)-1  
        damage[int_ind]<-dam_p[np,int_ind]+ (dam_p[np,int_ind+1]-dam_p[np,int_ind])*(prot_RP_p[np]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
        damage[int_ind]<-(damage[int_ind]+dam_p[np,int_ind+1])/2
        cumul_p[int_ind]<-1/prot_RP_p[np]-1/RP_classes[int_ind+1]
        ead_p[np,1]<-damage[int_ind]*cumul_p[int_ind]  
      }
      damage[max(unprot_RP_p)]<-dam_p[np,max(unprot_RP_p)]    
      ## consider RP classes above local protection level    
      if(min(unprot_RP_p)<8){
        for (yy in min(unprot_RP_p):(max(unprot_RP_p)-1)) {
          damage[yy]<-(dam_p[np,yy]+dam_p[np,yy+1])/2
        }                                    
      }
      ead_p[np,1]<-ead_p[np,1]+sum(damage[unprot_RP_p]*cumul_p[unprot_RP_p])   
    }
  }
  
  print("- built-up area affected")
#   for (np in 1:num_ep) {
#     #   print(paste("EP ",np,sep=""))
#     if(prot_RP_p[np]>=RP_classes[8]){     ## residual risk
#       eab_p[np,1]<-bld_p[np,8]*(1/prot_RP_p[np])
#     }else{
#       unprot_RP_p<- which(prot_RP_p[np]< RP_classes[])
#       bldaff<-RP_classes*0
#       cumul_p<-RP_freq    
#       ## interpolate bldaff between highest-protected and lowest-unprotected ref RP  
#       if(min(unprot_RP_p)>1){
#         int_ind<-min(unprot_RP_p)-1  
#         bldaff[int_ind]<-bld_p[np,int_ind]+ (bld_p[np,int_ind+1]-bld_p[np,int_ind])*(prot_RP_p[np]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
#         bldaff[int_ind]<-(bldaff[int_ind]+bld_p[np,int_ind+1])/2
#         cumul_p[int_ind]<-1/prot_RP_p[np]-1/RP_classes[int_ind+1]                                     
#         eab_p[np,1]<-bldaff[int_ind]*cumul_p[int_ind]          
#       }
#       bldaff[max(unprot_RP_p)]<-bld_p[np,max(unprot_RP_p)]    
#       ## consider RP classes above local protection level    
#       if(min(unprot_RP_p)<8){
#         for (yy in min(unprot_RP_p):(max(unprot_RP_p)-1)) {
#           bldaff[yy]<-(bld_p[np,yy]+bld_p[np,yy+1])/2
#         }                                    
#       }
#       eab_p[np,1]<-eab_p[np,1]+sum(bldaff[unprot_RP_p]*cumul_p[unprot_RP_p])   
#     }
#   }  

  ## write ep values to check  
  fileDir<- paste0(workDir,"/tables/EAI_Europe")
  write.csv(ead_p,file = paste0(fileDir,"/ead_present/ead_ep_present_fp",prot_par,"_ed",ead_qnt[2],".csv"),row.names=FALSE)
  write.csv(eapa_p,file = paste0(fileDir,"/eapa_present/eapa_ep_present_fp",prot_par,".csv"),row.names=FALSE)  
  
  print("step 1P completed")
  
  ######################
  print("aggregate expected annual impacts at country and NUTS2 scale")
  
  for (np in 1:nNU) {
    # np<-1
  #  print(paste("NUTS2 ",list_nuts2[np],sep=""))
    ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
    ead_p_nuts2[np,1]<-list_nuts2[np] 
    ead_p_nuts2[np,2]<-sum(ead_p[ind_nuts]) 
    eapa_p_nuts2[np,1]<-list_nuts2[np] 
    eapa_p_nuts2[np,2]<-sum(eapa_p[ind_nuts])   
#     eab_p_nuts2[np,1]<-list_nuts2[np] 
#     eab_p_nuts2[np,2]<-sum(eab_p[ind_nuts])     
    ## NOTE: still to decide how to sum EAD by country using tables!!!! 
  }
  ead_p_nuts2<-ead_p_nuts2[order(ead_p_nuts2[,1]),]
  eapa_p_nuts2<-eapa_p_nuts2[order(eapa_p_nuts2[,1]),]
#  eab_p_nuts2<-eab_p_nuts2[order(eab_p_nuts2[,1]),]    
  ## calculate results accounting for the uncertainty in damage functions 
  ##  (we use a confidence interval given by q1-q3 quantiles)    
  ead_p_nuts2_q1 <- ead_p_nuts2
  ead_p_nuts2_q1[,2] <- ead_p_nuts2[,2]*(1+ead_int[1])
  ead_p_nuts2_q3 <- ead_p_nuts2
  ead_p_nuts2_q3[,2] <- ead_p_nuts2[,2]*(1+ead_int[2])

  ## calculate country-scale results accounting also for the uncertainty in damage functions  
  ead_p_countries_q1 <- ead_p_countries
  ead_p_countries_q3 <- ead_p_countries  
  for (nc in 1:nCY) {
    #    print(paste("Country ",list_countries[nc],sep=""))
    ind_cnty<-which(table_countries[,1]==list_countries[nc])
    ead_p_countries[nc,1]<-list_countries[nc] 
    ead_p_countries[nc,2]<-sum(ead_p_nuts2[ind_cnty,2]) # doesn't work if there are NA values
    ead_p_countries_q1[nc,1]<-list_countries[nc] 
    ead_p_countries_q1[nc,2]<-sum(ead_p_nuts2_q1[ind_cnty,2]) # doesn't work if there are NA values
    ead_p_countries_q3[nc,1]<-list_countries[nc] 
    ead_p_countries_q3[nc,2]<-sum(ead_p_nuts2_q3[ind_cnty,2]) # doesn't work if there are NA values    
    eapa_p_countries[nc,1]<-list_countries[nc] 
    eapa_p_countries[nc,2]<-sum(eapa_p_nuts2[ind_cnty,2]) # doesn't work if there are NA values  
#     eab_p_countries[nc,1]<-list_countries[nc] 
#     eab_p_countries[nc,2]<-sum(eab_p_nuts2[ind_cnty,2]) # doesn't work if there are NA values     
  }
  
  ## SAVE ALL FILES 
  fileDir<- paste0(workDir,"/tables/EAI_Europe")
  write.csv(ead_p_nuts2,file = paste0(fileDir,"/ead_present/ead_nuts2_present_fp",prot_par,"_ed",ead_qnt[2],".csv"),row.names=FALSE)
  write.csv(ead_p_nuts2_q1,file = paste0(fileDir,"/ead_present/ead_nuts2_present_fp",prot_par,"_ed",ead_qnt[1],".csv"),row.names=FALSE)
  write.csv(ead_p_nuts2_q3,file = paste0(fileDir,"/ead_present/ead_nuts2_present_fp",prot_par,"_ed",ead_qnt[3],".csv"),row.names=FALSE)
  write.csv(ead_p_countries,file = paste0(fileDir,"/ead_present/ead_countries_present_fp",prot_par,"_ed",ead_qnt[2],".csv"),row.names=FALSE)  
  write.csv(ead_p_countries_q1,file = paste0(fileDir,"/ead_present/ead_countries_present_fp",prot_par,"_ed",ead_qnt[1],".csv"),row.names=FALSE)
  write.csv(ead_p_countries_q3,file = paste0(fileDir,"/ead_present/ead_countries_present_fp",prot_par,"_ed",ead_qnt[3],".csv"),row.names=FALSE)
  write.csv(eapa_p_nuts2,file = paste0(fileDir,"/eapa_present/eapa_nuts2_present_fp",prot_par,".csv"),row.names=FALSE)
  write.csv(eapa_p_countries,file = paste0(fileDir,"/eapa_present/eapa_countries_present_fp",prot_par,".csv"),row.names=FALSE)
#   write.csv(eab_p_nuts2,file = paste0(fileDir,"/eab_present/eab_nuts2_present_fp",prot_par,".csv""),row.names=FALSE)
#   write.csv(eab_p_countries,file = paste0(fileDir,"/eab_present/eab_countries_present_fp",prot_par,".csv"),row.names=FALSE)
  
  print('Risk analysis for present conditions COMPLETED')

}else{
  print("skip calculation of present-day impacts (step 1P)")
}

#############################
print('---')
print('PERFORM RISK ANALYSIS FOR FUTURE SCENARIOS')
print('---')
#############################

print("extract present protection values (RP) in terms of Q (step 1F)")

## general formula for extrapolating linearly
# Q[RP2>prot_RP_p>RP1] <- Qbase[RP1]+ (Qbase[RP2]-Qbase[RP1])*(prot_RP_p[RP2>prot_RP_p>RP1] - RP1)/(RP2-RP1) 

mask_P<- which(prot_RP_p[]>2 & prot_RP_p[]<=10)
prot_QP[mask_P] <-dis_p[mask_P,1] + (dis_p[mask_P,2]-dis_p[mask_P,1])*(prot_RP_p[prot_RP_p[]>2 & prot_RP_p[]<=10] - 2)/(10-2)
mask_P<- which(prot_RP_p[]>10 & prot_RP_p[]<=20)
prot_QP[mask_P] <-dis_p[mask_P,2] + (dis_p[mask_P,3]-dis_p[mask_P,2])*(prot_RP_p[prot_RP_p[]>10 & prot_RP_p[]<=20] - 10)/(20-10)
mask_P<- which(prot_RP_p[]>20 & prot_RP_p[]<=50)
prot_QP[mask_P] <-dis_p[mask_P,3] + (dis_p[mask_P,4]-dis_p[mask_P,3])*(prot_RP_p[prot_RP_p[]>20 & prot_RP_p[]<=50] - 20)/(50-20)
mask_P<- which(prot_RP_p[]>50 & prot_RP_p[]<=100)
prot_QP[mask_P] <-dis_p[mask_P,4] + (dis_p[mask_P,5]-dis_p[mask_P,4])*(prot_RP_p[prot_RP_p[]>50 & prot_RP_p[]<=100] - 50)/(100-50)
mask_P<- which(prot_RP_p[]>100 & prot_RP_p[]<=200)
prot_QP[mask_P] <-dis_p[mask_P,5] + (dis_p[mask_P,6]-dis_p[mask_P,5])*(prot_RP_p[prot_RP_p[]>100 & prot_RP_p[]<=200] - 100)/(200-100)
mask_P<- which(prot_RP_p[]>200 & prot_RP_p[]<=500)
prot_QP[mask_P] <-dis_p[mask_P,6] + (dis_p[mask_P,7]-dis_p[mask_P,6])*(prot_RP_p[prot_RP_p[]>200 & prot_RP_p[]<=500] - 200)/(500-200)
mask_P<- which(prot_RP_p[]>500 & prot_RP_p[]<=1000)
prot_QP[mask_P] <-dis_p[mask_P,7] + (dis_p[mask_P,8]-dis_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>500 & prot_RP_p[]<=1000] - 500)/(1000-500)
mask_P<- which(prot_RP_p[]>1000)
prot_QP[mask_P] <-dis_p[mask_P,8] + (dis_p[mask_P,8]-dis_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>1000] - 1000)/(1000-500)

rm(mask_P)
print("step 1F completed")

#########################
print("calculate future RP of present protection standards (step 2F)")
## all future RP > rescaled protected RP will cause damage

for (np in 1:num_ep) {
#  print(paste("EP ",np,sep=""))  
  for (ny in 1:nRY) {
#    ny<-1
    if (dis_f[np,ny,1]==0){
      prot_RP_pf[np,ny]<-100000
    }else{
      if (prot_QP[np]>=dis_f[np,ny,nRP]) {  # RP given using linear intepolation from 1000 and 500y values
#      prot_RP_pf[np,ny] <-1000  
        prot_RP_pf[np,ny]<-RP_classes[nRP-1]+ (RP_classes[nRP]-RP_classes[nRP-1])*(prot_QP[np]-dis_f[np,ny,nRP-1])/(dis_f[np,ny,nRP]-dis_f[np,ny,nRP-1])      
      } else if (prot_QP[np]<=dis_f[np,ny,1]) {
        prot_RP_pf[np,ny] <-2  
      } else {  
        irp <-min(which(dis_f[np,ny,]>prot_QP[np]))
        prot_RP_pf[np,ny]<-RP_classes[irp-1]+ (RP_classes[irp]-RP_classes[irp-1])*(prot_QP[np]-dis_f[np,ny,irp-1])/(dis_f[np,ny,irp]-dis_f[np,ny,irp-1])
      }
    }
  }
}
print("step 2F completed")

########################
print("calculate present RP and associated impacts of future discharges  (step 3F)")
## actualize damage for reference future RPs to present (Q return levels for the future will differ from present values)

for (np in 1:num_ep) {
#  print(paste("EP ",np,sep=""))
  for (ny in 1:nRY) {
    #    ny<-1
    for (nr in 1:nRP){
      if (dis_f[np,ny,1]==0){  # to avoid NAs where there's no discharge values
        prot_RP_pf[np,ny]<-100000
      } else {
        if (dis_f[np,ny,nr]>=dis_p[np,nRP]) {   # RP given using linear intepolation from 1000 and 500y values
#          RP_fp[np,ny,nr]<- RP_classes[nRP-1]+ (RP_classes[nRP]-RP_classes[nRP-1])*(dis_f[np,ny,nr]-dis_p[np,nRP-1])/(dis_p[np,nRP]-dis_p[np,nRP-1])    
          dam_fp[np,ny,nr] <-dam_p[np,nRP]  
          pop_fp[np,ny,nr] <-pop_p[np,nRP]          
          bld_fp[np,ny,nr] <-bld_p[np,nRP]           
        } else if (dis_f[np,ny,nr]<=dis_p[np,1]) {
#          RP_fp[np,ny,nr]<- 2    
          dam_fp[np,ny,nr] <-dam_p[np,1]         
          pop_fp[np,ny,nr] <-pop_p[np,1]                 
          bld_fp[np,ny,nr] <-bld_p[np,1]                           
        } else {
          irp <-min(which(dis_p[np,]>dis_f[np,ny,nr]))
#          RP_fp[np,ny,nr]<- RP_classes[irp-1]+ (RP_classes[irp]-RP_classes[irp-1])*(dis_f[np,ny,nr]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])
          dam_fp[np,ny,nr]<- dam_p[np,irp-1]+ (dam_p[np,irp]-dam_p[np,irp-1])*(dis_f[np,ny,nr]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])
          pop_fp[np,ny,nr]<- pop_p[np,irp-1]+ (pop_p[np,irp]-pop_p[np,irp-1])*(dis_f[np,ny,nr]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])
          bld_fp[np,ny,nr]<- bld_p[np,irp-1]+ (bld_p[np,irp]-bld_p[np,irp-1])*(dis_f[np,ny,nr]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])          
        }
      }
    }
  }
}
print("step 3F completed")

######################
print("calculate expected annual impacts for the future  (step 4F)")

## ACTIVATE TO TEST WITH NO PROT
# prot_RP_p <- prot_RP_p*0 

print("- population affected")
for (np in 1:num_ep) {
#  print(paste("EP ",np,sep=""))
  for (ny in 1:nRY){ 
    if(prot_RP_pf[np,ny]>=RP_classes[8]){     ## residual risk
      eapa_f[np,ny]<-pop_fp[np,ny,8]*(1/prot_RP_pf[np,ny])
    }else{
      unprot_RP_fp<- which(prot_RP_pf[np,ny]< RP_classes[])
      popaff<-RP_classes*0
      cumul_p<-RP_freq    
## interpolate popaff between highest-protected and lowest-unprotected ref RP  
      if(min(unprot_RP_fp)>1){
        int_ind<-min(unprot_RP_fp)-1  
        popaff[int_ind]<-pop_fp[np,ny,int_ind]+ (pop_fp[np,ny,int_ind+1]-pop_fp[np,ny,int_ind])*(prot_RP_pf[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
        popaff[int_ind]<-(popaff[int_ind]+pop_fp[np,ny,int_ind+1])/2
        cumul_p[int_ind]<-1/prot_RP_pf[np,ny]-1/RP_classes[int_ind+1]                                     
        eapa_f[np,ny]<-popaff[int_ind]*cumul_p[int_ind]          
      }
      popaff[max(unprot_RP_fp)]<-pop_fp[np,ny,max(unprot_RP_fp)]    
## consider RP classes above local protection level    
      if(min(unprot_RP_fp)<8){
        for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
          popaff[yy]<-(pop_fp[np,ny,yy]+pop_fp[np,ny,yy+1])/2
        }                                    
      }
      eapa_f[np,ny]<-eapa_f[np,ny]+sum(popaff[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
    }
  }
}

print("- economic damage")
for (np in 1:num_ep) {
#  print(paste("EP ",np,sep="")) 
  for (ny in 1:nRY){ 
    if(prot_RP_pf[np,ny]>=RP_classes[8]){     ## residual risk
      ead_f[np,ny]<-dam_fp[np,ny,8]*(1/prot_RP_pf[np,ny])
    }else{
      unprot_RP_fp<- which(prot_RP_pf[np,ny]< RP_classes[])
      damage<-RP_classes*0
      cumul_p<-RP_freq    
      ## interpolate damage between highest-protected and lowest-unprotected ref RP  
      if(min(unprot_RP_fp)>1){
        int_ind<-min(unprot_RP_fp)-1  
        damage[int_ind]<-dam_fp[np,ny,int_ind]+ (dam_fp[np,ny,int_ind+1]-dam_fp[np,ny,int_ind])*(prot_RP_pf[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
        damage[int_ind]<-(damage[int_ind]+dam_fp[np,ny,int_ind+1])/2
        cumul_p[int_ind]<-1/prot_RP_pf[np,ny]-1/RP_classes[int_ind+1]                                     
        ead_f[np,ny]<-damage[int_ind]*cumul_p[int_ind]          
      }
      damage[max(unprot_RP_fp)]<-dam_fp[np,ny,max(unprot_RP_fp)]    
      ## consider RP classes above local protection level    
      if(min(unprot_RP_fp)<8){
        for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
          damage[yy]<-(dam_fp[np,ny,yy]+dam_fp[np,ny,yy+1])/2
        }                                    
      }
      ead_f[np,ny]<-ead_f[np,ny]+sum(damage[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
    }
  }
}

print("- builtup area affected")
# for (np in 1:num_ep) {
#   #  print(paste("EP ",np,sep=""))
#   for (ny in 1:nRY){ 
#     if(prot_RP_pf[np,ny]>=RP_classes[8]){     ## residual risk
#       eab_f[np,ny]<-bld_fp[np,ny,8]*(1/prot_RP_pf[np,ny])
#     }else{
#       unprot_RP_fp<- which(prot_RP_pf[np,ny]< RP_classes[])
#       bldaff<-RP_classes*0
#       cumul_p<-RP_freq    
#       ## interpolate bldaff between highest-protected and lowest-unprotected ref RP  
#       if(min(unprot_RP_fp)>1){
#         int_ind<-min(unprot_RP_fp)-1  
#         bldaff[int_ind]<-bld_fp[np,ny,int_ind]+ (bld_fp[np,ny,int_ind+1]-bld_fp[np,ny,int_ind])*(prot_RP_pf[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
#         bldaff[int_ind]<-(bldaff[int_ind]+bld_fp[np,ny,int_ind+1])/2
#         cumul_p[int_ind]<-1/prot_RP_pf[np,ny]-1/RP_classes[int_ind+1]                                     
#         eab_f[np,ny]<-bldaff[int_ind]*cumul_p[int_ind]          
#       }
#       bldaff[max(unprot_RP_fp)]<-bld_fp[np,ny,max(unprot_RP_fp)]    
#       ## consider RP classes above local protection level    
#       if(min(unprot_RP_fp)<8){
#         for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
#           bldaff[yy]<-(bld_fp[np,ny,yy]+bld_fp[np,ny,yy+1])/2
#         }                                    
#       }
#       eab_f[np,ny]<-eab_f[np,ny]+sum(bldaff[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
#     }
#   }
# }
print("step 4F completed")

######################
print("aggregate expected annual impacts at NUTS2 scale (step 5F")
for (ny in 1:nRY){ 
  for (np in 1:nNU) {
#    print(paste("NUTS2 ",list_nuts2[np],sep=""))
    ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
    ead_f_nuts2[np,ny]<-sum(ead_f[ind_nuts,ny]) 
    eapa_f_nuts2[np,ny]<-sum(eapa_f[ind_nuts,ny]) 
#    eab_f_nuts2[np,ny]<-sum(eab_f[ind_nuts,ny])     
  }
}

print("step 5F completed")

#####################
print("- calculate annual impacts and impacts for warming levels (step 6F)")
## NOTE: WL averages are not calculated anymore, annual values are already 
## averaged over 30 years

## interpolate model values every 5 years
## damage
ead_nuts2_ext <- array(0, dim = c(nNU,nRYT),dimnames = list(list_nuts2,c(startY:max(RY_classes))))
# ead_nuts2_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))
ead_nuts2_ext[,1]<-ead_f_nuts2[,1]
for (ii in 1:(nRY-1)){
  ead_nuts2_ext[,ii*5+1]<-ead_f_nuts2[,ii+1]
}
for (jj in 1:(nRY-1)){
  ead_nuts2_ext[,jj*5-3]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])/5
  ead_nuts2_ext[,jj*5-2]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])*2/5
  ead_nuts2_ext[,jj*5-1]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])*3/5
  ead_nuts2_ext[,jj*5-0]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])*4/5  
}

## population
eapa_nuts2_ext <- array(0, dim = c(nNU,nRYT),dimnames = list(list_nuts2,c(startY:max(RY_classes))))
# eapa_nuts2_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))
eapa_nuts2_ext[,1]<-eapa_f_nuts2[,1]
for (ii in 1:(nRY-1)){
  eapa_nuts2_ext[,ii*5+1]<-eapa_f_nuts2[,ii+1]
}
for (jj in 1:(nRY-1)){
  eapa_nuts2_ext[,jj*5-3]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])/5
  eapa_nuts2_ext[,jj*5-2]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])*2/5
  eapa_nuts2_ext[,jj*5-1]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])*3/5
  eapa_nuts2_ext[,jj*5-0]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])*4/5  
}

# ## damage
# for (np in 1:nNU) {
#   ead_nuts2_WL[np,1]<-mean(ead_nuts2_ext[np,max(1,ind_base-14):min(ind_base+15,nRYT)])  
#   ead_nuts2_WL[np,2]<-mean(ead_nuts2_ext[np,max(1,ind_WL15-14):min(ind_WL15+15,nRYT)])
#   ead_nuts2_WL[np,3]<-mean(ead_nuts2_ext[np,max(1,ind_WL20-14):min(ind_WL20+15,nRYT)])
#   if(WL_table[ind_MR,"X3C"]==-1){
#     ead_nuts2_WL[np,4]<-0
#   }else{
#     ead_nuts2_WL[np,4]<-mean(ead_nuts2_ext[np,max(1,ind_WL30-14):min(ind_WL30+15,nRYT)])
#   }
# }
# ## population
# for (np in 1:nNU) {
#   eapa_nuts2_WL[np,1]<-mean(eapa_nuts2_ext[np,max(1,ind_base-14):min(ind_base+15,nRYT)])  
#   eapa_nuts2_WL[np,2]<-mean(eapa_nuts2_ext[np,max(1,ind_WL15-14):min(ind_WL15+15,nRYT)])
#   eapa_nuts2_WL[np,3]<-mean(eapa_nuts2_ext[np,max(1,ind_WL20-14):min(ind_WL20+15,nRYT)])
#   if(WL_table[ind_MR,"X3C"]==-1){
#     eapa_nuts2_WL[np,4]<-0
#   }else{
#     eapa_nuts2_WL[np,4]<-mean(eapa_nuts2_ext[np,max(1,ind_WL30-14):min(ind_WL30+15,nRYT)])
#   }
# }

print("step 6F completed")

#####################
print("print and save results")
## write file of EP results every 5 years
write.csv(ead_f,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_clim_fp",prot_par,"/ead_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"))
write.csv(eapa_f,file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_clim_fp",prot_par,"/eapa_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"))
# write.csv(eab_f,file = paste0(workDir,"/tables/EAI_Europe/eab_ep_clim_",prot_lev,"/eab_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"))
## write impacts at NUTS2 level for all years
write.csv(ead_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_clim_fp",prot_par,"/ead_nuts2_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
 write.csv(eapa_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_clim_fp",prot_par,"/eapa_nuts2_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
## write impacts for warming levels
# write.csv(ead_nuts2_WL,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_WL_clim/ead_nuts2_WL_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
#  write.csv(eapa_nuts2_WL,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_WL_clim/eapa_nuts2_WL_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)


#####################
print('Risk analysis for future conditions COMPLETED')
print('script ended')

