## second step of risk analysis, with the following steps:
## - calculate socio-economic impacts including socio-economic projections 
## - interpolate average annual impacts at NUTS2 level
## - aggregate impacts for representative warming levels (WLs) at NUTS2 level

## NOTE: THIS NEW VERSION OF THE SCRIPT NEEDS TO BE TESTED STILL!!!


#### SET RUN MODE
run<-"cluster" ## choose whether to run on (local) or (cluster)
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
flag_prj<-"REF" ## how to consider socio-economic scenarios: "REF" use two reference JRC periods (2050-2010)- "JRC" "SSP": variable JRC or SSP projections
## CHECK maybe not needed anymore
flag_ref <-2100 # reference JRC period to use if jrj="ref" (2050-2010)
prot_par<-"050" # level of protection used for sensitivity analysis (050=50% 150=150%)

## load libraries
library(sp,lib.loc="/home/dottofr/Rlib")     	# needed to load rdgdal library
Sys.sleep(1)
library(raster,lib.loc="/home/dottofr/Rlib")
Sys.sleep(1)
library(rgdal,lib.loc="/home/dottofr/Rlib") 	# use to read .map raster format
#### 

if(run=='cluster'){
  args <- commandArgs(TRUE)
  scriptDir <- as.character(args[1])     # directory of config file
  mainDir <- as.character(args[2])     # directory of config file  
} else if(run=='local'){
  scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}
workDir <-paste0(mainDir,"/PESETA_IV_data")


#############################
print('---')
print('PERFORM RISK ANALYSIS FOR PRESENT AND FUTURE SCENARIOS')
print('---')
#############################

print('input data')

workDir <-paste0(scriptDir,"/damageMaps_EFAS")

##########
## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)
## PERIOD: pf: present rescaled to future (p->f) fp: future actualized to present (f->p)

# num_ep <-dim(dis_f)[1] # number of EFAS network cells 
num_ep <- 80553

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
RY_classes_prj<- c(2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
nRP<-length(RP_classes)
nRY<-length(RY_classes)

nRYT<-max(RY_classes_prj)-min(RY_classes_prj)+1
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
} 
RYT_classes<-c((min(RY_classes)-1):max(RY_classes))

#########
## READ INPUT TABLES

## read table of model runs and warming levels
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]

## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))
## read table of XY coordinates for EFAS points
table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))
## NOTE: Alternative calculation of NUTS2 grid
table_EP_nuts_XY<-cbind(table_EP_nuts,table_EP_XY)

#########
## FORMAT AND INITIALIZE  DATA

## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

#############################
print('---')
print('PERFORM RISK ANALYSIS FOR SOCIO-ECONOMIC PROJECTIONS')
print('---')

#####################
print("- calculate impacts from future socio-economic projections - step 7F")
## elaborate all models together
## NOTE: this version only works for JRC data, it has to be adapted for SSP 

## prepare 5km grid of nuts region to aggregate results
# nuts_points<-cbind(table_EP_nuts_XY[,"X"],table_EP_nuts_XY[,"Y"],table_EP_nuts_XY[,"NUTS2_CODE"])
# colnames(nuts_points) <-c("X","Y","NUTS2_CODE")
# nuts_grid<- rasterFromXYZ(nuts_points, res=c(5000,5000), crs=NA, digits=5)  
# writeGDAL(as(nuts_grid,'SpatialGridDataFrame'), fname=paste0(workDir,"/maps/nuts_grid.tif"),band, drivername='GTiff',type="Int32",mvFlag=-9999,options=c("INTERLEAVE=PIXEL","COMPRESS=LZW"))                                                                                                           

if(flag_prj=="SSP"){  
  table_gdp_ssp1<-read.csv(paste(workDir,"/tables/projections/table_nuts_gdp_ssp1.csv", sep=""))
  table_gdp_ssp3<-read.csv(paste(workDir,"/tables/projections/table_nuts_gdp_ssp3.csv", sep=""))
  table_gdp_ssp5<-read.csv(paste(workDir,"/tables/projections/table_nuts_gdp_ssp5.csv", sep=""))
  gdp_ref<-table_gdp_ssp1[,"X2015"]
}  else {
  table_gdp_jrc<-read.csv(paste(workDir,"/tables/projections/table_nuts_gdp_PIV.csv", sep=""))
  table_pop_jrc<-read.csv(paste(workDir,"/tables/projections/table_nuts_pop_PIII.csv", sep=""))    
  gdp_ref<-table_gdp_jrc[,"X2015"]
}
coeff_GDP <- 0.3558   ## USE SAME COEFFICIENT 0.3558 for all sectors
table_gdp_ref<- array(0, dim=c(nNU,nRY-6) ,dimnames = list(list_nuts2,RY_classes_prj))
table_pop_ref<- array(0, dim=c(nNU,nRY-6) ,dimnames = list(list_nuts2,RY_classes_prj))
table_gdp_ref<- table_gdp_ref+table_gdp_jrc[,"X2015"]
table_gdp_ratio <- (table_gdp_jrc[,3:21]^coef_GDP)/(table_gdp_ref^coef_GDP) 
## NOTE: for PESETA, we use population after 2050, years up to 2050 are covered by spatial maps
table_pop_ref<- table_pop_ref+table_pop_jrc[,"X2050"]
table_pop_ratio  <- table_pop_jrc[,3:21]/table_pop_ref
table_pop_ratio[,1:8]  <- 1

# for(iM in 1:lMod){
iM=2
  Pr_mod<-WL_table[iM,"Model"]
  Pr_rcp<-WL_table[iM,"RCP"]
  Pr_wu<-WL_table[iM,"WU"]
  print(paste("elaborate model ",Pr_mod))  

## NOTE: use only transient climate because we anyway focus on time windows of each WL
  ead_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_clim_fp",prot_par,"/ead_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1) 
  eapa_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_clim_fp",prot_par,"/eapa_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1) 
  
 ## initialize arrays with values corrected with projection  
  ead_f_prj <- ead_f
  eapa_f_prj <- eapa_f 
  ead_f_prj_bis <- ead_f
  eapa_f_prj_bis <- eapa_f 

## NOT IN USE ANYMORE  
#  eapa_prj_nuts2<-array(0, dim=c(nNU,nRY-6),dimnames = list(table_countries[,2],c(RY_classes[7:nRY])))
#  ead_prj_nuts2<-array(0, dim=c(nNU,nRY-6),dimnames = list(table_countries[,2],c(RY_classes[7:nRY])))
## additional arrays (used for REF 2100 and SSP-RCP8.5)
#  eapa_prj_nuts2_bis <-eapa_prj_nuts2
#  ead_prj_nuts2_bis <-ead_prj_nuts2

## create and write raster file of EP results
    for(i in 7:nRY){  # start from 2010 and then every 5 years (years with projection)
    
#    write.csv(nuts_points, file = paste0(workDir,"/tables/test2_nuts.csv"))  
    ead_fs<-cbind(table_EP_XY[,2:3],ead_f[,i])
    ead_grid<- rasterFromXYZ(ead_fs, res=c(5000,5000), crs=NA, digits=5)
    #  writeGDAL(as(ead_grid,'SpatialGridDataFrame'), fname=paste0(workDir,"/maps/EAD/EAD_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,"_",RY_classes[i],".tif"),band, drivername='GTiff',type="Float32",mvFlag=-9999,options=c("INTERLEAVE=PIXEL","COMPRESS=LZW"))                                                                                                           
    eapa_fs<-cbind(table_EP_XY[,2:3],eapa_f[,i])
    eapa_grid<- rasterFromXYZ(eapa_fs, res=c(5000,5000), crs=NA, digits=5)
    ## CHECK maybe use only 2050 grids!!
## read population and urban projections (give as ratio to present values) and scale future impacts  
    if(flag_prj=="REF"){
      popMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Population/Qi_5km_",flag_ref,"_change.tif"),silent=TRUE)) 
      urbMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Urban/HA_Urban_5km_",flag_ref,"_change.tif"),silent=TRUE))
      popMap<-(popMap+100)/100 # NOTE: LUISA maps are expressed as percentage increase (Vf-Vp)/Vp,            
      urbMap<-(urbMap+100)/100 # this is needed to convert to ratio    
      urbMap[is.na(urbMap)]<-0
      popMap[is.na(popMap)]<-0      
      popMap[popMap==0]<-1 # to avoid eliminating cells with values
      urbMap[urbMap==0]<-1
    } else if(flag_prj=="JRC"){
      popMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Population/Qi_5km_",RY_classes[i],"_change.tif"),silent=TRUE)) 
      urbMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Urban/HA_Urban_5km_",RY_classes[i],"_change.tif"),silent=TRUE))
      popMap<-(popMap+100)/100            
      urbMap<-(urbMap+100)/100
      urbMap[is.na(urbMap)]<-0
      popMap[is.na(popMap)]<-0      
      popMap[popMap==0]<-1 # to avoid eliminating cells with values
      urbMap[urbMap==0]<-1
    } else if(flag_prj=="SSP"){
      if(Pr_rcp=="rcp45"){
        popMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_SSPs/ssp1_",RY_classes[i],"_total_ETRS89.tif"),silent=TRUE)) 
        urbMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_SSPs/ssp1_",RY_classes[i],"_urban_ETRS89.tif"),silent=TRUE)) 
      }else if(Pr_rcp=="rcp85"){   
        popMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_SSPs/ssp3_",RY_classes[i],"_total_ETRS89.tif"),silent=TRUE)) 
        urbMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_SSPs/ssp3_",RY_classes[i],"_urban_ETRS89.tif"),silent=TRUE)) 
      }  
    }
    
    popClip<-popMap*eapa_grid
    urbClip<-urbMap*ead_grid 

    ead_prj <- rasterToPoints(damClip)      
    eapa_prj <- rasterToPoints(popClip)                 
    ead_f_prj[,i] <- ead_prj[,3]
    eapa_f_prj[,i] <- eapa_prj[,3]  

## NOT IN USE ANYMORE, zonal sum possibly not working correctly
#    pop_tmp <- zonal(popClip, nuts_grid, fun='sum', na.rm=TRUE)
#    urb_tmp <- zonal(urbClip, nuts_grid, fun='sum', na.rm=TRUE) 
#    pop_test <- zonal(eapa_grid, nuts_grid, fun='sum', na.rm=TRUE) ## use to test with climate-only EAPA
    
#    for(j in 1:dim(pop_tmp)[1]){
#      ind<-which( dimnames(ead_prj_nuts2)[[1]]==as.character(pop_tmp[j,"zone"]))
#      ead_prj_nuts2[ind,as.character(RY_classes[i])]<-urb_tmp[j,2]      
#      eapa_prj_nuts2[ind,as.character(RY_classes[i])]<-pop_tmp[j,2]
#      eapa_prj_nuts22[ind,as.character(RY_classes[i])]<-pop_test[j,2] ## use to test with climate-only EAPA      
#    }
#    rm(pop_tmp,urb_tmp,popMap,urbMap,popClip,urbClip)

# CHECK- this is not needed for JRC maps because they do not change after 2050    
    if(flag_prj=="SSP" & Pr_rcp=="rcp85"){     
      popMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_SSPs/ssp5_",RY_classes[i],"_total_ETRS89.tif"),silent=TRUE)) 
      urbMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_SSPs/ssp5_",RY_classes[i],"_urban_ETRS89.tif"),silent=TRUE)) 
      popClip<-popMap*eapa_grid
      urbClip<-urbMap*ead_grid     

    ead_prj <- rasterToPoints(damClip)      
    eapa_prj <- rasterToPoints(popClip)                 
    ead_f_prj_bis[,i] <- ead_prj[,3]
    eapa_f_prj_bis[,i] <- eapa_prj[,3]  
    }
 
    rm(ead_fs,eapa_fs,ead_grid,eapa_grid,popMap,damMap,popClip,damClip)    
  }  
 
 
  ## dam tables to be  multiplied with GDP ratios in all cases
  ## pop tables for REF-2100 scenario to be multiplied with country-scale Pop ratios

  gdp_ep <- array(1, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),RY_classes))
  gdp_ep_bis <- array(1, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),RY_classes))
  pop_ep_bis <- array(1, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),RY_classes))

  if(flag_prj=="REF"){  
    for (i in 1:nNU){
   	ind1<-which(table_EP_nuts[,2]==table_gdp_jrc[i,"NUTS_CODE"])
   	ind2<-which(table_EP_nuts[,2]==table_pop_jrc[i,"NUTS_CODE"])
   	for (j in 1:dim(table_gdp_ratio)[2]){
    	 gdp_ep[ind1,(j+6)]<-table_gdp_ratio[i,"2050"]
    	 gdp_ep_bis[ind1,(j+6)]<-table_gdp_ratio[i,"2100"]
     	 pop_ep_bis[ind2,(j+6)]<-table_pop_ratio[i,"2100"]       
   	}
    }
    ead_f_prj <- ead_f_prj*gdp_ep
    ead_f_prj_bis <- ead_f_prj_bis*gdp_ep_bis
    eapa_f_prj_bis <- eapa_f_prj_bis*pop_ep_bis
   
  } else if(flag_prj=="JRC"){
    for (i in 1:nNU){
   	ind1<-which(table_EP_nuts[,2]==table_gdp_jrc[i,"NUTS_CODE"])
   	ind2<-which(table_EP_nuts[,2]==table_pop_jrc[i,"NUTS_CODE"])
   	for (j in 1:dim(table_gdp_ratio)[2]){
     	  gdp_ep[ind1,(j+6)]<-table_gdp_ratio[i,j]
        pop_ep[ind2,(j+6)]<-table_pop_ratio[i,j]       
   	}
    }
    ead_f_prj <- ead_f_prj*gdp_ep
    eapa_f_prj <- eapa_f_prj*pop_ep

  } else if(flag_prj=="SSP"){
    for (i in 1:nNU){
   	ind1<-which(table_EP_nuts[,2]==table_gdp_ssp1[i,"NUTS_CODE"])
   	ind2<-which(table_EP_nuts[,2]==table_pop_ssp1[i,"NUTS_CODE"])
   	for (j in 1:dim(table_gdp_ratio)[2]){
    	 gdp_ep[ind1,(j+6)]<-table_gdp_ratio_ssp1[i,j]
    	 gdp_ep_bis[ind1,(j+6)]<-table_gdp_ratio_ssp3[i,j]
     	 pop_ep_bis[ind2,(j+6)]<-table_pop_ratio_ssp5[i,j]       
   	}
    }
    ead_f_prj <- ead_f_prj*gdp_ep
    ead_f_prj_bis <- ead_f_prj_bis*gdp_ep_bis
    eapa_f_prj_bis <- eapa_f_prj_bis*pop_ep_bis
   }
 
## TEST: AGGREGATE AT NUTS2 LEVEL TO COMPARE WITH 05B
print("aggregate expected annual impacts at NUTS2 scale (step 5F")

ead_f_nuts2 <- array(0, dim = c(nNU,nRY), dimnames=list(list_nuts2,RY_classes))
eapa_f_nuts2 <- array(0, dim = c(nNU,nRY), dimnames=list(list_nuts2,RY_classes))
ead_f_nuts2_bis <- ead_f_nuts2
eapa_f_nuts2_bis <- ead_f_nuts2
   for (ny in 1:nRY){ 
     for (np in 1:nNU) {
   #    print(paste("NUTS2 ",list_nuts2[np],sep=""))
       ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
       ead_f_nuts2[np,ny]<-sum(ead_f_prj[ind_nuts,ny]) 
       eapa_f_nuts2[np,ny]<-sum(eapa_f_prj[ind_nuts,ny]) 
       ead_f_nuts2_bis[np,ny]<-sum(ead_f_prj_bis[ind_nuts,ny]) 
       eapa_f_nuts2_bis[np,ny]<-sum(eapa_f_prj_bis[ind_nuts,ny]) 
     }
   }

#####################
 ## interpolate model values every 5 years starting from 2010
   
  ead_nuts2_ext <- array(0, dim = c(nNU,nRYT),dimnames=list(table_countries[,2],c(min(RY_classes_prj):max(RY_classes_prj)))
  eapa_nuts2_ext <- ead_nuts2_ext
  ead_nuts2_bis_ext <- ead_nuts2_ext
  eapa_nuts2_bis_ext <- ead_nuts2_ext  
  
  ead_nuts2_ext[,1]<-ead_prj_nuts2[,1]
  eapa_nuts2_ext[,1]<-eapa_prj_nuts2[,1]  
  for (ii in 1:(dim(ead_prj_nuts2)[2]-1)){
    ead_nuts2_ext[,ii*5+1]<-ead_prj_nuts2[,ii+1]
    eapa_nuts2_ext[,ii*5+1]<-eapa_prj_nuts2[,ii+1]    
  }
 
  for (jj in 1:(dim(ead_prj_nuts2)[2]-1)){
    for(kk in 0:3){
      ead_nuts2_ext[,jj*5-kk]<-ead_prj_nuts2[,jj]+(ead_prj_nuts2[,jj+1]-ead_prj_nuts2[,jj])*(4-kk)/5     
      eapa_nuts2_ext[,jj*5-kk]<-eapa_prj_nuts2[,jj]+(eapa_prj_nuts2[,jj+1]-eapa_prj_nuts2[,jj])*(4-kk)/5        
    }
  }  
  ## used only when (flag_prj=="REF") or if(flag_prj=="SSP" & Pr_rcp=="rcp85") 
  ead_nuts2_bis_ext[,1]<-ead_prj_nuts2_bis[,1]
  eapa_nuts2_bis_ext[,1]<-eapa_prj_nuts2_bis[,1]  
  for (ii in 1:(dim(ead_prj_nuts2)[2]-1)){
    ead_nuts2_bis_ext[,ii*5+1]<-ead_prj_nuts2_bis[,ii+1]
    eapa_nuts2_bis_ext[,ii*5+1]<-eapa_prj_nuts2_bis[,ii+1]    
  }
  for (jj in 1:(dim(ead_prj_nuts2_bis)[2]-1)){
    for(kk in 0:3){
      ead_nuts2_bis_ext[,jj*5-kk]<-ead_prj_nuts2_bis[,jj]+(ead_prj_nuts2_bis[,jj+1]-ead_prj_nuts2_bis[,jj])*(4-kk)/5     
      eapa_nuts2_bis_ext[,jj*5-kk]<-eapa_prj_nuts2_bis[,jj]+(eapa_prj_nuts2_bis[,jj+1]-eapa_prj_nuts2_bis[,jj])*(4-kk)/5        
    }
  }
  
  #####################
  print("print and save results")
  if(flag_prj=="REF" ){ 
    ## write impacts at NUTS2 level for all years
## CHECK- save both 2050 and 2100 results! 
#    write.csv(ead_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_2050/ead_nuts2_2050_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
#    write.csv(eapa_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_2050/eapa_nuts2_2050_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
#    write.csv(ead_nuts2_bis_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_2100/ead_nuts2_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
#    write.csv(eapa_nuts2_bis_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_2100/eapa_nuts2_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)    
    write.csv(ead_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_2100_fp",prot_par,"/ead_nuts2_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,"_no_prj.csv"),row.names=TRUE)
    write.csv(eapa_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_2100_fp",prot_par,"/eapa_nuts2_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,"_no_prj.csv"),row.names=TRUE)
    ## TEST
  } else if(flag_prj=="JRC" ){ 
    ## write impacts at NUTS2 level for all years
      write.csv(ead_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_JRC/ead_nuts2_JRC_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
      write.csv(eapa_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_JRC/eapa_nuts2_JRC_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
  } else if(flag_prj=="SSP"){     
    if(Pr_rcp=="rcp45"){     
    ## write impacts at NUTS2 level for all years
      write.csv(ead_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_SSP1/ead_nuts2_SSP1_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
      write.csv(eapa_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_SSP1/eapa_nuts2_SSP1_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
    }else if(Pr_rcp=="rcp85"){
      write.csv(ead_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_SSP3/ead_nuts2_SSP3_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
      write.csv(eapa_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_SSP3/eapa_nuts2_SSP3_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
      write.csv(ead_nuts2_bis_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_SSP5/ead_nuts2_SSP5_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)
      write.csv(eapa_nuts2_bis_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_SSP5/eapa_nuts2_SSP5_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=TRUE)    
    }
  }
## TEST
# }
#####################
print('Risk analysis for future conditions COMPLETED')
print('script ended')

