
## calculate ensemble averages of discharge - to be used for adaptation scenarios
## option 1: calculate separate averages for rcp 8.5 and 4.5 (now deactivated)
## option 2: calculate separate averages for WLs 1.5C , 2C , 3C (replaced with values of year hitting each WL, representing 30y average)

#### SET RUN MODE
run<-"cluster" ## choose whether to run on (local) or (cluster)
test<-"no"   ##  run test data (yes) or real data (no)
ens<-"mean" ## choose whether to calculate ensemble values with median or mean
compute <-"yes" ## choose whether to calculate model WL values (yes) or to read them (no)
#### 

if(run=='cluster'){
  args <- commandArgs(TRUE)
  scriptDir <- as.character(args[1])     # directory of config file
} else if(run=='local'){
  scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}
workDir <-paste0(scriptDir,"/PESETA_IV_data")

#######################################

## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]
run1=run

num_ep <-80553 # number of EFAS network cells 

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2, 10, 15, 20, 30, 50, 70, 100,150, 200, 350, 500, 700, 1000, 1500, 2000)
# RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
nRP<-length(RP_classes)
nRY<-length(RY_classes)

nRYT<-max(RY_classes)-min(RY_classes)+1
startY=min(RY_classes)
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
  startY=startY-1
} 

## initialize ensemble avg discharges
if (test=='yes'){
#   dis_45<-array(0, dim = c(100, nRY, nRP), dimnames = list(c(1:100),RY_classes,RP_classes))
#   dis_85<-array(0, dim = c(100, nRY, nRP), dimnames = list(c(1:100),RY_classes,RP_classes))
  dis_ens<-array(0, dim = c(100, 4, nRP), dimnames = list(c(1:100),c("base","1.5C","2C","3"),RP_classes))  
}else{
#   dis_45<-array(0, dim = c(num_ep, nRY, nRP), dimnames = list(c(1:num_ep),RY_classes,RP_classes))
#   dis_85<-array(0, dim = c(num_ep, nRY, nRP), dimnames = list(c(1:num_ep),RY_classes,RP_classes))
  dis_ens<-array(0, dim = c(num_ep, 4, nRP), dimnames = list(c(1:num_ep),c("base","1.5C","2C","3C"),RP_classes))  
}
## test - save Q100 for all model runs  
# dis100 <- array(0, dim = c(num_ep, nRY, lMod), dimnames = list(c(1:num_ep),RY_classes,WL_table[,2]))

#######################################
if(compute=="yes"){
  print(" calculate model WL values")
  for(iM in 1:lMod){
    #for(iM in 1:3){
    Pr_mod<-WL_table[iM,2]
    Pr_rcp<-WL_table[iM,3]
    Pr_wu<-WL_table[iM,4]
    ind_base<-WL_table[iM,"Base"]-min(RY_classes)+1
    ind_WL15<-WL_table[iM,"X1.5C"]-min(RY_classes)+1
    ind_WL20<-WL_table[iM,"X2C"]-min(RY_classes)+1
    ind_WL30<-WL_table[iM,"X3C"]-min(RY_classes)+1  
    print(paste0("elaborate model ",WL_table[iM,"ModelRun"]))
    load(file = paste0(workDir,"/Rdata/",WL_table[iM,"ModelRun"],".RData"))
    if(run1=='local'){
      scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/Damage")   
      workDir<-(paste(scriptDir,"/damageMaps_EFAS",sep=""))  
    }
    
    ## interpolate RP 200y using 150y and 250y
    dis_150_250 <- dis_f[,,c(14,15,15)]  
    dis_150_250[,,3] <- (dis_150_250[,,1]+dis_150_250[,,2])/2    
    ## replace 250y wit interpolated 200y
    dis_f[,,15] <- dis_150_250[,,3]
    rm(dis_150_250)
    
    ## keep only used RPs (see RP_classes)
    dis_f<-dis_f[,,c(2,7:21)]  
    #dis_f <- dis_f[,-c(1:4),] # delete years 1981-1995
    dis_f[is.na(dis_f)] <- 0
    
    ## option 1: calculate ensemble averages for each RCP 
    #   if(Pr_rcp=="rcp45"){
    #     dis_45 <- dis_45+dis_f
    #   }else if(Pr_rcp=="rcp85"){
    #     dis_85 <- dis_85+dis_f    
    #   }
    
    ## option 2: calculate ensemble averages for each WLs
    
    ## create a specific variable named after the model 
    ## NOTE: despite the name dis_ens_, dis_m is only based on single models
    dis_m <- paste("dis_ens_", Pr_mod,"_",Pr_rcp, sep = "")
    assign(dis_m, array(0, dim = c(num_ep, 4, nRP), dimnames = list(c(1:num_ep),c("base","1.5C","2C","3C"),RP_classes))) 
    dis_m2 <- array(0, dim = c(num_ep, 4, nRP), dimnames = list(c(1:num_ep),c("base","1.5C","2C","3C"),RP_classes)) 
    ## interpolate discharge values every 5 years
    dis_ext<-array(0, dim = c(num_ep, nRYT, nRP), dimnames = list(c(1:num_ep),c(1980:2100),RP_classes)) 
    dis_ext[,1,]<-dis_f[,1,]
    for (ii in 1:(nRY-1)){
      dis_ext[,ii*5+1,]<-dis_f[,ii+1,]
    }
    for (jj in 1:(nRY-1)){
      dis_ext[,jj*5-3,]<-dis_f[,jj,]+(dis_f[,jj+1,]-dis_f[,jj,])/5
      dis_ext[,jj*5-2,]<-dis_f[,jj,]+(dis_f[,jj+1,]-dis_f[,jj,])*2/5
      dis_ext[,jj*5-1,]<-dis_f[,jj,]+(dis_f[,jj+1,]-dis_f[,jj,])*3/5
      dis_ext[,jj*5-0,]<-dis_f[,jj,]+(dis_f[,jj+1,]-dis_f[,jj,])*4/5  
    }
    for (nr in 1:nRP){
      for (np in 1:num_ep) {

## UPDATE: take values for the year of reaching WL        
        dis_m2[np,"base",nr] <- dis_ext[np,as.character(WL_table[iM,"Base"]),nr]
        dis_m2[np,"1.5C",nr] <- dis_ext[np,as.character(WL_table[iM,"X1.5C"]),nr]
        dis_m2[np,"2C",nr] <- dis_ext[np,as.character(WL_table[iM,"X2C"]),nr]
        if(WL_table[iM,"X3C"]==-1){
          dis_m2[np,"3C",nr]<-0
        }else{
          dis_m2[np,"3C",nr]<-dis_ext[np,as.character(WL_table[iM,"X3C"]),nr]
        }
## commented: averages are not correct as Q values were already smoothed with a 30y moving average      
#        dis_m2[np,"base",nr] <- mean(dis_ext[np,max(1,ind_base-14):min(ind_base+15,nRYT),nr])
#        dis_m2[np,"1.5C",nr] <- mean(dis_ext[np,max(1,ind_WL15-14):min(ind_WL15+15,nRYT),nr])
#        dis_m2[np,"2C",nr] <- mean(dis_ext[np,max(1,ind_WL20-14):min(ind_WL20+15,nRYT),nr])
#        if(WL_table[iM,"X3C"]==-1){
#          dis_m2[np,"3C",nr]<-0
#        }else{
#          dis_m2[np,"3C",nr]<-mean(dis_ext[np,max(1,ind_WL30-14):min(ind_WL30+15,nRYT),nr])
#        }
      }
    }
    assign(dis_m,dis_m2)
    dis_ens<-dis_ens+dis_m2 
## test - save Q100 for all model runs  
#  dis100[,,iM] <- dis_f[,,5]
  }
######
  ## option 1: calculate ensemble averages for each RCP 
  # dis_45 <- dis_45/(lMod/2)
  # dis_85 <- dis_85/(lMod/2)
  # obj_list<-c("dis_45","dis_85")
  if(ens=="mean"){
  ## option 2: calculate ensemble averages for each WLs
    dis_ens[,"base",]<- dis_ens[,"base",]/(lMod)
    dis_ens[,"1.5C",]<- dis_ens[,"1.5C",]/(lMod)
    dis_ens[,"2C",]<- dis_ens[,"2C",]/(lMod)
    dis_ens[,"3C",]<- dis_ens[,"3C",]/(lMod/2)
    obj_list <-character(lMod+1)
    obj_list[1]<-"dis_ens"
    #for(iM in 1:3){
    for(iM in 1:lMod){  
      obj_list[iM+1]<-paste("dis_ens_", WL_table[iM,2],"_",WL_table[iM,3], sep = "")
    }
    
    ## activate to save ensemble discharges in R workspace
    save(list = obj_list, file = paste0(workDir,"/Rdata/ens_discharges_mean.RData"))  
  }
  
} else if (compute=="no"){
  print("read ensemble averages of discharge")
  load(file = paste0(workDir,"/Rdata/ens_discharges_mean.RData"))
  if(ens=="median"){
    dis_all= array(0, dim = c(num_ep, 4, nRP,lMod), dimnames = list(c(1:num_ep),c("base","1.5C","2C","3C"),RP_classes,WL_table[,1]))
    for(iM in 1:lMod){
      print(paste0("elaborate model ",WL_table[iM,"ModelRun"]))
      Pr_mod<-WL_table[iM,2]
      Pr_rcp<-WL_table[iM,3]
      dis_m1 <- paste("dis_ens_", Pr_mod,"_",Pr_rcp, sep = "")
      dis_mod <- array(0, dim = dim(dis_ens), dimnames = dimnames(dis_ens)) 
      dis_mod <- get(dis_m1)
      dis_all[,,,iM]<-dis_mod  
      if(Pr_rcp=="rcp45"){
        dis_all[,"3C",,iM]<-NA
      }
    }
    dis_ens_median<-apply(dis_all,c(1,2,3),median,na.rm = TRUE)
    rm(dis_all,iM,run,run1,dis_mod,dis_m1)
## evaluate differences between median and mean
#    diff=(dis_ens_median-dis_ens)/dis_ens_median
    
    obj_list <-character(lMod+1)
    obj_list[1]<-"dis_ens_median"
    #for(iM in 1:3){
    for(iM in 1:lMod){  
      obj_list[iM+1]<-paste("dis_ens_", WL_table[iM,2],"_",WL_table[iM,3], sep = "")
    }
    
    ## activate to save ensemble discharges in R workspace
    save(list = obj_list, file = paste0(workDir,"/Rdata/ens_discharges_median.RData"))  
    
  }
}
##### 

# load(file = paste0(workDir,"/Rdata/ens_discharges_median.RData"))

print('reading of discharge data done')

