

## RISK ANALYSIS INCLUDING ADAPTATION SCENARIOS

## NOTE: all parte related to calculation of exposed built-up area are commented but 
##      they can be reactivated if necessary

#### SET RUN MODE
run<-"cluster" ## choose whether to run on (local) or (cluster)
MR_data<-"Rdata" ## read (nectdf) file or using R dataset (Rdata)
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
                ## stabilized "sta" (data after year of hitting WLs is replaced with constant)
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean

## load libraries
library(sp,lib.loc="/home/dottofr/Rlib")     	# needed to load rdgdal library
Sys.sleep(1)
library(raster,lib.loc="/home/dottofr/Rlib")
Sys.sleep(1)
library(rgdal,lib.loc="/home/dottofr/Rlib") 	# use to read .map raster format
#### 
#### 

if(run=='cluster'){
  args <- commandArgs(TRUE)
  modelRun <- as.character(args[1])     # model run to analyse
  mainDir <- as.character(args[2])     # directory of config file   
} else if(run=='local'){
#  modelRun <- "projection_dis_rcp45_CLMcom-CCLM4-8-17_BC_ICHEC-EC-EARTH_wuConst_statistics_ptId.nc"
  modelRun <- "projection_dis_rcp85_CLMcom-CCLM4-8-17_BC_CNRM-CERFACS-CNRM-CM5_wuConst_statistics_ptId.nc" 
  mainDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts") 
}
#### SET PARAMETERS USED FOR THE SENSISTIVITY ANALYSIS (for sensitivity analysis)
flag_WL <- as.character(args[3]) # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
prot_par<- as.character(args[4]) # level of flood protection - 050: 50% of estimated average protection standards
# flag_WL <- "3C" 
# prot_par<- "050"

workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))
workDir2<-paste0(mainDir,"/PESETA_IV_data") # backup variable to be used if Rdata has different workDir 

#######################################
print('-----')
print("handle netcdf maps")
## NOTE: many commands are not necessary but are useful as examples for handling netcdf maps
#######################################

## READ and elaborate DISCHARGE DATA from NETCDF or RDATA files
if(MR_data=='netcdf'){
  
  library(ncdf4) ###reapaing and saving netcdf files 
  
  print ('Load map of return levels')
  rl_map <- nc_open(paste0(workDir,"/maps/modelRuns/",modelRun))
  print ('Load map of EFAS point ids')
  id_map <- nc_open(paste0(workDir,"/maps/idChan_5km.nc"))

  ## get relevant variables
  # idEP <- ncvar_get(id_map, "idChan_5km.map")
  v3 <- rl_map$var[['rl']]
  varsize <- v3$varsize
  ndims <- v3$ndims
  nt <- varsize[ndims] # Remember timelike dim is always the LAST dimension!
  print(paste("var size is ",varsize,sep=""))
  print(paste("ndims is ",ndims,sep=""))

  RP <-rl_map$dim['return_period']
  lRP <-RP$return_period$vals # get total number of RP
  print(paste(length(lRP)," return periods",sep=""))
  RY <-rl_map$dim['year']
  lRY <-RY$year$vals # get total number reference years
  print(paste(length(lRY)," reference years",sep=""))
  ID <-rl_map$dim['ptId']
  lID <-ID$year$vals # get total number reference years
  print(paste(length(lID)," points ",sep=""))

## CHECK: maybe the loop is not needed and data can directly be extracted!
  for(ii in 1:length(lRY)) {    # length(lRY)
    for(jj in 1:length(lRP)) {   # length(lRP)
#      print(paste("extract map for year ",lRY[ii]," and RP ",lRP[jj],sep=""))   
      data<- ncvar_get(rl_map, 'rl', count=c(ii,jj, -1))
    }
  }
  dis_f<-aperm(data,c(3,1,2))
  nc_close(rl_map)
  nc_close(id_map)
  rm(data,rl_map,id_map,v3,varsize,ndims,ID,RP,RY)
  
## activate to save data in R workspace
# save(list = ls(all = TRUE), file = paste0(workDir,"/Rdata/",modelRun,".RData"))
gc ()  

}  else if (MR_data=='Rdata'){
  
  print("cHECJK - use Rdata")
  print(paste0("CHECK Work dir ", workDir))
  run2<-run
  load(file = paste0(workDir,"/Rdata/",modelRun,".RData"))
  MR_data=='Rdata'
  if(run2=='local'){  
    run='local'
  }
}
workDir <-workDir2

## interpolate RP 200y using 150y and 250y
dis_150_250 <- dis_f[,,c(14,15,15)]  
dis_150_250[,,3] <- (dis_150_250[,,1]+dis_150_250[,,2])/2    
## replace 250y wit interpolated 200y
dis_f[,,15] <- dis_150_250[,,3]
rm(dis_150_250)

## keep only used RPs (2,10,20,50,100,200,500,1000)
dis_f<-dis_f[,,c(2,7,9,11,13,15,17,19)]

## delete years not used
dis_p <- dis_f[,8,] # year 2015 as reference to match discharge and impacts
#dis_f <- dis_f[,-c(1:4),] # delete years 1981-1995

dis_f[is.na(dis_f)] <- 0
dis_p[is.na(dis_p)] <- 0

## read ensemble averages of discharge
load(file = paste0(workDir,"/Rdata/ens_discharges_",ensemble,".RData"))
if(ensemble=="median"){
   dis_ens <- dis_ens_median
} 
print('reading of discharge data done')

#############################
print('---')
print('read input data')

## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)
## PERIOD: pf: present rescaled to future (p->f) fp: future actualized to present (f->p)

num_ep <-dim(dis_f)[1] # number of EFAS network cells 

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
## RP_freq = c(2y(0.5-0.1),10y(0.1-0.05),20y(0.05-0.02),50y(0.02-0.01),100y(0.01-0.005),200y(0.005-0.002),500y(0.002-0.001)
## integral is calculated as sum of trapezoids (x=prob-y=dam) RP_freq are the "heights" of each trapezoid
RP_freq<- c(0.4,0.05,0.03,0.01,0.005,0.003,0.001,0.001)
nRP<-length(RP_classes)
nRY<-length(RY_classes)

nRYT<-max(RY_classes)-min(RY_classes)+1
startY=min(RY_classes)
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
  startY=startY-1
} 
#########
## READ INPUT TABLES

## read tables of present impacts - POPULATION, DAMAGE, BUILT-UP
dam_p_t <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_Cost_EP_v4_all.csv", sep=""),row.name=1)
pop_p_t <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_PopExp_EP_luisa.csv", sep=""),row.name=1)
# bld_p_t <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_Builtup_EP_ghsl.csv", sep=""))

## read present level of protection in each point (RP in years)
prot_RP_p_table <- read.csv(paste(workDir,"/tables/protection_levels/table_EP_protLev_2018_fp",prot_par,".csv", sep=""))

## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))

## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))

## read table of XY coordinates for EFAS points
table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))

#########
## FORMAT AND INITIALIZE  DATA
print('initialize data')

dimnames(dis_f)<-list(c(1:num_ep),lRY,RP_classes)

## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

## format tables of present impacts 
pop_p<-array(0, dim= c(nrow(pop_p_t),nRP))
dam_p<-array(0, dim= c(nrow(dam_p_t),nRP))
# bld_p<-array(0, dim= c(nrow(bld_p_t),nRP))
for (ii in 1:dim(dam_p_t)[2]) {
  pop_p[,ii+1]<-pop_p_t[,ii]
  dam_p[,ii+1]<-dam_p_t[,ii]
#  bld_p[,ii+1]<-bld_p_t[,ii]   
}
pop_p[,8]<-pop_p[,7]
dam_p[,8]<-dam_p[,7]
# bld_p[,8]<-bld_p[,7]
rm(pop_p_t,dam_p_t,bld_p_t)

## read model run set up and corresponding warming levels
ind_MR<- which(WL_table[,"ModelRun"]==modelRun)
Pr_mod<-WL_table[ind_MR,"Model"]
Pr_rcp<-WL_table[ind_MR,"RCP"]
Pr_wu<-WL_table[ind_MR,"WU"]
ind_base<-WL_table[ind_MR,"Base"]-min(RY_classes)+1
ind_WL15<-WL_table[ind_MR,"X1.5C"]-min(RY_classes)+1
ind_WL20<-WL_table[ind_MR,"X2C"]-min(RY_classes)+1
ind_WL30<-WL_table[ind_MR,"X3C"]-min(RY_classes)+1

## format protection level tables and assign synthetic values (if requested)
prot_RP_p <- prot_RP_p_table[,2] 
rm(prot_RP_p_table)
prot_QP <- prot_RP_p*0

## initialize present RP of future protection standards for refererence design year (eg. 2050)
# prot_RP_des <- array(0, dim = c(num_ep, 2), dimnames = list(c(1:num_ep),c("present","design_2050")))
prot_RP_des <- array(0, dim = c(num_ep, dim(dis_ens)[[3]]+1), dimnames = list(c(1:num_ep),c("present",dimnames(dis_ens)[[3]])))
## initialize future RP protection - variable according to design standard - replaces prot_RP_pf)
prot_RP_f <- array(0, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),lRY))
prot_RP_pf<-prot_RP_f
## initialize present impacts of future discharges
dam_fp <- array(0, dim = c(num_ep, nRY, nRP))
pop_fp <- array(0, dim = c(num_ep, nRY, nRP))
# bld_fp <- array(0, dim = c(num_ep, nRY, nRP))


## initialize present and future EAD,EAPA (output variables)
ead_f <- array(0, dim = c(num_ep, nRY))
eapa_f <- array(0, dim = c(num_ep, nRY))
colnames(ead_f) <- RY_classes
colnames(eapa_f) <- RY_classes

# eab_f_na <- array(0, dim = c(num_ep, nRY))
ead_f_na <- array(0, dim = c(num_ep, nRY))
eapa_f_na <- array(0, dim = c(num_ep, nRY))
#colnames(eab_f_na) <- RY_classes
colnames(ead_f_na) <- RY_classes
colnames(eapa_f_na) <- RY_classes

ead_f_nuts2 <- array(0, dim = c(nNU,nRY))
eapa_f_nuts2 <- array(0, dim = c(nNU,nRY))
colnames(ead_f_nuts2) <- RY_classes
colnames(eapa_f_nuts2) <- RY_classes

print('data initialization done')

#############################
print('---')
print('PERFORM RISK ANALYSIS INCLUDING ADAPTATION SCENARIOS')
print('---')
#############################

## OK: DOCUMENT IN NOTES

## in case of stabilized climate scenarios, replace discharge values with WL-averaged value 
if(flag_MS=="sta"){
  print(paste0("use stabilized climate for a ",flag_WL," warming scenario"))
  dis_m1 <- paste("dis_ens_", Pr_mod,"_",Pr_rcp, sep = "")
  dis_mod <- array(0, dim = dim(dis_ens), dimnames = dimnames(dis_ens)) 
  dis_mod <- get(dis_m1)
  dis_mod <-dis_mod[,,c(1,2,4,6,8,10,12,14)]  # keep only uses RPs

  if(flag_WL=="1.5C"){ 
    ind<- min(which(RY_classes> WL_table[ind_MR,"X1.5C"])) 
  } else if(flag_WL=="2C"){
    ind<- min(which(RY_classes> WL_table[ind_MR,"X2C"]))
  } else if(flag_WL=="3C" & Pr_rcp=="rcp85"){
    ind<- min(which(RY_classes> WL_table[ind_MR,"X3C"]))  
  }
  for(ii in ind:nRY){
    dis_f[,ii,]<-dis_mod[,flag_WL,]
  }
} else{
  print("use transient climate, based on complete model run")
}
# CHECK results
# write.csv(dis_ens[,flag_WL,],file = paste0(workDir,"/tables/EAI_Europe/test_06b/dis_ens_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)
# write.csv(dis_f[,8,],file = paste0(workDir,"/tables/EAI_Europe/test_06b/dis_f_RP2015_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)
# write.csv(dis_f[,,5],file = paste0(workDir,"/tables/EAI_Europe/test_06b/dis_f_YR100_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)

#############################
print("define future protection standards according to a range of adaptation scenarios")

prot_QD <- array(0, dim= c(num_ep,nRY))
colnames(prot_QD)<- lRY

## initialize possible return periods for future protection standards
## uniform and variable RP (urban and rural/natural)
## NOTE: variable RP to be used for adaptation based on storage areas
# RP_unif<-c("2", "10", "15", "20", "30", "50", "70", "100", "150", "200", "350", "500", "700", "1000", "1500", "2000")
   RP_var<-c("2", "10", "15", "20", "30", "50", "50", "50", "100", "100", "100", "100", "150", "150", "150", "200")

list_RP_unif<-dimnames(dis_ens)[[3]]
list_RP_var<-cbind(dimnames(dis_ens)[[3]],RP_var)
colnames(list_RP_var)<-c("urban","rural")

#############################
print("extract present protection values (RP) in terms of Q (step 1-NA)")

## general formula for extrapolating linearly
# Q[RP2>prot_RP_p>RP1] <- Qbase[RP1]+ (Qbase[RP2]-Qbase[RP1])*(prot_RP_p[RP2>prot_RP_p>RP1] - RP1)/(RP2-RP1) 

mask_P<- which(prot_RP_p[]>2 & prot_RP_p[]<=10)
prot_QP[mask_P] <-dis_p[mask_P,1] + (dis_p[mask_P,2]-dis_p[mask_P,1])*(prot_RP_p[prot_RP_p[]>2 & prot_RP_p[]<=10] - 2)/(10-2)
mask_P<- which(prot_RP_p[]>10 & prot_RP_p[]<=20)
prot_QP[mask_P] <-dis_p[mask_P,2] + (dis_p[mask_P,3]-dis_p[mask_P,2])*(prot_RP_p[prot_RP_p[]>10 & prot_RP_p[]<=20] - 10)/(20-10)
mask_P<- which(prot_RP_p[]>20 & prot_RP_p[]<=50)
prot_QP[mask_P] <-dis_p[mask_P,3] + (dis_p[mask_P,4]-dis_p[mask_P,3])*(prot_RP_p[prot_RP_p[]>20 & prot_RP_p[]<=50] - 20)/(50-20)
mask_P<- which(prot_RP_p[]>50 & prot_RP_p[]<=100)
prot_QP[mask_P] <-dis_p[mask_P,4] + (dis_p[mask_P,5]-dis_p[mask_P,4])*(prot_RP_p[prot_RP_p[]>50 & prot_RP_p[]<=100] - 50)/(100-50)
mask_P<- which(prot_RP_p[]>100 & prot_RP_p[]<=200)
prot_QP[mask_P] <-dis_p[mask_P,5] + (dis_p[mask_P,6]-dis_p[mask_P,5])*(prot_RP_p[prot_RP_p[]>100 & prot_RP_p[]<=200] - 100)/(200-100)
mask_P<- which(prot_RP_p[]>200 & prot_RP_p[]<=500)
prot_QP[mask_P] <-dis_p[mask_P,6] + (dis_p[mask_P,7]-dis_p[mask_P,6])*(prot_RP_p[prot_RP_p[]>200 & prot_RP_p[]<=500] - 200)/(500-200)
mask_P<- which(prot_RP_p[]>500 & prot_RP_p[]<=1000)
prot_QP[mask_P] <-dis_p[mask_P,7] + (dis_p[mask_P,8]-dis_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>500 & prot_RP_p[]<=1000] - 500)/(1000-500)
mask_P<- which(prot_RP_p[]>1000)
prot_QP[mask_P] <-dis_p[mask_P,8] + (dis_p[mask_P,8]-dis_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>1000] - 1000)/(1000-500)
rm(mask_P)

## CHECK RESULTS - CORRECT
# write.csv(prot_QP,file = paste0(workDir,"/tables/EAI_Europe/test_06b/prot_QP_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)

print("step 1-NA completed")
print("---")

#############################
print("calculate future RP of present protection standards (step 2-NA)")

for (np in 1:num_ep) {
    #  print(paste("EP ",np,sep=""))  
  for (ny in 1:nRY) {
      #    ny<-1
      if (dis_f[np,ny,1]==0){
        prot_RP_pf[np,ny]<-100000
      }else{
        if (prot_QP[np]>=dis_f[np,ny,nRP]) {  # RP given using linear intepolation from 1000 and 500y values
          #      prot_RP_pf[np,ny] <-1000  
          prot_RP_pf[np,ny]<-RP_classes[nRP-1]+ (RP_classes[nRP]-RP_classes[nRP-1])*(prot_QP[np]-dis_f[np,ny,nRP-1])/(dis_f[np,ny,nRP]-dis_f[np,ny,nRP-1])      
        } else if (prot_QP[np]<=dis_f[np,ny,1]) {
          prot_RP_pf[np,ny] <-2  
        } else {  
          ##CHECK THIS ONE!
          irp <-min(which(dis_f[np,ny,]>prot_QP[np]))
          prot_RP_pf[np,ny]<-RP_classes[irp-1]+ (RP_classes[irp]-RP_classes[irp-1])*(prot_QP[np]-dis_f[np,ny,irp-1])/(dis_f[np,ny,irp]-dis_f[np,ny,irp-1])
        }
      }
  }
}
prot_RP_pf[prot_RP_pf<2] <-2  # check to avoid negative protection standards

## CHECK RESULTS - CORRECT
# write.csv(prot_RP_pf,file = paste0(workDir,"/tables/EAI_Europe/test_06b/prot_RP_pf_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)

print("step 2-NA completed")
print("---")

########################
print("calculate present RP and associated impacts of future discharges  (step 3-NA)")
## actualize damage for reference future RPs to present (Q return levels for the future will differ from present values)

for (np in 1:num_ep) {
  #  print(paste("EP ",np,sep=""))
  if(sum(dam_p[np,])==0 & sum(pop_p[np,])==0){
    dam_fp[np,,] <-0  
    pop_fp[np,,] <-0          
    # bld_fp[np,,] <-0
  } else {
    for (ny in 1:nRY) {
    #    ny<-1
      for (nr in 1:nRP){
        if (dis_f[np,ny,1]==0){  # to avoid NAs where there's no discharge values 
          dam_fp[np,ny,nr] <-0  
          pop_fp[np,ny,nr] <-0      
          prot_RP_pf[np,ny]<-100000
          prot_RP_f[np,ny]<-100000        
        } else {
          if (dis_f[np,ny,nr]>=dis_p[np,nRP]) {   # RP given using linear intepolation from 1000 and 500y values
            dam_fp[np,ny,nr] <-dam_p[np,nRP]  
            pop_fp[np,ny,nr] <-pop_p[np,nRP]          
#           bld_fp[np,ny,nr] <-bld_p[np,nRP]              
          } else if (dis_f[np,ny,nr]<=dis_p[np,1]) {
            dam_fp[np,ny,nr] <-dam_p[np,1]         
            pop_fp[np,ny,nr] <-pop_p[np,1]  
#           bld_fp[np,ny,nr] <-bld_p[np,1]           
          } else {
            irp <-min(which(dis_p[np,]>dis_f[np,ny,nr]))
            dam_fp[np,ny,nr]<- dam_p[np,irp-1]+ (dam_p[np,irp]-dam_p[np,irp-1])*(dis_f[np,ny,nr]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])
            pop_fp[np,ny,nr]<- pop_p[np,irp-1]+ (pop_p[np,irp]-pop_p[np,irp-1])*(dis_f[np,ny,nr]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])        
#           bld_fp[np,ny,nr]<- bld_p[np,irp-1]+ (bld_p[np,irp]-bld_p[np,irp-1])*(dis_f[np,ny,nr]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])          
          }
        }
      }
    }
  }
}

## CHECK RESULTS - CORRECT
# write.csv(dam_fp[,,5],file = paste0(workDir,"/tables/EAI_Europe/test_06b/dam_fp_RP100_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)

print("step 3-NA completed")
print("---")

########################
print("calculate expected annual impacts for the future using present protection standards  (step 4-NA)")

print("- economic damage")
for (np in 1:num_ep) {
  #  print(paste("EP ",np,sep="")) 
  for (ny in 1:nRY){ 
    if(prot_RP_pf[np,ny]>=RP_classes[8]){     ## residual risk
      ead_f_na[np,ny]<-dam_fp[np,ny,8]*(1/prot_RP_pf[np,ny])
    }else{
      unprot_RP_fp<- which(prot_RP_pf[np,ny]< RP_classes[])
      damage<-RP_classes*0
      cumul_p<-RP_freq    
      ## interpolate damage between highest-protected and lowest-unprotected ref RP  
      if(min(unprot_RP_fp)>1){
        int_ind<-min(unprot_RP_fp)-1  
        damage[int_ind]<-dam_fp[np,ny,int_ind]+ (dam_fp[np,ny,int_ind+1]-dam_fp[np,ny,int_ind])*(prot_RP_pf[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
        damage[int_ind]<-(damage[int_ind]+dam_fp[np,ny,int_ind+1])/2
        cumul_p[int_ind]<-1/prot_RP_pf[np,ny]-1/RP_classes[int_ind+1]                                     
        ead_f_na[np,ny]<-damage[int_ind]*cumul_p[int_ind]          
      }
      damage[max(unprot_RP_fp)]<-dam_fp[np,ny,max(unprot_RP_fp)]    
      ## consider RP classes above local protection level    
      if(min(unprot_RP_fp)<8){
        for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
          damage[yy]<-(dam_fp[np,ny,yy]+dam_fp[np,ny,yy+1])/2
        }                                    
      }
      ead_f_na[np,ny]<-ead_f_na[np,ny]+sum(damage[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
    }
  }
}

print("- population affected")
for (np in 1:num_ep) {
  #  print(paste("EP ",np,sep=""))
  for (ny in 1:nRY){ 
    if(prot_RP_pf[np,ny]>=RP_classes[8]){     ## residual risk
      eapa_f_na[np,ny]<-pop_fp[np,ny,8]*(1/prot_RP_pf[np,ny])
    }else{
      unprot_RP_fp<- which(prot_RP_pf[np,ny]< RP_classes[])
      popaff<-RP_classes*0
      cumul_p<-RP_freq    
      ## interpolate popaff between highest-protected and lowest-unprotected ref RP  
      if(min(unprot_RP_fp)>1){
        int_ind<-min(unprot_RP_fp)-1  
        popaff[int_ind]<-pop_fp[np,ny,int_ind]+ (pop_fp[np,ny,int_ind+1]-pop_fp[np,ny,int_ind])*(prot_RP_pf[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
        popaff[int_ind]<-(popaff[int_ind]+pop_fp[np,ny,int_ind+1])/2
        cumul_p[int_ind]<-1/prot_RP_pf[np,ny]-1/RP_classes[int_ind+1]                                     
        eapa_f_na[np,ny]<-popaff[int_ind]*cumul_p[int_ind]          
      }
      popaff[max(unprot_RP_fp)]<-pop_fp[np,ny,max(unprot_RP_fp)]    
      ## consider RP classes above local protection level    
      if(min(unprot_RP_fp)<8){
        for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
          popaff[yy]<-(pop_fp[np,ny,yy]+pop_fp[np,ny,yy+1])/2
        }                                    
      }
      eapa_f_na[np,ny]<-eapa_f_na[np,ny]+sum(popaff[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
    }
  }
}

print("- builtup area affected")
## NOTE: not used
# for (np in 1:num_ep) {
#   #  print(paste("EP ",np,sep=""))
#   for (ny in 1:nRY){ 
#     if(prot_RP_pf[np,ny]>=RP_classes[8]){     ## residual risk
#       eab_f_na[np,ny]<-bld_fp[np,ny,8]*(1/prot_RP_pf[np,ny])
#     }else{
#       unprot_RP_fp<- which(prot_RP_pf[np,ny]< RP_classes[])
#       bldaff<-RP_classes*0
#       cumul_p<-RP_freq    
#       ## interpolate bldaff between highest-protected and lowest-unprotected ref RP  
#       if(min(unprot_RP_fp)>1){
#         int_ind<-min(unprot_RP_fp)-1  
#         bldaff[int_ind]<-bld_fp[np,ny,int_ind]+ (bld_fp[np,ny,int_ind+1]-bld_fp[np,ny,int_ind])*(prot_RP_pf[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
#         bldaff[int_ind]<-(bldaff[int_ind]+bld_fp[np,ny,int_ind+1])/2
#         cumul_p[int_ind]<-1/prot_RP_pf[np,ny]-1/RP_classes[int_ind+1]                                     
#         eab_f_na[np,ny]<-bldaff[int_ind]*cumul_p[int_ind]          
#       }
#       bldaff[max(unprot_RP_fp)]<-bld_fp[np,ny,max(unprot_RP_fp)]    
#       ## consider RP classes above local protection level    
#       if(min(unprot_RP_fp)<8){
#         for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
#           bldaff[yy]<-(bld_fp[np,ny,yy]+bld_fp[np,ny,yy+1])/2
#         }                                    
#       }
#       eab_f_na[np,ny]<-eab_f_na[np,ny]+sum(bldaff[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
#     }
#   }
# }

print("print and save results with no adaptation")
dir.create( paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_",ensemble,"_fp",prot_par))
dir.create( paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_",ensemble,"_fp",prot_par))   
# dir.create( paste0(workDir,"/tables/EAI_Europe/eab_ep_adapt_",ensemble,"_fp",prot_par))
write.csv(ead_f_na,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_",ensemble,"_fp",prot_par,"/ead_ep_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"))
write.csv(eapa_f_na,file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_",ensemble,"_fp",prot_par,"/eapa_ep_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"))
# write.csv(eab_f_na,file = paste0(workDir,"/tables/EAI_Europe/eab_ep_adapt_",ensemble,"_fp",prot_par,"/eab_ep_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"))

print("step 4-NA completed")
print("---")

#############################
## CALCULATE ALL POSSIBLE ADAPTATION SCENARIOS (RP(2100) range from 20y to 2000y)
## loop starts here
#############################

 for(iad in 1:length(list_RP_unif)){
# for(iad in 7:8){
  print ("---")
  print(paste0("adaptation scenario ",iad," of ",length(list_RP_unif)," - RP ",list_RP_unif[iad],"y"))
  
#############################
  print("define return periods for future protection standards (step 1-AD")
## take as design standard the ensemble mean discharge for 2100

  if(adst=="unif"){
    print ("apply uniform protection standard")

##  use as design level the 2050 discharge of the reference WL 
    prot_QD[,"2050"] <- dis_ens[,flag_WL,list_RP_unif[iad]] 
    prot_QD[,"2020"] <- prot_QP 
## TEST
#     prot_QD[,"2050"] <- 800
  } else   if(adst=="var"){
    print ("apply variable protection standard")
    print ("NOT CODED YET")
  ## TO ASSIGN VARIABLE urban-rural RP standards I need to identify urban-rural EPs 
  }  
  
  nQD <-which(lRY=="2050")-which(lRY=="2020")
  coef_QD <- (prot_QD[,"2050"]-prot_QD[,"2020"])/nQD
  ## we assume linear increase in protection standards between 2020 and 2050
  for(it in which(lRY=="2025"):which(lRY=="2045")){
    prot_QD[,it] <- prot_QD[,"2020"]+coef_QD*(it-which(lRY=="2020"))
  }
  ## assign same value to all years after 2050
  for(it in which(lRY=="2055"):which(lRY=="2100")){
    prot_QD[,it] <- prot_QD[,"2050"]
  }
  print("---")
  
    ## CHECK RESULTS - correct 
  ## here we also consider decrease of prot if Q(design)<QP, which is adjusted in next step
  # write.csv(prot_QD,file = paste0(workDir,"/tables/EAI_Europe/test_06b/prot_QD_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)
  
#########################
  print("calculate future RP of protection standards (step 2-AD)")
  ## all future RP > rescaled protected RP will cause damage
  
  ## note: fixed (present day) protection standards are used up to 2020  
  ## note: adapted protection standards are used from 2025 onwards, and only where they are higher
  ## than present standards
  
  print("calculate future RP of design protection standards")  
  for (np in 1:num_ep) {
  ## from 1981 to 2020 we use fixed present protections standards (prot_QP) 
    for (ny in 1:which(lRY=="2020")) {
## NEW: just replace with already calculated vaues      
      prot_RP_f[np,ny]<-prot_RP_pf[np,ny]
    }
  ## calculation from 2025 onward with increased protections standards  (prot_QD)   
  ## check where design discharge is actually higher than present protected value, where not leave present value  
    if(prot_QD[np,"2050"]>prot_QP[np]){  
      for (ny in which(lRY=="2025"):nRY) {
        if (dis_f[np,ny,1]==0){
          prot_RP_f[np,ny]<-100000
        }else{
          if (prot_QD[np,ny]>=dis_f[np,ny,nRP]) {  # RP given using linear intepolation from 1000 and 500y values
            prot_RP_f[np,ny]<-RP_classes[nRP-1]+ (RP_classes[nRP]-RP_classes[nRP-1])*(prot_QD[np,ny]-dis_f[np,ny,nRP-1])/(dis_f[np,ny,nRP]-dis_f[np,ny,nRP-1])      
          } else if (prot_QD[np,ny]<=dis_f[np,ny,1]) {
            prot_RP_f[np,ny] <-2  
          } else {  
            irp <-min(which(dis_f[np,ny,]>prot_QD[np,ny]))
            prot_RP_f[np,ny]<-RP_classes[irp-1]+ (RP_classes[irp]-RP_classes[irp-1])*(prot_QD[np,ny]-dis_f[np,ny,irp-1])/(dis_f[np,ny,irp]-dis_f[np,ny,irp-1])
          }
        }      
      }
    }else{    
      for (ny in which(lRY=="2025"):nRY) {
        ## NEW: just replace with already calculated vaues      
        prot_RP_f[np,ny]<-prot_RP_pf[np,ny]        
      }      
    }
  }
  prot_RP_f[prot_RP_f<2] <-2  # check to avoid negative protection standards
  
  ## CHECK RESULTS - correct
  # write.csv(prot_RP_f,file = paste0(workDir,"/tables/EAI_Europe/test_06b/prot_RP_f_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)
    
  print("step 2-AD completed")
  print("---")
  
  ########################
  print("calculate present RP of design discharges  (step 3-AD)")
  ## actualize damage for reference future RPs to present (Q return levels for the future will differ from present values)
#  RP_unif<-c("2", "10", "15", "20", "30", "50", "70", "100", "150", "200", "350", "500", "700", "1000", "1500", "2000")  
#  prot_RP_des <- array(0, dim = c(num_ep, length(RP_unif)+1), dimnames = list(c(1:num_ep),c("present", RP_unif)))  
#  prot_QD <- read.csv(paste(workDir,"/tables/EAI_Europe/test_06b/prot_QD_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv", sep=""))

  for (np in 1:num_ep) {
    # print(paste("EP ",np,sep=""))
    prot_RP_des[np,1] <-prot_RP_p[np]  
      if (prot_QD[np,"2050"]==0){  # to avoid NAs where there's no discharge values 
        prot_RP_des[np,iad+1] <-2  
      } else {
        if (prot_QD[np,"2050"]>=dis_p[np,nRP]) {   # RP given using linear intepolation from 1000 and 500y values
          prot_RP_des[np,iad+1]<- RP_classes[nRP-1]+ (RP_classes[nRP]-RP_classes[nRP-1])*(prot_QD[np,"2050"]-dis_p[np,nRP-1])/(dis_p[np,nRP]-dis_p[np,nRP-1])            
        } else if (prot_QD[np,"2050"]<=dis_p[np,1]) {
          prot_RP_des[np,iad+1]<- 2   
        } else {
          irp <-min(which(dis_p[np,]>prot_QD[np,"2050"]))
          prot_RP_des[np,iad+1]<- RP_classes[irp-1]+ (RP_classes[irp]-RP_classes[irp-1])*(prot_QD[np,"2050"]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])
        }
      }
  }
  print("step 3-AD completed") # now corrected
  print("---")
  
  ######################
  print("calculate expected annual impacts for the future  (step 4-AD)")
  
  ## ACTIVATE TO TEST WITH NO PROT
  # prot_RP_p <- prot_RP_p*0 
  
  print("- population affected")
  for (np in 1:num_ep) {
  #  print(paste("EP ",np,sep=""))
    for (ny in 1:nRY){       
      if(prot_RP_f[np,ny]>=RP_classes[8]){     ## residual risk
        eapa_f[np,ny]<-pop_fp[np,ny,8]*(1/prot_RP_f[np,ny])
      }else{
        unprot_RP_fp<- which(prot_RP_f[np,ny]< RP_classes[])
        popaff<-RP_classes*0
        cumul_p<-RP_freq    
  ## interpolate popaff between highest-protected and lowest-unprotected ref RP  
        if(min(unprot_RP_fp)>1){
          int_ind<-min(unprot_RP_fp)-1  
          popaff[int_ind]<-pop_fp[np,ny,int_ind]+ (pop_fp[np,ny,int_ind+1]-pop_fp[np,ny,int_ind])*(prot_RP_f[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
          popaff[int_ind]<-(popaff[int_ind]+pop_fp[np,ny,int_ind+1])/2
          cumul_p[int_ind]<-1/prot_RP_f[np,ny]-1/RP_classes[int_ind+1]                                     
          eapa_f[np,ny]<-popaff[int_ind]*cumul_p[int_ind]          
        }
        popaff[max(unprot_RP_fp)]<-pop_fp[np,ny,max(unprot_RP_fp)]    
  ## consider RP classes above local protection level    
        if(min(unprot_RP_fp)<8){
          for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
            popaff[yy]<-(pop_fp[np,ny,yy]+pop_fp[np,ny,yy+1])/2
          }                                    
        }
        eapa_f[np,ny]<-eapa_f[np,ny]+sum(popaff[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
      }
    }
  }
  
  print("- economic damage")
  for (np in 1:num_ep) {
  #  print(paste("EP ",np,sep="")) 
    for (ny in 1:nRY){ 
      if(prot_RP_f[np,ny]>=RP_classes[8]){     ## residual risk
        ead_f[np,ny]<-dam_fp[np,ny,8]*(1/prot_RP_f[np,ny])
      }else{
        unprot_RP_fp<- which(prot_RP_f[np,ny]< RP_classes[])
        damage<-RP_classes*0
        cumul_p<-RP_freq    
        ## interpolate damage between highest-protected and lowest-unprotected ref RP  
        if(min(unprot_RP_fp)>1){
          int_ind<-min(unprot_RP_fp)-1  
          damage[int_ind]<-dam_fp[np,ny,int_ind]+ (dam_fp[np,ny,int_ind+1]-dam_fp[np,ny,int_ind])*(prot_RP_f[np,ny]-RP_classes[int_ind])/(RP_classes[int_ind+1]-RP_classes[int_ind])      
          damage[int_ind]<-(damage[int_ind]+dam_fp[np,ny,int_ind+1])/2
          cumul_p[int_ind]<-1/prot_RP_f[np,ny]-1/RP_classes[int_ind+1]                                     
          ead_f[np,ny]<-damage[int_ind]*cumul_p[int_ind]          
        }
        damage[max(unprot_RP_fp)]<-dam_fp[np,ny,max(unprot_RP_fp)]    
        ## consider RP classes above local protection level    
        if(min(unprot_RP_fp)<8){
          for (yy in min(unprot_RP_fp):(max(unprot_RP_fp)-1)) {
            damage[yy]<-(dam_fp[np,ny,yy]+dam_fp[np,ny,yy+1])/2
          }                                    
        }
        ead_f[np,ny]<-ead_f[np,ny]+sum(damage[unprot_RP_fp]*cumul_p[unprot_RP_fp])   
      }
    }
  }
  
  print("step 4-AD completed")
  print("---")
  
  ######################
  print("aggregate expected annual impacts at NUTS2 scale (step 5-AD")

## THIS PART IS NOT NEEDED  
#   for (ny in 1:nRY){ 
#     for (np in 1:nNU) {
#   #    print(paste("NUTS2 ",list_nuts2[np],sep=""))
#       ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
#       ead_f_nuts2[np,ny]<-sum(ead_f[ind_nuts,ny]) 
#       eapa_f_nuts2[np,ny]<-sum(eapa_f[ind_nuts,ny]) 
#     }
#   }
  
  print("---")
  
  #####################
  print("- calculate annual impacts and impacts for warming levels (step 6-AD)")

## CLIMATE-ONLY PROJECTIONS OF STABLE CLIMATE NOT CALCULATED HERE  
  ## interpolate model values every 5 years
  ## damage
#   ead_nuts2_ext <- array(0, dim = c(nNU,nRYT),dimnames = list(list_nuts2,c(startY:max(RY_classes))))
#   ead_nuts2_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))
#   ead_nuts2_ext[,1]<-ead_f_nuts2[,1]
#   for (ii in 1:(nRY-1)){
#     ead_nuts2_ext[,ii*5+1]<-ead_f_nuts2[,ii+1]
#   }
#   for (jj in 1:(nRY-1)){
#     ead_nuts2_ext[,jj*5-3]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])/5
#     ead_nuts2_ext[,jj*5-2]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])*2/5
#     ead_nuts2_ext[,jj*5-1]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])*3/5
#     ead_nuts2_ext[,jj*5-0]<-ead_f_nuts2[,jj]+(ead_f_nuts2[,jj+1]-ead_f_nuts2[,jj])*4/5  
#   }
#   
#   ## population
#   eapa_nuts2_ext <- array(0, dim = c(nNU,nRYT),dimnames = list(list_nuts2,c(startY:max(RY_classes))))
#   eapa_nuts2_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))
#   eapa_nuts2_ext[,1]<-eapa_f_nuts2[,1]
#   for (ii in 1:(nRY-1)){
#     eapa_nuts2_ext[,ii*5+1]<-eapa_f_nuts2[,ii+1]
#   }
#   for (jj in 1:(nRY-1)){
#     eapa_nuts2_ext[,jj*5-3]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])/5
#     eapa_nuts2_ext[,jj*5-2]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])*2/5
#     eapa_nuts2_ext[,jj*5-1]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])*3/5
#     eapa_nuts2_ext[,jj*5-0]<-eapa_f_nuts2[,jj]+(eapa_f_nuts2[,jj+1]-eapa_f_nuts2[,jj])*4/5  
#   }
#   
#   ## damage
#   for (np in 1:nNU) {
#     ead_nuts2_WL[np,1]<-mean(ead_nuts2_ext[np,max(1,ind_base-14):min(ind_base+15,nRYT)])  
#     ead_nuts2_WL[np,2]<-mean(ead_nuts2_ext[np,max(1,ind_WL15-14):min(ind_WL15+15,nRYT)])
#     ead_nuts2_WL[np,3]<-mean(ead_nuts2_ext[np,max(1,ind_WL20-14):min(ind_WL20+15,nRYT)])
#     if(WL_table[ind_MR,"X3C"]==-1){
#       ead_nuts2_WL[np,4]<-0
#     }else{
#       ead_nuts2_WL[np,4]<-mean(ead_nuts2_ext[np,max(1,ind_WL30-14):min(ind_WL30+15,nRYT)])
#     }
#   }
#   ## population
#   for (np in 1:nNU) {
#     eapa_nuts2_WL[np,1]<-mean(eapa_nuts2_ext[np,max(1,ind_base-14):min(ind_base+15,nRYT)])  
#     eapa_nuts2_WL[np,2]<-mean(eapa_nuts2_ext[np,max(1,ind_WL15-14):min(ind_WL15+15,nRYT)])
#     eapa_nuts2_WL[np,3]<-mean(eapa_nuts2_ext[np,max(1,ind_WL20-14):min(ind_WL20+15,nRYT)])
#     if(WL_table[ind_MR,"X3C"]==-1){
#       eapa_nuts2_WL[np,4]<-0
#     }else{
#       eapa_nuts2_WL[np,4]<-mean(eapa_nuts2_ext[np,max(1,ind_WL30-14):min(ind_WL30+15,nRYT)])
#     }
#   }
  
  print("step 6-AD completed")
  print("---")
  
  #####################
  print("print and save results")
  ## write file of EP results every 5 years
   fileSuf<- paste0(flag_WL,"_",adst,"_RP",list_RP_unif[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
  write.csv(ead_f,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_",ensemble,"_fp",prot_par,"/ead_ep_",fileSuf))
  write.csv(eapa_f,file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf))
  # CLIMATE-ONLY PROJECTIONS OF STABLE CLIMATE NOT CALCULATED HERE   
  # write impacts at NUTS2 level for all years
#   write.csv(ead_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_clim_adapt_fp",prot_par,"/ead_nuts2_clim_",fileSuf),row.names=TRUE)
#   write.csv(eapa_nuts2_ext,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_clim_adapt_fp",prot_par,"/eapa_nuts2_clim_",fileSuf),row.names=TRUE)
  # write impacts for warming levels
#   write.csv(ead_nuts2_WL,file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_WL_clim_adapt_fp",prot_par,"/ead_nuts2_WL_clim_",fileSuf),row.names=TRUE)
#   write.csv(eapa_nuts2_WL,file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_WL_clim_adapt_fp",prot_par,"/eapa_nuts2_WL_clim_",fileSuf),row.names=TRUE)

} ## end loop for adaptation scenarios

print('Risk analysis for future conditions COMPLETED')
print("---")
#####################
print("print and save results of future RP of protection standards")
# write.csv(prot_RP_des,file = paste0(workDir,"/tables/EAI_Europe/prot_des_adapt/prot_RP_des_",flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.names=FALSE)


print('script ended')

