## 
## allocate storage volumes along EFAS river network using iterative procedure
## calculate cost, benefits and NPV 

#### SET RUN MODE
run<-"cluster" ## choose whether to run on (local) or (cluster)
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean
# MR_data<-"Rdata" ## read (nectdf) file or using R dataset (Rdata)

## load libraries
library(sp,lib.loc="/home/dottofr/Rlib")     	# needed to load rdgdal library
Sys.sleep(1)
# library(foreign,lib.loc="/home/dottofr/Rlib") 	# not needed anymore 
#### 

if(run=='cluster'){
  args <- commandArgs(TRUE)
  modelRun <- as.character(args[1])     # model run to analyse
  mainDir <- as.character(args[2])     # directory of config file    
} else if(run=='local'){
  modelRun <- "projection_dis_rcp85_CLMcom-CCLM4-8-17_BC_CNRM-CERFACS-CNRM-CM5_wuConst_statistics_ptId.nc" 
}
flag_WL <- as.character(args[3]) # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
prot_par<- as.character(args[4]) # level of flood protection - 050: 50% of estimated average protection standards
dam_par <- as.character(args[5]) # level of flood damage -q95
cost_par<- as.character(args[6]) # level of costs -q95
scriptDir <- as.character(args[7])     # directory of scripts and configuration files 
# flag_WL <- "3C" # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
# prot_par<- "050"/"100"/"150"/
# dam<- "q05"/"avg"/"q95"
# cost<- "q25"/"avg"/"q75"

workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))
workDir2<-workDir # backup variable to be used if Rdata has different workDir 
scriptDir2<-scriptDir # backup variable to be used if Rdata has different workDir 

print(paste("- run analysis for WL ",flag_WL," - ensemble ",ensemble,sep=""))
#######
## INPUT PARAMETERS FOR COST-BENEFIT CALCULATIONS

table_cost <- read.csv(paste(workDir,"/tables/table_unit_costs_nuts2_",cost_par,".csv", sep=""))
# table_cost <- read.csv(paste(workDir,"/tables/table_unit_costs_nuts2_avg.csv", sep=""))

# flag_dis="yes" # use annual discount rate or not
# DisRate <- 0.0 # annual discount rate  -replaced by country values
CostCoef <- 1 # correction coefficient for costs   
MaintCoef <- 0.01 # annual maintenance costs as fraction of construction costs    
ComYear <- 2050 # year of reference for completion of adaptation measures 
RefYear <- 2100 # year of reference for design of adaptation measure

## input parameters of storage areas design
Cst_AH=3.5      # max height of storage per m2       
Cred=0.5      # fraction of upstream volume (Vst +VF_RD) used to reduce VF for all downstream branches
Cst=1         # storage coefficient, if 1 only locally needed volume is stored  
Cinc=3        # increase of unit cost for storing residual flood volumes, as compared to standard cost

##########
print('read discharges from Rdata file')

run2<-run  ## correction in case value in RData is different
load(file = paste0(workDir,"/Rdata/",modelRun,".RData"))
if(run2=='local'){
  run='local'
}
workDir <-workDir2
scriptDir<-scriptDir2

## interpolate RP 200y using 150y and 250y
dis_150_250 <- dis_f[,,c(14,15,15)]  
dis_150_250[,,3] <- (dis_150_250[,,1]+dis_150_250[,,2])/2    
## replace 250y wit interpolated 200y
dis_f[,,15] <- dis_150_250[,,3]
rm(dis_150_250)

## keep only used RPs (2,10,20,50,100,200,500,1000)
dis_f<-dis_f[,,c(2,7,9,11,13,15,17,19)]
## delete years not used
dis_p <- dis_f[,8,] # year 2015 as reference to match discharge and impacts
dis_f[is.na(dis_f)] <- 0
dis_p[is.na(dis_p)] <- 0

print("read ensemble averages/median of discharge")
load(file = paste0(workDir,"/Rdata/ens_discharges_",ensemble,".RData"))
if(ensemble=="median"){
  dis_ens <- dis_ens_median
}
print('reading of discharge data done')

##########
print('input data')
## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)

num_ep <-dim(dis_p)[1] # number of EFAS network cells 

# initialize RP classes of dis_p
 RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
nRP<-length(RP_classes)
nRY<-length(RY_classes)
nRYT<-max(RY_classes)-min(RY_classes)+1
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
} 

#########
## READ INPUT TABLES

## read present level of protection in each point (RP in years)
prot_RP_p_table <- read.csv(paste(workDir,"/tables/protection_levels/table_EP_protLev_2018_fp",prot_par,".csv", sep=""))


## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]

## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))

## read EP- river branches tables
table_EP_branches <- read.csv(paste(workDir,"/tables/table_EP_FP_branches_v3.csv", sep=""))
table_branches <- read.csv(paste(workDir,"/tables/table_branches.csv", sep=""))
lBR<-dim(table_branches)[1]

## read Q ratio table
table_ratios <- read.csv(paste(workDir,"/tables/Qratio/Qratio_all.csv", sep=""))

## read list of regions
list_regions <- read.table(paste(scriptDir,"/config_files/listRegions.txt", sep=""))
lReg<-dim(list_regions)[1]    

#########
## FORMAT AND INITIALIZE  DATA

# dimnames(dis_f)<-list(c(1:num_ep),lRY,RP_classes)

nRYT<-max(RY_classes)-min(RY_classes)+1
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
} 
nRYC<-max(RY_classes)-2015+1
#######         
      
## initialize return periods for future protection standards
## uniform and variable RP (urban and rural/natural)
## NOTE: variable RP to be used for adaptation based on storage areas
# RP_unif<-c("2", "10", "15", "20", "30", "50", "70", "100", "150", "200", "350", "500", "700", "1000", "1500", "2000")
RP_var<-c("2", "10", "15", "20", "30", "50", "50", "50", "100", "100", "100", "100", "150", "150", "150", "200")
      
list_RP_unif<-dimnames(dis_ens)[[3]]
list_RP_var<-cbind(dimnames(dis_ens)[[3]],RP_var)
colnames(list_RP_var)<-c("urban","rural")
      
## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)      
      
## read model run set up and corresponding warming levels
ind_MR<- which(WL_table[,"ModelRun"]==modelRun)
Pr_mod<-WL_table[ind_MR,"Model"]
Pr_rcp<-WL_table[ind_MR,"RCP"]
Pr_wu<-WL_table[ind_MR,"WU"]
ind_base<-WL_table[ind_MR,"Base"]-min(RY_classes)+1
ind_WL15<-WL_table[ind_MR,"X1.5C"]-min(RY_classes)+1
ind_WL20<-WL_table[ind_MR,"X2C"]-min(RY_classes)+1
ind_WL30<-WL_table[ind_MR,"X3C"]-min(RY_classes)+1

## format protection level tables and assign synthetic values (if requested)
prot_RP_p <- prot_RP_p_table[,2] 
rm(prot_RP_p_table)
if (test=='yes'){
  prot_RP_p <- numeric(num_ep)
  prot_RP_p[1:(0.5*num_ep)] <- 80
  prot_RP_p[(0.5*num_ep+1):num_ep] <- 80  
}
prot_QP <- prot_RP_p*0

## initialize cost - benefit annual tables 
cost_f_dis <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes)))) 
ben_f_dis <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes))))  
cost_f <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes)))) 
ben_f <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes))))  

modCost <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif)) 
modBen <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))
modNPV <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))   
modPopR_2100 <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))   
modDamR_2100 <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))   
modCost_ud<- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif)) 
modWLI <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))
# cost_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# ben_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# popR_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# NPV_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))

## SAVE EP values to CHECK total costs etc  
cost_tot_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
ben_tot_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
cost_ud_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
NPV_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
damR_2100_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
popR_2100_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))

print('input data reading done')
      
##########
##  PREPARE INPUT DATA TO CALCULATE STORAGE
##########

print("build floodplain land use table for all EPs, using regional tables")
      
LUfl<-array(0, dim = c(num_ep,4), dimnames = list(c(1:num_ep),c("A_tot","A_agr","A_nat","A_urb")))
table_lu <-read.csv(paste(workDir,"/tables/EP_LU_tables/table_EP_LU_Europe.csv",sep=""))
for (j in 1:dim(table_lu)[1]){
    LUfl[table_lu[j,1],"A_tot"]<-table_lu[j,"area"] 
    LUfl[table_lu[j,1],"A_agr"]<-table_lu[j,"A_agr"] 
    LUfl[table_lu[j,1],"A_nat"]<-table_lu[j,"A_nat"]  
    LUfl[table_lu[j,1],"A_urb"]<-table_lu[j,"A_urb"]      
}

######
print("build EP-BR table using regional ID of branches (IDR)")
      
NETfl<-array(0, dim = c(num_ep,2), dimnames = list(c(1:num_ep),c("IDR","EU_IDR")))
NETfl[,"IDR"]<-table_EP_branches[,"IDR_corr"]
for (j in 1:lBR){
  region<-as.character(table_branches[j,"Region"])
  ind_net<-which(table_EP_branches[,"IDR_corr"]==table_branches[j,"ARCID"] & table_EP_branches[,"Region"]==region) 
  NETfl[ind_net,"EU_IDR"]<-table_branches[j,"EU_IDR"]
  ind_reg<-which(list_regions[]==region)                
}      
      
######
print("prepare tables for elaborating storage")

## NOTE: ADD Q_PEAK AND TC TO DETECT UNREALISTIC VALUES IN RIVER BRANCHES
table_input <-array(0, dim= c(lBR,10), dimnames = list(c(1:lBR),c("IDR","TO_IDR","order","Q_peak","TC","V_hyd","V_prot","V_agr","V_nat","V_urb")))
            
table_input[,"IDR"]<-table_branches[,"EU_IDR"]  
table_input[,"order"]<-table_branches[,"ORDER_"]      
for (nc in 1:lBR){
## convert TO_ARCID (regional id) to EU ID
 if(table_branches[nc,"TO_ARCID"]==0){
    table_input[nc,"TO_IDR"]<-9999 # branch is the outlet, put standard ID
 }else{
    region<-as.character(table_branches[nc,"Region"])
    ind_down<-which(table_branches[,"ARCID"]==table_branches[nc,"TO_ARCID"] & table_branches[,"Region"]==region)
    table_input[nc,"TO_IDR"]<-table_branches[ind_down,"EU_IDR"]  
 }  
##  Convert area to volume assuming Cst_AH as max height of storage  
 ind_br<-which(NETfl[,"EU_IDR"]==table_branches[nc,"EU_IDR"])
 table_input[nc,"V_agr"]<-sum(LUfl[ind_br,"A_agr"])*Cst_AH*10^6  
 table_input[nc,"V_urb"]<-sum(LUfl[ind_br,"A_urb"])*Cst_AH*10^6   
 table_input[nc,"V_nat"]<-sum(LUfl[ind_br,"A_nat"])*Cst_AH*10^6
}
## write in output to check elaboration   
#  write.csv(table_input,file = paste0(workDir,"/tables/table_storage_input.csv"),row.names=FALSE)
            
## TEST: synthetic river basin with 13 branches
if(test=="yes"){
  table_input <-read.csv(paste(workDir,"/tables/table_test_basin.csv", sep=""))
  nBR<-dim(table_input)[1] # number of river branches 
}
            
print("initialize output table")
table_output <- array(0,dim=c(lBR,6), dimnames = list(c(1:lBR),c("VF_T","VF_ST","VF_RD","Vst_nat","Vst_agr","Vst_UP")))
      
#############################
print('---')
## CALCULATE ALL POSSIBLE ADAPTATION SCENARIOS (RP(2100) range from 20y to 2000y)
## loop starts here
for(iad in 1:length(list_RP_unif)){
# for(iad in 4:6){
    
## WRITE IN NOTES
  print("---")
  print(paste0("adaptation scenario ",iad," of ",length(list_RP_unif)," -  RP ", list_RP_unif[iad], " years"))

  
  ########################  
## TEST: USE CLIMATOLOGY TO CALCULATE FLOOD VOLUMES, RESCALED WITH MODEL RPs   
## this part is still to be fully tested - maybe useful for future analyses 
#  print("read EFAS climatology")

#   ##  dis_clim<- merge all EPs from from EFAS
#   dis_res[np] <-array(0, dim = c(num_ep,8))
#   colnames(dis_res)<- c(2,10,20,50,100,200,500,1000)
#   
#   table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))
#   grid_mask<- rasterFromXYZ(table_EP_XY[,2:4], res=c(5000,5000), crs=NA, digits=5)
#   #  grid_id<- rasterFromXYZ(cbind(table_EP_XY[,2:3],table_EP_XY[,1]), res=c(5000,5000), crs=NA, digits=5) 
#   # rl_id<-rasterToPoints(grid_id) 
#   climDir<-("//ies.jrc.it/H07/ClimateRun1/flood_hazard_mapping/Eu_EFAS/Eu_EFAS/calibration_2018")   
#   rl_002<-raster(readGDAL(paste0(climDir,"/rl_002.map"),silent=TRUE)) 
#   rl_002_clip<-rl_002*grid_check
#   rl_002_p<-rasterToPoints(rl_002_clip)
# ## etc...
#   N3EP<-dim(rl_002_clip)[1]  
#    for(i in 1:N3EP){
#     ind=which(rl_002_p[i,"x"]== table_EP_XY[,"X"] & rl_002_p[i,"y"]== table_EP_XY[,"Y"])
#         dis_res(ind,"2")<-rl_002_p[i,"layer"]
#         dis_res(ind,"10")<-rl_010_p[i,"layer"] 
# # etc...    
#    }
  
  ########################  
#   print("rescale discharge value using EFAS climatology")
#   ## actualize damage for reference future RPs to present (Q return levels for the future will differ from present values)
#   
#   ## cHECK IF RESULTS are correct!!!
# 
#   for (np in 1:num_ep) {
#     #  print(paste("EP ",np,sep=""))
#     if (  dis_ens[np,flag_WL,iad]==0){  # to avoid NAs where there's no discharge values
#       # RP_fp[np,ny,nr]<-2     
#       dis_res[np] <-0 
#     } else {
#       if (dis_ens[np,flag_WL,iad]>=dis_p[np,nRP]) {   # values given using linear intepolation from 1000 and 500y values
#         # RP_fp[np,ny,nr]<- RP_classes[nRP-1]+ (RP_classes[nRP]-RP_classes[nRP-1])*(dis_ens[np,flag_WL,iad]-dis_p[np,nRP-1])/(dis_p[np,nRP]-dis_p[np,nRP-1])    
#         dis_res[np]<- dis_clim[np,nRP-1]+ (dis_clim[np,nRP]-dis_clim[np,nRP-1])*(dis_ens[np,flag_WL,iad]-dis_p[np,nRP-1])/(dis_p[np,nRP]-dis_p[np,nRP-1])    
#       } else if (dis_ens[np,flag_WL,iad]<=dis_p[np,1]) {
#         # RP_fp[np,ny,nr]<- 2    
#         dis_res[np] <-dis_clim[np,1]        
#       } else {
#         irp <-min(which(dis_p[np,]>dis_ens[np,flag_WL,iad]))
#         # RP_fp[np,ny,nr]<- RP_classes[irp-1]+ (RP_classes[irp]-RP_classes[irp-1])*(dis_ens[np,flag_WL,iad]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])
#         dis_res[np]<- dis_clim[np,irp-1]+ (dis_clim[np,irp]-dis_clim[np,irp-1])*(dis_ens[np,flag_WL,iad]-dis_p[np,irp-1])/(dis_p[np,irp]-dis_p[np,irp-1])
#       }
#     }
#   }
#   print("step completed") 
#   ### TEST: USE CLIMATOLOGY TO CALCULATE FLOOD VOLUMES, RESCALED WITH MODEL RPs   
  
#############################        
  print("extract present protection values (RP) in terms of Q")
  
  mask_P<- which(prot_RP_p[]>2 & prot_RP_p[]<=10)
  prot_QP[mask_P] <-dis_p[mask_P,1] + (dis_p[mask_P,2]-dis_p[mask_P,1])*(prot_RP_p[prot_RP_p[]>2 & prot_RP_p[]<=10] - 2)/(10-2)
  mask_P<- which(prot_RP_p[]>10 & prot_RP_p[]<=20)
  prot_QP[mask_P] <-dis_p[mask_P,2] + (dis_p[mask_P,3]-dis_p[mask_P,2])*(prot_RP_p[prot_RP_p[]>10 & prot_RP_p[]<=20] - 10)/(20-10)
  mask_P<- which(prot_RP_p[]>20 & prot_RP_p[]<=50)
  prot_QP[mask_P] <-dis_p[mask_P,3] + (dis_p[mask_P,4]-dis_p[mask_P,3])*(prot_RP_p[prot_RP_p[]>20 & prot_RP_p[]<=50] - 20)/(50-20)
  mask_P<- which(prot_RP_p[]>50 & prot_RP_p[]<=100)
  prot_QP[mask_P] <-dis_p[mask_P,4] + (dis_p[mask_P,5]-dis_p[mask_P,4])*(prot_RP_p[prot_RP_p[]>50 & prot_RP_p[]<=100] - 50)/(100-50)
  mask_P<- which(prot_RP_p[]>100 & prot_RP_p[]<=200)
  prot_QP[mask_P] <-dis_p[mask_P,5] + (dis_p[mask_P,6]-dis_p[mask_P,5])*(prot_RP_p[prot_RP_p[]>100 & prot_RP_p[]<=200] - 100)/(200-100)
  mask_P<- which(prot_RP_p[]>200 & prot_RP_p[]<=500)
  prot_QP[mask_P] <-dis_p[mask_P,6] + (dis_p[mask_P,7]-dis_p[mask_P,6])*(prot_RP_p[prot_RP_p[]>200 & prot_RP_p[]<=500] - 200)/(500-200)
  mask_P<- which(prot_RP_p[]>500 & prot_RP_p[]<=1000)
  prot_QP[mask_P] <-dis_p[mask_P,7] + (dis_p[mask_P,8]-dis_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>500 & prot_RP_p[]<=1000] - 500)/(1000-500)
  mask_P<- which(prot_RP_p[]>1000)
  prot_QP[mask_P] <-dis_p[mask_P,8] + (dis_p[mask_P,8]-dis_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>1000] - 1000)/(1000-500)
  
  rm(mask_P)
  
######
print("calculate flood volumes for peak discharges and present defence standards")
    
  Vfl<-array(0, dim = c(num_ep,4))
  colnames(Vfl)<- c("Prot_VF","Des_VF","Des_QP","TC")
  N3EP<-dim(table_ratios)[1]  
  if(adst=="unif"){
    print ("apply uniform protection standard")
    for(i in 1:N3EP){
      lHyd <- table_ratios[i,"timeCon"]-1
      if(lHyd>=0 & table_ratios[i,"ratio_D0.001"]>=0){
        Vfl[table_ratios[i,"EP"],"Prot_VF"] <- prot_QP[table_ratios[i,"EP"]]*24*3600*sum(table_ratios[i,5:(5+lHyd)])
        Vfl[table_ratios[i,"EP"],"Des_VF"] <- dis_ens[table_ratios[i,"EP"],flag_WL,iad]*24*3600*sum(table_ratios[i,5:(5+lHyd)])
        Vfl[table_ratios[i,"EP"],"Des_QP"] <- dis_ens[table_ratios[i,"EP"],flag_WL,iad]
        Vfl[table_ratios[i,"EP"],"TC"] <- table_ratios[i,"timeCon"]       
      }
    }
    ##  use as design level the 2050 discharge of the reference WL   
  } else   if(adst=="var"){
    print ("apply variable protection standard")
    print ("NOT CODED YET")
    ## TO ASSIGN VARIABLE urban-rural RP standards I need to identify urban-rural EPs 
  }  
    
  ## CHECK OF DES_QP VALUES
  ## read table of XY coordinates for EFAS points
#     table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))
#     val_check<-cbind(table_EP_XY[,2:3],Vfl[,"Des_QP"])
#     grid_check<- rasterFromXYZ(val_check, res=c(5000,5000), crs=NA, digits=5)
#     writeGDAL(as(grid_check,'SpatialGridDataFrame'), fname=paste0(workDir,"/maps/grid_check.tif"),band, drivername='GTiff',type="Float32",mvFlag=-9999,options=c("INTERLEAVE=PIXEL","COMPRESS=LZW"))                                                                                                           
  
######
  print("calculate flood volumes")
  
  for (nc in 1:lBR){
  ## retrieve EP linked to each branch  
    
## NOTE: ADD Q_PEAK AND TC TO DETECT UNREALISTIC VALUES IN RIVER BRANCHES    
    ind_br<-which(NETfl[,"EU_IDR"]==table_branches[nc,"EU_IDR"])
    table_input[nc,"V_hyd"]<-max(0,median(Vfl[ind_br,"Des_VF"],na.rm=TRUE))   # take the median of flood volumes among all EPs
    table_input[nc,"V_prot"]<-max(0,median(Vfl[ind_br,"Prot_VF"],na.rm=TRUE)) # this avoids some extremely high value, e.g. in Spain    
    table_input[nc,"Q_peak"]<-max(0,median(Vfl[ind_br,"Des_QP"],na.rm=TRUE)) # this avoids some extremely high value, e.g. in Spain    
    table_input[nc,"TC"]<-max(0,median(Vfl[ind_br,"TC"],na.rm=TRUE)) # this avoids some extremely high value, e.g. in Spain    
## NOTE: ADD Q_PEAK AND TC TO DETECT UNREALISTIC VALUES IN RIVER BRANCHES
    
    table_input[is.na(table_input)]<-0  
  }
    
  print("adjust peak flows to avoid unrealistic values")  
  count1=0
  for (io in 1:max(table_input[,"order"])){  ## loop over all branch orders in the river network
    #  io=3
    loop_BR=which(table_input[,"order"]==io) # indices of branches with order=io
    if(length(loop_BR)>0){
      for (ibr in 1:length(loop_BR)){
#        print(paste("order ",io," - ",ibr," of ",length(loop_BR)),sep="")
        ind<-loop_BR[ibr]
        BR_down<- table_input[ind,"TO_IDR"]  
        if(BR_down==9999){
#          print("outlet branch")
          count1=count1
        }else if( table_input[BR_down,"Q_peak"]==0){
#          print("downstream branch with no associated discharge")
          count1=count1
        }else{ 
          Q_check<-table_input[ind,"Q_peak"]/table_input[BR_down,"Q_peak"]        
          if(Q_check>1.1){
  ## use downstream values to correct upstream values
            count1=  count1+1
            table_input[ind,"V_hyd"]<-1.1*table_input[BR_down,"V_hyd"]*table_input[ind,"TC"]/table_input[BR_down,"TC"]              
            table_input[ind,"V_prot"]<-1.1*table_input[BR_down,"V_prot"]*table_input[ind,"TC"]/table_input[BR_down,"TC"]                         
          }                  
        }
     }
    }
  }
  
## write in output to check elaboration   
#  write.csv(table_input,file = paste0(workDir,"/tables/table_storage_input.csv"),row.names=FALSE)

######
  print("calculate storage volumes with volume allocation routine")  

  ## version 4: use of Cst >1 allowed
  ## version 3: part of total upstream volume (Vst +VF_RD) is used to reduce VF for all downstream branches
  ## version 2: all VF_RD upstream used to reduce downstream volume to be stored
  
  ## initialize theoretical flood volume to be stored  
  table_output[,2:6]<-0
  table_output[,"VF_T"]<-table_input[,"V_hyd"]-table_input[,"V_prot"]
  table_output[table_output<0]<-0
#  table_output[,"VF_ST"]<-table_output[,"VF_T"] # commented in Version 3 and 4
  for (io in 1:max(table_input[,"order"])){  ## loop over all branch orders in the river network
    #  io=3
    loop_BR=which(table_input[,"order"]==io) # indices of branches with order=io
    if(length(loop_BR)>0){
      for (ibr in 1:length(loop_BR)){
#        print(paste("order ",io," - ",ibr," of ",length(loop_BR)),sep="")
        ind<-loop_BR[ibr]
        Vst_agr_t=0
        Vst_nat_t=0  

## version 4
## use  Vst_check to calculate Vst    
## check on V_hyd to make sure there is enough flood volume to store          
#         Vst_check<-min(table_input[ind,"V_hyd"],Cst*table_output[ind,"VF_T"])         # version 4 
# ## use Vrd_check to compute VF_RD based on the actual volume to be stored ( Cst always =1)
#         Vrd_check<-min(table_input[ind,"V_hyd"],(table_output[ind,"VF_T"]-table_output[ind,"Vst_UP"])) # version 4          
#         ## calculate availability of storage areas, first semi-natural and then agricultural          
#         Vst_nat_t<-min(table_input[ind,"V_nat"],Vst_check)
#         if(table_input[ind,"V_nat"]<Vst_check){
#           Vst_agr_t<-min(table_input[ind,"V_agr"],(Vst_check-Vst_nat_t)) # store residual volume in agricultural areas 
#           if(table_input[ind,"V_agr"]<(Vst_check-Vst_nat_t)){
#             ## residual flood  volume only if available storage is not enough
#             table_output[ind,"VF_RD"]<- max(0,(Vrd_check-Vst_nat_t-table_input[ind,"V_agr"]))        
#           }           
#         }   
## end part version 4         
## version 3: we use only one V_check, works only if Cst=1 
## check on V_hyd to make sure there is enough flood volume to store              
      V_check<-min(table_input[ind,"V_hyd"],Cst*(table_output[ind,"VF_T"]-table_output[ind,"Vst_UP"]))  # version 3          
##      V_check<-min(table_input[ind,"V_hyd"],Cst*table_output[ind,"VF_ST"])  # version 2        
## calculate availability of storage areas, first semi-natural and then agricultural#            
        Vst_nat_t<-min(table_input[ind,"V_nat"],V_check)
        if(table_input[ind,"V_nat"]<V_check){
          Vst_agr_t<-min(table_input[ind,"V_agr"],(V_check-Vst_nat_t)) # store residual volume in agricultural areas 
          if(table_input[ind,"V_agr"]<(V_check-Vst_nat_t)){
            ## residual flood  volume only if available storage is not enough
            table_output[ind,"VF_RD"]<- (V_check-Vst_nat_t)-table_input[ind,"V_agr"] 
          }           
        }
## end part version 3        
        ## update stored flood volumes in [ind] and downstream 
        table_output[ind,"Vst_agr"]=max(0,Vst_agr_t)
        table_output[ind,"Vst_nat"]=max(0,Vst_nat_t)
        BR_down<- table_input[ind,"TO_IDR"]  
        table_output[ind,"VF_ST"]<- max(0,Vst_nat_t+Vst_agr_t)    
#       table_output[ind,"VF_ST"]<- Vst_nat_t+Vst_agr_t  # version 2           
        if(BR_down==9999){
#          print("outlet branch")
          count1=count1
        }else{ 
## part of the stored volume in cell "ind" will be available to downstream branch to reduce storage need
##  = previusly  calculated reduction (e.g. from other branches) 
##  + all reduction available from upstream in cell ind (already "discounted" in previous)
##  + volume stored by cell ind reduced by coefficient Cred          
          table_output[BR_down,"Vst_UP"]<- table_output[BR_down,"Vst_UP"] + table_output[ind,"Vst_UP"] +Cred*(table_output[ind,"VF_ST"]+table_output[ind,"VF_RD"]) # version 3-4
#         table_output[BR_down,"VF_ST"]<- table_output[BR_down,"VF_ST"] -abs(table_output[ind,"VF_ST"])-table_output[ind,"VF_RD"]  # version 2
#         table_output[BR_down,"VF_ST"]<- table_output[BR_down,"VF_ST"] -abs(table_output[ind,"VF_ST"]) # version 1
        }
      }
    }
  }
 
## write in output to check results  
#  fileSuf<- paste0(flag_WL,"_",adst,"_RP",list_RP_unif[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
#  write.csv(table_output,file = paste0(workDir,"/tables/table_storage_output_",fileSuf,".csv"),row.names=FALSE)      

  ###################################
  print("calculate costs")

  Vst_ep <- array(0, dim = c(num_ep,2), dimnames = list(c(1:num_ep),c("VF_ST","VF_RD")))

  for (nc in 1:lBR){
## retrieve EP linked to each branch  
    ind_br<-which(NETfl[,"EU_IDR"]==table_branches[nc,"EU_IDR"])
## distribute stored volumes among all EPs
    Vst_ep[ind_br,"VF_ST"]<-table_output[nc,"VF_ST"]/length(ind_br)
## assume that VF_RD can be stored using standard cost in branches with order 1
    if(table_branches[nc,"ORDER_"]==1){
      Vst_ep[ind_br,"VF_ST"]<- Vst_ep[ind_br,"VF_ST"]+table_output[nc,"VF_RD"]/length(ind_br)
    } else {
      Vst_ep[ind_br,"VF_RD"]<- table_output[nc,"VF_RD"]/length(ind_br)
    }    
  }
  
  ## calculate construction and maintenance costs
  UC<-c(1:num_ep)*0 
  for (np in 1:num_ep) {
    if(table_EP_nuts[np,2]>0){
      ind <-which(table_cost[,"NUTS_CODE"]==table_EP_nuts[np,2])
      UC[np]<-table_cost[ind,"UC_st_areas"]
    }
  }
  ## assume that residual flood volumes have to be stored at higher costs (Cinc)  
  AnnCost <- (Vst_ep[,"VF_ST"]+Cinc*Vst_ep[,"VF_RD"])*UC/(ComYear-2020)  # not discounted annual cost (total cost / period for construction)
  MaCost <- (Vst_ep[,"VF_ST"]+Cinc*Vst_ep[,"VF_RD"])*UC*MaintCoef  

  ## calculate annual costs - discounting done later    
  for (ny in (2020-2015+1):(ComYear-2015+1)) {
#    cost_f[,ny] <- AnnCost/((1+DisRate)^(ny-1))  
    cost_f[,ny] <- AnnCost    
  }
  for (ny in (ComYear-2015+2):(RefYear-2015+1)) {
#    cost_f[,ny] <- MaCost/((1+DisRate)^(ny-1))   
    cost_f[,ny] <- MaCost        
  }  
  
  print("cost calculation completed")    
#############################
  
  print("calculate benefits")  
  ## open files of EP results for the adaptation scenario  
  fileSuf<- paste0(flag_WL,"_",adst,"_RP",list_RP_unif[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
  ead_ad <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf,sep=""),row.name=1)
  eapa_ad <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf,sep=""),row.name=1)
  ## open files of EP results for the climate-only scenario
  if(flag_MS=="sta"){
    fileSuf2<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
    ead_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf2,sep=""),row.name=1)
    eapa_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf2,sep=""),row.name=1)
  }else if(flag_MS=="tra"){  
    ## CHECK THIS BEFORE USING IT!  
    ead_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_clim/ead_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv",sep=""),row.name=1)
    eapa_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_clim/eapa_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv",sep=""),row.name=1)    
  }
  
  ind1=which(colnames(ead_ad)=="X2015")
  ind2=which(colnames(ead_ad)=="X2100")
  ben_ep <- ead_na-ead_ad
  ben_ep[ben_ep<0]<-0
  ben_ep <- ben_ep[,ind1:ind2]  
  
  ## calculate also reduction in population affected
  popR_ep <- eapa_na-eapa_ad
  popR_ep[popR_ep<0]<-0
  popR_ep <- popR_ep[,ind1:ind2]
  
  ## interpolate
  ben_f <- cost_f*0
  popR_f<- cost_f*0
  ben_f[,1]<-ben_ep[,1]
  popR_f[,1]<-popR_ep[,1]
  for (ii in 1:(dim(ben_ep)[2]-1)){
    ben_f[,ii*5+1]<-ben_ep[,ii+1]
    popR_f[,ii*5+1]<-popR_ep[,ii+1]
  }
  for (jj in 1:(dim(ben_ep)[2]-1)){
    ben_f[,jj*5-3]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])/5
    ben_f[,jj*5-2]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*2/5
    ben_f[,jj*5-1]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*3/5
    ben_f[,jj*5-0]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*4/5  
    popR_f[,jj*5-3]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])/5
    popR_f[,jj*5-2]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*2/5
    popR_f[,jj*5-1]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*3/5
    popR_f[,jj*5-0]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*4/5  
  }
  
  ## adjust cost calculation: in points where there is no benefit, we assume no measure is taken
  ## commented because storage areas have a basin-scale distribution on costs and benefits
#  cost_f[ben_f==0]<-0
#  ben_f[cost_f==0]<-0  
  
  print("apply discount rates")
  for (ny in 1:(RefYear-2015+1)) {
    for (np in 1:nNU) {
      ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
      DisRate<-table_countries[np,"DisRate"]
      cost_f_dis[ind_nuts,ny] <- cost_f[ind_nuts,ny]/((1+DisRate)^(ny-1))  
      ben_f_dis[ind_nuts,ny] <- ben_f[ind_nuts,ny]/((1+DisRate)^(ny-1))        
    }
  }  
  # NPV_f <- ben_f-cost_f  

  ## calculate costs- benefits (discounted) and impact reduction in 2100 (undiscounted)
  cost_tot_ep[,iad]<-rowSums(cost_f_dis)
  cost_ud_ep[,iad]<-rowSums(cost_f)
  ben_tot_ep[,iad]<-rowSums(ben_f_dis)  
  #  popR_tot_ep[,iad]<-rowSums(popR_f) 
  NPV_ep[,iad]<-ben_tot_ep[,iad]-cost_tot_ep[,iad]
  damR_2100_ep[,iad]<-ben_f[,nRYC]  
  popR_2100_ep[,iad]<-popR_f[,nRYC]
  
  print("aggregate costs and benefits at NUTS2 scale")
  for (np in 1:nNU) {
    ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
    modBen[np,iad] <-sum(ben_tot_ep[ind_nuts,iad])
    modCost[np,iad] <-sum(cost_tot_ep[ind_nuts,iad])
    modCost_ud[np,iad] <-sum(cost_ud_ep[ind_nuts,iad])      
    modPopR_2100[np,iad] <-sum(popR_2100_ep[ind_nuts,iad])  
    modDamR_2100[np,iad] <-sum(damR_2100_ep[ind_nuts,iad])         
    modNPV[np,iad] <- sum(NPV_ep[ind_nuts,iad])  
  }    
  ## not used anymore
  #   print("aggregate costs and benefits at NUTS2 scale")
  #   for (ny in 1:nRYC){ 
  #     for (np in 1:nNU) {
  #       ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
  #       ben_f_nuts2[np,ny] <- sum(ben_f[ind_nuts,ny]) 
  #       cost_f_nuts2[np,ny]<- sum(cost_f[ind_nuts,ny]) 
  #       popR_f_nuts2[np,ny]<- sum(popR_f[ind_nuts,ny]) 
  #       NPV_f_nuts2[np,ny] <- sum(NPV_f[ind_nuts,ny])       
  #     }
  #   }
  #   modBen[,iad] <-rowSums(ben_f_nuts2)
  #   modCost[,iad] <-rowSums(cost_f_nuts2)   
  #   modPop[,iad] <-rowSums(popR_f_nuts2)  
  #   modNPV[,iad] <- rowSums(NPV_f_nuts2)    
  
  print(paste0("adaptation scenario ",iad," of ",length(list_RP_unif)," completed"))                    
  #### END LOOP ON ADAPTATION SCENARIOS  
}  
  
print('---')
print('Cost-benefit analysis COMPLETED')

if(flag_MS=="sta"){
  fileSuf<- paste0("ST_",adst,"_",flag_WL,"_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
} else if(flag_MS=="tra"){
  fileSuf<- paste0("ST_",adst,"_transient_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
}

dir.create( paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par))
# dir.create( paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par))

## check EP costs
# write.csv(cost_tot_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_tot_EP_",fileSuf),row.names=TRUE)
# write.csv(damR_2100_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Ben_2100_EP_",fileSuf),row.names=TRUE)
# write.csv(popR_2100_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/PopR_2100_EP_",fileSuf),row.names=TRUE)
# write.csv(wl_inc_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/wl_inc_EP_",fileSuf),row.names=TRUE)

write.csv(modNPV,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/NPV_NUTS2_",fileSuf),row.names=TRUE)    
write.csv(modCost,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modBen,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Ben_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modPopR_2100,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/PopR_2100_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modDamR_2100,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/DamR_2100_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modCost_ud,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_ud_NUTS2_",fileSuf),row.names=TRUE)

#####################
print('---')
print('script ended')

