## 
## calcolate costs of relocation and flood proofing measures
## CALCULATE  FOR A RANGE OF ADAPTATION SCENARIOS

#### SET RUN MODE
run<-"cluster" ## choose whether to run on (local) or (cluster)
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean


## load libraries
library(sp,lib.loc="/home/dottofr/Rlib")     	# needed to load rdgdal library
Sys.sleep(1)
library(raster,lib.loc="/home/dottofr/Rlib")
Sys.sleep(1)
library(rgdal,lib.loc="/home/dottofr/Rlib") 	# use to read .map raster format
#### 

if(run=='cluster'){
  args <- commandArgs(TRUE)
  modelRun <- as.character(args[1])     # model run to analyse
  mainDir <- as.character(args[2])     # directory of config file    
} else if(run=='local'){
  modelRun <- "projection_dis_rcp85_CLMcom-CCLM4-8-17_BC_CNRM-CERFACS-CNRM-CM5_wuConst_statistics_ptId.nc" 
}
flag_WL <- as.character(args[3]) # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
prot_par<- as.character(args[4]) # level of flood protection - 050: 50% of estimated average protection standards
dam_par <- as.character(args[5]) # level of flood damage -q95
cost_par<- as.character(args[6]) # level of adaptation costs -q95
flag_AD <- as.character(args[7]) # consider flood proofing of buildings "FP" or relocation "RE"
#### SET PARAMETERS USED FOR THE SENSISTIVITY ANALYSIS (for sensitivity analysis)
# flag_WL <- "3C" # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
# prot_par<- "050"/"100"/"150"/
# dam<- "q05"/"avg"/"q95" quantiles estimated from reported costs
# cost<- "q05"/"avg"/"q95"
# flag_AD <- "FP" ## consider flood proofing of buildings "FP" or relocation "RE" 

workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))

print(paste("- run analysis for WL ",flag_WL," - ensemble ",ensemble,sep=""))
#############################
## INPUT PARAMETERS

# flag_dis="yes" # use annual discount rate or not
# DisRate <- 0.0 # annual discount rate  -replaced by country values
CostCoef <- 0.8 # correction coefficient for costs (e.g. using <1 for FP we assume scale economy)  
if(flag_AD=='RE'){
  MaintCoef <- 0.0 # annual maintenance costs as fraction of construction costs    
}else if(flag_AD=='FP'){
  MaintCoef <- 0.01 # annual maintenance costs as fraction of construction costs
}  
ComYear <- 2050 # year of reference for completion of adaptation measures 
RefYear <- 2100 # year of reference for design of adaptation measures    
ComYear_fp <- 2060 # year of reference for starting reinstallation of flood proofing measures 
RefYear_fp <- 2090 # year of reference for ending reinstallation of flood proofing measures 
## bldcoef now replaced with actual shares of inf-agr damage for each EP
#BldCoef <- 0.9 # correction coefficient for damage reduction (we assume that part of damages (e.g. agriculture and infrastructure) are not reduced)  
StoCoef <- 1.5 # correction coefficient for average number of storeys (used for relocation)

##################
print('read input data')

## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)
## PERIOD: pf: present rescaled to future (p->f) fp: future actualized to present (f->p)

# num_ep <-dim(dis_f)[1] # number of EFAS network cells 
num_ep = 80553

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
nRP<-length(RP_classes)
nRY<-length(RY_classes)

#########
## READ INPUT TABLES

## read present level of protection in each point (RP in years)
prot_RP_p_table <- read.csv(paste(workDir,"/tables/protection_levels/table_EP_protLev_2018_fp",prot_par,".csv", sep=""))

## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]

## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))

## read unit cost table
 table_cost <- read.csv(paste(workDir,"/tables/table_unit_costs_nuts2_",cost_par,".csv", sep=""))
## read builtup area flooded
bld_p <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_Builtup_EP_ghsl.csv", sep=""))
## read agricultural and infrastructure damage shares
damInf_p <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_Cost_EP_v4_inf.csv", sep=""))
damAgr_p <- read.csv(paste(workDir,"/tables/EAI_Europe/RP_impacts_EP/RP_Cost_EP_v4_agr.csv", sep=""))

#########
## FORMAT AND INITIALIZE  DATA

nRYT<-max(RY_classes)-min(RY_classes)+1
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
} 
nRYC<-max(RY_classes)-2015+1
## read model run set up and corresponding warming levels
ind_MR<- which(WL_table[,"ModelRun"]==modelRun)
Pr_mod<-WL_table[ind_MR,"Model"]
Pr_rcp<-WL_table[ind_MR,"RCP"]
Pr_wu<-WL_table[ind_MR,"WU"]
ind_base<-WL_table[ind_MR,"Base"]-min(RY_classes)+1
ind_WL15<-WL_table[ind_MR,"X1.5C"]-min(RY_classes)+1
ind_WL20<-WL_table[ind_MR,"X2C"]-min(RY_classes)+1
ind_WL30<-WL_table[ind_MR,"X3C"]-min(RY_classes)+1

## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

## format protection level tables and assign synthetic values (if requested)
prot_RP_p <- prot_RP_p_table[,2] 
rm(prot_RP_p_table)

## set reduction values
# RP_unif<-c("2", "10", "15", "20", "30", "50", "70", "100", "150", "200", "350", "500", "700", "1000", "1500", "2000")
SC_classes<-seq(from = 0.05, to = 1, by = 0.05)

## initialize cost - benefit annual tables 
UC<-prot_RP_p*0
cost_f_dis <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes)))) 
ben_f_dis <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes))))  
cost_f <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes)))) 
ben_f <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes))))  

modCost <- array(0, dim = c(nNU, length(SC_classes)), dimnames = list(list_nuts2 ,SC_classes)) 
modBen <- array(0, dim = c(nNU, length(SC_classes)), dimnames = list(list_nuts2 ,SC_classes))
modNPV <- array(0, dim = c(nNU, length(SC_classes)), dimnames = list(list_nuts2 ,SC_classes))   
modCost_ud <- array(0, dim = c(nNU, length(SC_classes)), dimnames = list(list_nuts2 ,SC_classes)) 
modPopR_2100 <- array(0, dim = c(nNU, length(SC_classes)), dimnames = list(list_nuts2 ,SC_classes))   
modDamR_2100 <- array(0, dim = c(nNU, length(SC_classes)), dimnames = list(list_nuts2 ,SC_classes))   

# modCost_countries <- array(0, dim = c(nCY, length(SC_classes)), dimnames = list(list_countries,SC_classes)) 
# modBen_countries <- array(0, dim = c(nCY, length(SC_classes)), dimnames = list(list_countries,SC_classes))
# modNPV_countries <- array(0, dim = c(nCY, length(SC_classes)), dimnames = list(list_countries,SC_classes))     
# modPop_countries <- array(0, dim = c(nCY, length(SC_classes)), dimnames = list(list_countries,SC_classes))      

# cost_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# ben_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# popR_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# NPV_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))

## CHECK: SAVE EP values to CHECK total costs etc  
cost_tot_ep <- array(0, dim = c(num_ep,length(SC_classes)), dimnames = list(c(1:num_ep) ,SC_classes))
ben_tot_ep <- array(0, dim = c(num_ep,length(SC_classes)), dimnames = list(c(1:num_ep) ,SC_classes))
NPV_ep <- array(0, dim = c(num_ep,length(SC_classes)), dimnames = list(c(1:num_ep) ,SC_classes))
cost_ud_ep <- array(0, dim = c(num_ep,length(SC_classes)), dimnames = list(c(1:num_ep) ,SC_classes))
damR_2100_ep <- array(0, dim = c(num_ep,length(SC_classes)), dimnames = list(c(1:num_ep) ,SC_classes))
popR_2100_ep <- array(0, dim = c(num_ep,length(SC_classes)), dimnames = list(c(1:num_ep) ,SC_classes))

print('input data reading done')

#############################
print("read data for each model run")  

## open files of EP results 
if(flag_MS=="sta"){
  fileSuf<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
  ead_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf,sep=""),row.name=1)
  eapa_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf,sep=""),row.name=1)
#  eab_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eab_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eab_ep_",fileSuf,sep=""),row.name=1)
}else if(flag_MS=="tra"){
  fileSuf<- paste0(Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
  ead_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_clim_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf,sep=""),row.name=1)
  eapa_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_clim_fp",prot_par,"/eapa_ep_",fileSuf,sep=""),row.name=1)    
#  eab_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eab_ep_clim_fp",prot_par,"/eab_ep_",fileSuf,sep=""),row.name=1)      
}

################
## test to compare EAD / EAB
# ead_nuts2 <- array(0, dim = c(nNU,dim(ead_na)[2]), dimnames = list(list_nuts2 ,RY_classes))
# eab_nuts2 <- array(0, dim = c(nNU,dim(ead_na)[2]), dimnames = list(list_nuts2 ,RY_classes))
# print("aggregate costs and benefits at NUTS2 scale")
# for (ny in 1:dim(ead_na)[2]){ 
#   for (np in 1:nNU) {
#     ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
#     ead_nuts2[np,ny] <- sum(ead_na[ind_nuts,ny]) 
#     eab_nuts2[np,ny]<- sum(eab_na[ind_nuts,ny]) 
#   }
# }  
# ratio_nu <-ead_nuts2/eab_nuts2
################

# CALCULATE ALL POSSIBLE ADAPTATION SCENARIOS (going from 5% to 100% reduction, either by relocation or flood proofing)
# loop starts here
 for(iad in 1:length(SC_classes)){
# for(iad in 8:10){

   ## WRITE IN NOTES
   print("---")
   print(paste0("adaptation scenario ",iad," of ",length(SC_classes)," -  damage reduction factor ", SC_classes[iad]))
  #############################     
 ## NOTE: we assume to adapt all exposed buildings (as by 2100) under stabilized climate by 2050 
 ## dual implementation of measures (linear increase in impacts reduction between 2020 and 2050)
  ben_ep<-ead_na*0
  popR_ep<-eapa_na*0
 ## NOTE: we assume that population can all be relocated/protected
  BldCoef= (1-damInf_p[,"median_inf_tot"])*(1-damAgr_p[,"median_agr_tot"])
  BldCoef[BldCoef<0]=0
  ben_ep[,"X2050"]  <- ead_na[,"X2050"]*SC_classes[iad]*BldCoef
  popR_ep[,"X2050"] <- eapa_na[,"X2050"]*SC_classes[iad]

  nQD <-which(RY_classes=="2050")-which(RY_classes=="2020")
  for(it in which(RY_classes=="2025"):which(RY_classes=="2045")){
    ben_ep[,it] <- ead_na[,"X2050"]*SC_classes[iad]*BldCoef*(it-which(RY_classes=="2020"))/nQD
    popR_ep[,it] <- eapa_na[,"X2050"]*SC_classes[iad]*(it-which(RY_classes=="2020"))/nQD
  }
  ## assign reduction factor to all years after 2050
  for(it in which(RY_classes=="2055"):which(RY_classes=="2100")){
    ben_ep[,it]  <- ead_na[,it]*SC_classes[iad]*BldCoef
    popR_ep[,it]  <- eapa_na[,it]*SC_classes[iad]    
  }
  
   ind1=which(colnames(ead_na)=="X2015")
   ind2=which(colnames(ead_na)=="X2100")
   
   popR_ep <- popR_ep[,ind1:ind2]
   ben_ep <- ben_ep[,ind1:ind2]  
   
  print("benefit calculation completed")  
  #############################  
  ## the builtup area from GHSL data includes only buildings and not infrastructure networks, so it's ok to assume that
  ## costs are attributed to all builtup 
  UC<-c(1:num_ep*0)
  ## calculate construction and maintenance costs
  for (np in 1:num_ep) {
    if(table_EP_nuts[np,2]>0){
      ind <-which(table_cost[,"NUTS_CODE"]==table_EP_nuts[np,2])
      if(flag_AD=="FP"){
        UC[np]<-table_cost[ind,"UC_flood_p"]*SC_classes[iad]*CostCoef # cost refer to hypothetical 100% damage reduction
      } else if(flag_AD=="RE"){
        UC[np]<-table_cost[ind,"UC_relocation"]*SC_classes[iad]*StoCoef # cost refer to hypothetical 100% damage reduction
      }
    }
  }
## costs based on the builtup area exposed to 500y flood
   AnnCost <- bld_p[,"RP500y"]*UC/(ComYear-2020)  # not discounted annual cost (total cost / period for construction)
   MaCost <- bld_p[,"RP500y"]*UC*MaintCoef  
## previous version - costs based on the expected builtup area exposed   
#   AnnCost <- eab_na[,"X2100"]*UC/(ComYear-2020)  # not discounted annual cost (total cost / period for construction)
#   MaCost <- eab_na[,"X2100"]*UC*MaintCoef  
   for (ny in (2020-2015+1):(ComYear-2015+1)) {
     cost_f[,ny] <- AnnCost     
   }
   if(flag_AD=="FP"){  # we assume reinstallation in 2060-2090     
     for (ny in (ComYear-2015+2):(ComYear_fp-2015+1)) {
       cost_f[,ny] <- MaCost        
     }  
     for (ny in (ComYear_fp-2015+2):(RefYear_fp-2015+1)) {
       cost_f[,ny] <- AnnCost         
     } 
     for (ny in (RefYear_fp-2015+2):(RefYear-2015+1)) {
       cost_f[,ny] <- MaCost        
     } 
   } else if(flag_AD=="RE"){
     for (ny in (ComYear-2015+2):(RefYear-2015+1)) {
       cost_f[,ny] <- MaCost        
     } 
   }

  print("cost calculation completed")  
  #############################  
  
  ## interpolate
  ben_f <- cost_f*0
  popR_f<- cost_f*0
  ben_f[,1]<-ben_ep[,1]
  popR_f[,1]<-popR_ep[,1]
  for (ii in 1:(dim(ben_ep)[2]-1)){
    ben_f[,ii*5+1]<-ben_ep[,ii+1]
    popR_f[,ii*5+1]<-popR_ep[,ii+1]
  }
  for (jj in 1:(dim(ben_ep)[2]-1)){
    ben_f[,jj*5-3]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])/5
    ben_f[,jj*5-2]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*2/5
    ben_f[,jj*5-1]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*3/5
    ben_f[,jj*5-0]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*4/5  
    popR_f[,jj*5-3]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])/5
    popR_f[,jj*5-2]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*2/5
    popR_f[,jj*5-1]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*3/5
    popR_f[,jj*5-0]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*4/5  
  }
   
   ## adjust cost calculation: where there is no benefit, we assume no measure is taken and viceversa
   ## NOTE: it may happen that for some EPs there is a benefit but no costs, because
   ## builtup impacts for RP500y are referred to another EP nearby
   cost_f[ben_f==0]<-0
#   ben_f[cost_f==0]<-0   
   
  print("apply discount rates")
  for (ny in 1:(RefYear-2015+1)) {
     for (np in 1:nNU) {
       ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
       DisRate<-table_countries[np,"DisRate"]
       cost_f_dis[ind_nuts,ny] <- cost_f[ind_nuts,ny]/((1+DisRate)^(ny-1))  
       ben_f_dis[ind_nuts,ny] <- ben_f[ind_nuts,ny]/((1+DisRate)^(ny-1))         
     }
  }  
## calculate costs- benefits (discounted) and total cost-impact reduction in 2100 (undiscounted)
  cost_tot_ep[,iad]<-rowSums(cost_f_dis)
  cost_ud_ep[,iad]<-rowSums(cost_f)
  ben_tot_ep[,iad]<-rowSums(ben_f_dis)  
#  popR_tot_ep[,iad]<-rowSums(popR_f) 
  NPV_ep[,iad]<-ben_tot_ep[,iad]-cost_tot_ep[,iad]
   
## keep costs and benefits only in EPs with NPV>0
  for(ne in 1:num_ep){
    if(NPV_ep[ne,iad]<0){
      cost_tot_ep[ne,iad]<-0
      ben_tot_ep[ne,iad]<-0  
      cost_ud_ep[ne,iad]<-0      
    } else {
      damR_2100_ep[ne,iad]<-ben_f[ne,nRYC]  
      popR_2100_ep[ne,iad]<-popR_f[ne,nRYC]
    }
  }
   
  print("aggregate costs and benefits at NUTS2 scale")

#   for (ny in 1:nRYC){ 
     for (np in 1:nNU) {
       ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
#       ben_f_nuts2[np,ny] <- sum(ben_f_dis[ind_nuts,ny]) 
#       cost_f_nuts2[np,ny]<- sum(cost_f_dis[ind_nuts,ny]) 
#       popR_f_nuts2[np,ny]<- sum(popR_f[ind_nuts,ny]) 
#       NPV_f_nuts2[np,ny] <- ben_f_nuts2[np,ny]-cost_f_nuts2[np,ny]   
   modBen[np,iad] <-sum(ben_tot_ep[ind_nuts,iad])
   modCost[np,iad] <-sum(cost_tot_ep[ind_nuts,iad])  
   modCost_ud[np,iad] <-sum(cost_ud_ep[ind_nuts,iad])         
   modPopR_2100[np,iad] <-sum(popR_2100_ep[ind_nuts,iad])  
   modDamR_2100[np,iad] <-sum(damR_2100_ep[ind_nuts,iad])         
   modNPV[np,iad] <- sum(NPV_ep[ind_nuts,iad])  
     }
#   }  
  
  ### aggregate at country level and save results
#   for (nc in 1:nCY) {
#     ind_cy<-which(table_countries[,1]==list_countries[nc])
#     modNPV_countries[nc,iad]<-sum(modNPV[ind_cy,iad])
#     modCost_countries[nc,iad]<-sum(modCost[ind_cy,iad])
#     modBen_countries[nc,iad]<-sum(modBen[ind_cy,iad])     
#     modPop_countries[nc,iad]<-sum(modPop[ind_cy,iad])      
#   }
  
  print(paste0("adaptation scenario ",iad," of ",length(SC_classes)," completed"))                    
  
}  #### END LOOP ON ADAPTATION SCENARIOS  
print('---')
print('Cost-benefit analysis COMPLETED')

## calculate costs-benefits ratio at EP scale
BCR_ep<-ben_tot_ep/cost_tot_ep
BCR_ep[is.na(BCR_ep)]<-0
#BCR_ep[is.inf(BCR_ep)]<-10

if(flag_MS=="sta"){
  fileSuf<- paste0(flag_AD,"_",adst,"_",flag_WL,"_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
} else if(flag_MS=="tra"){
  fileSuf<- paste0(flag_AD,"_",adst,"_transient_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
}
dir.create( paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par))
# dir.create( paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par))

## NOTE: we assume that uncertainty on damage cost is not affecting popution exposed
# write.csv(cost_tot_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_tot_EP_",fileSuf),row.names=TRUE)
# write.csv(ben_tot_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Ben_tot_EP_",fileSuf),row.names=TRUE)
# write.csv(damR_2100_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/damR_2100_EP_",fileSuf),row.names=TRUE)
# write.csv(popR_2100_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/PopR_2100_EP_",fileSuf),row.names=TRUE)
# write.csv(BCR_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/BCR_EP_",fileSuf),row.names=TRUE)

write.csv(modNPV,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/NPV_NUTS2_",fileSuf),row.names=TRUE)    
write.csv(modCost,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modBen,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Ben_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modPopR_2100,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/PopR_2100_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modDamR_2100,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/DamR_2100_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modCost_ud,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_ud_NUTS2_",fileSuf),row.names=TRUE)

# write.csv(modNPV_countries,file = paste0(workDir,"/tables/EAI_Europe/adaptation_countries/NPV_countries_",fileSuf),row.names=TRUE)    
# write.csv(modCost_countries,file = paste0(workDir,"/tables/EAI_Europe/adaptation_countries/Cost_countries_",fileSuf),row.names=TRUE)
# write.csv(modBen_countries,file = paste0(workDir,"/tables/EAI_Europe/adaptation_countries/Ben_countries_",fileSuf),row.names=TRUE)
# write.csv(modPop_countries,file = paste0(workDir,"/tables/EAI_Europe/adaptation_countries/Pop_countries_",fileSuf),row.names=TRUE)

#####################
print('script ended')
