
## CALCULATE COSTS AND BENEFITS FOR DIKE HEIGHTENING FOR A RANGE OF ADAPTATION SCENARIOS

#### SET RUN MODE
run<-"cluster" ## choose whether to run on (local) or (cluster)
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
MR_data='Rdata'
coef="no"  ## elaborate coefficient data
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean

## load libraries
library(sp,lib.loc="/home/dottofr/Rlib")     	# needed to load rdgdal library
Sys.sleep(1)
library(raster,lib.loc="/home/dottofr/Rlib")
Sys.sleep(1)
library(rgdal,lib.loc="/home/dottofr/Rlib") 	# use to read .map raster format
#### 

## INPUT DIRECTORIES
if(run=='cluster'){
  args <- commandArgs(TRUE)
  modelRun <- as.character(args[1])     # model run to analyse
  mainDir <- as.character(args[2])     # directory of config file    
} else if(run=='local'){
  scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")
  modelRun <- "projection_dis_rcp85_CLMcom-CCLM4-8-17_BC_CNRM-CERFACS-CNRM-CM5_wuConst_statistics_ptId.nc" 
}
flag_WL <- as.character(args[3]) # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
prot_par<- as.character(args[4]) # level of flood protection - 050: 50% of estimated average protection standards
dam_par <- as.character(args[5]) # level of flood damage -q95
cost_par<- as.character(args[6]) # level of costs -q95
scriptDir <- as.character(args[7])     # directory of scripts and configuration files 
# flag_WL <- "3C" # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
# prot_par<- "050"/"100"/"150"/
# dam<- "q05"/"avg"/"q95"
# cost<- "q25"/"avg"/"q75"

workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))
workDir2<-workDir # backup variable to be used if Rdata has different workDir 
scriptDir2<-scriptDir # backup variable to be used if Rdata has different workDir 

print(paste("- run analysis for WL ",flag_WL," - ensemble ",ensemble,sep=""))

#######
## INPUT PARAMETERS FOR COST-BENEFIT CALCULATIONS

# flag_dis="no" # use annual discount rate or not
# DisRate <- 0.0 # annual discount rate  -replaced by country values
CostCoef <- 1 # correction coefficient for costs   
MaintCoef <- 0.01 # annual maintenance costs as fraction of construction costs    
ComYear <- 2050 # year of reference for completion of adaptation measures 
RefYear <- 2100 # year of reference for design of adaptation measures  

#######################################
print('-----')
print("handle netcdf maps and read discharge data")
#######################################

run2<-run  ## correction in case value in RData is different
MR_data2<-MR_data ## correction in case value in RData is different
load(file = paste0(workDir,"/Rdata/",modelRun,".RData"))
## restore correct paths in case Rdata has different workDir 
workDir <-workDir2
scriptDir<-scriptDir2 

## interpolate RP 200y using 150y and 250y
dis_150_250 <- dis_f[,,c(14,15,15)]  
dis_150_250[,,3] <- (dis_150_250[,,1]+dis_150_250[,,2])/2    
## replace 250y wit interpolated 200y
dis_f[,,15] <- dis_150_250[,,3]
rm(dis_150_250)

## keep only used RPs (2,10,20,50,100,200,500,1000)
dis_f<-dis_f[,,c(2,7,9,11,13,15,17,19)]
## delete years not used
dis_p <- dis_f[,8,] # year 2015 as reference to match discharge and impacts
dis_f[is.na(dis_f)] <- 0
dis_p[is.na(dis_p)] <- 0

print('reading of discharge data done')

print("read ensemble averages of discharge")
load(file = paste0(workDir,"/Rdata/ens_discharges_",ensemble,".RData"))
if(ensemble=="median"){
  dis_ens <- dis_ens_median
}

print('read parameters of EP rating curves')
if(coef=="no"){
   load(file = paste0(workDir,"/Rdata/wl_dis_fit.RData"))
} else if(coef=='yes') {
    wl_dis <- nc_open(paste0(workDir,"/maps/wl_dis_fit.nc"))
    rm(wl_dis)
    ## Get a list of the NetCDF's R attributes:
    attributes(wl_dis)$names
    ## USE THIS TO EXTRACT INFO about the nc file
    print(paste("The file has",wl_dis$nvars,"variables,",wl_dis$ndims,"dimensions and",wl_dis$natts,"NetCDF attributes"))
    
    ## Get a list of the nc variable names.
    attributes(wl_dis$var)$names
    ## Retrieve a matrix of the variable using the ncvar_get function:
    coef_A <- ncvar_get(wl_dis, attributes(wl_dis$var)$names[1])
    coef_C <- ncvar_get(wl_dis, attributes(wl_dis$var)$names[2])
    coef_X <- ncvar_get(wl_dis, attributes(wl_dis$var)$names[4])
    coef_Y <- ncvar_get(wl_dis, attributes(wl_dis$var)$names[5])   
    ## activate to save data in R workspace
#     obj_list<-c("wl_dis","coef_A","coef_C","coef_X","coef_Y")
#     save(list = obj_list, file = paste0(workDir,"/Rdata/wl_dis_fit.RData"))    
}
## NOTE: values in netcdf file are already sorted as EP list
table_coef <- cbind(coef_A,coef_C,coef_X,coef_Y)
colnames(table_coef) <- c("coef_A","coef_C","coef_X","coef_Y")  
workDir <-workDir2 # restore correct paths in case Rdata has different workDir

#############################
print('read input data')

##########
## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)
## PERIOD: pf: present rescaled to future (p->f) fp: future actualized to present (f->p)

num_ep <-dim(dis_f)[1] # number of EFAS network cells 

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
nRP<-length(RP_classes)
nRY<-length(RY_classes)

#########
## READ INPUT TABLES

## read unit cost table
# table_cost <- read.csv(paste(workDir,"/tables/table_unit_costs_nuts2_avg.csv", sep=""))
table_cost <- read.csv(paste(workDir,"/tables/table_unit_costs_nuts2_",cost_par,".csv", sep=""))

## read present level of protection in each point (RP in years)
prot_RP_p_table <- read.csv(paste(workDir,"/tables/protection_levels/table_EP_protLev_2018_fp",prot_par,".csv", sep=""))

## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))

## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))

## branch lenght table
table_EP_FP <- read.csv(paste(workDir,"/tables/table_EP_FP_branches_v2.csv", sep=""))  
table_length <- array(0, dim = c(num_ep,2),dimnames = list(c(1:num_ep),c("EP","length")) )   
## calculate the channel length related to each EP using flood points  
## note that we take a minimum length of25m, to avoid problems
for (i in 1:num_ep){
  table_length[i,"EP"]<-i
  table_length[i,"length"]<-max(length(which(table_EP_FP[,"EP"]==i))*5000,25)
}
# table_length[,2]=5000  # we assume a generic branch length of 8km for each EP 
 
#########
## FORMAT AND INITIALIZE  DATA

dimnames(dis_f)<-list(c(1:num_ep),lRY,RP_classes)
dimnames(dis_p)<-list(c(1:num_ep),RP_classes)
nRYT<-max(RY_classes)-min(RY_classes)+1
if(nRYT==120) {
   nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
} 
nRYC<-max(RY_classes)-2015+1
#######   

## read model run set up and corresponding warming levels
ind_MR<- which(WL_table[,"ModelRun"]==modelRun)
Pr_mod<-WL_table[ind_MR,"Model"]
Pr_rcp<-WL_table[ind_MR,"RCP"]
Pr_wu<-WL_table[ind_MR,"WU"]
ind_base<-WL_table[ind_MR,"Base"]-min(RY_classes)+1
ind_WL15<-WL_table[ind_MR,"X1.5C"]-min(RY_classes)+1
ind_WL20<-WL_table[ind_MR,"X2C"]-min(RY_classes)+1
ind_WL30<-WL_table[ind_MR,"X3C"]-min(RY_classes)+1

## initialize water level arrays
wl_p <- array(0, dim = c(num_ep, nRP),dimnames =list(c(1:num_ep),RP_classes))
wl_ens <- array(0, dim = dim(dis_ens),dimnames =dimnames(dis_ens))
   
## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

## format protection level tables and assign synthetic values (if requested)
prot_RP_p <- prot_RP_p_table[,2] 
rm(prot_RP_p_table)
prot_wlP <- prot_RP_p*0

## initialize return periods for future protection standards
list_RP_unif<-dimnames(dis_ens)[[3]]

## initialize cost - benefit annual tables 
cost_f_dis <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes)))) 
ben_f_dis <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes))))  
cost_f <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes)))) 
ben_f <- array(0, dim = c(num_ep, nRYC), dimnames = list(c(1:num_ep),c(2015:max(RY_classes))))  

modCost <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif)) 
modBen <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))
modNPV <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))   
modPopR_2100 <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))   
modDamR_2100 <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))   
modCost_ud<- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif)) 
modWLI <- array(0, dim = c(nNU, length(list_RP_unif)), dimnames = list(list_nuts2 ,list_RP_unif))
# cost_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# ben_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# popR_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))
# NPV_f_nuts2 <- array(0, dim = c(nNU,nRYC), dimnames = list(list_nuts2 ,c(2015:max(RY_classes))))

## SAVE EP values to CHECK total costs etc  
cost_tot_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
ben_tot_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
cost_ud_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
NPV_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
damR_2100_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
popR_2100_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))
wl_inc_ep <- array(0, dim = c(num_ep,length(list_RP_unif)), dimnames = list(c(1:num_ep) ,list_RP_unif))

print('input data reading done')
#############################

## in case of stabilized climate scenarios, replace discharge values with WL-averaged value 
## CHECK: possibly not needed anymore??
if(flag_MS=="sta"){
  print(paste0("use stabilized climate for a ",flag_WL," warming scenario"))
  dis_m1 <- paste("dis_ens_", Pr_mod,"_",Pr_rcp, sep = "")
  dis_mod <- array(0, dim = dim(dis_ens), dimnames = dimnames(dis_ens)) 
  dis_mod <- get(dis_m1)
  dis_mod <-dis_mod[,,c(1,2,4,6,8,10,12,14)]  # keep only uses RPs
  
  if(flag_WL=="1.5C"){ 
    ind<- min(which(RY_classes> WL_table[ind_MR,"X1.5C"])) 
  } else if(flag_WL=="2C"){
    ind<- min(which(RY_classes> WL_table[ind_MR,"X2C"]))
  } else if(flag_WL=="3C" & Pr_rcp=="rcp85"){
    ind<- min(which(RY_classes> WL_table[ind_MR,"X3C"]))  
  }
  for(ii in ind:nRY){
    dis_f[,ii,]<-dis_mod[,flag_WL,]
  }
} else{
  print("use transient climate, based on complete model run")
}

#############################
print("---")
print("calculate water levels using WL-dis fitting coefficients")

#wl_f <- table_coef[,2]*dis_f^table_coef[,1]
wl_p <- table_coef[,2]*dis_p^table_coef[,1]
wl_ens <- table_coef[,2]*dis_ens^table_coef[,1] 
   
## test
# tm<-cbind(c(2,3,3),c(4,4,2))
# tc<-c(0.5,1,2)
# ta<-c(1,1,2)   
# tx<-tc*tm^ta
   
## CHECK wl results
# write.csv(wl_ens[,flag_WL,],file = paste0(workDir,"/tables/EAI_Europe/test_06b/wl_ens_",flag_WL,".csv"),row.names=TRUE)

#######################   

## CALCULATE ALL POSSIBLE ADAPTATION SCENARIOS (RP(2100) range from 20y to 2000y)
## loop starts here
 for(iad in 1:length(list_RP_unif)){
# for(iad in 6:12){
  
  ## WRITE IN NOTES
  print("---")
  print(paste0("adaptation scenario ",iad," of ",length(list_RP_unif)," -  RP ", list_RP_unif[iad], " years"))
  
#############################
  print("-> extract present protection values (RP) in terms of WL")
  
  ## general formula for extrapolating linearly
  # Q[RP2>prot_RP_p>RP1] <- Qbase[RP1]+ (Qbase[RP2]-Qbase[RP1])*(prot_RP_p[RP2>prot_RP_p>RP1] - RP1)/(RP2-RP1) 
  
  mask_P<- which(prot_RP_p[]>2 & prot_RP_p[]<=10)
  prot_wlP[mask_P] <-wl_p[mask_P,1] + (wl_p[mask_P,2]-wl_p[mask_P,1])*(prot_RP_p[prot_RP_p[]>2 & prot_RP_p[]<=10] - 2)/(10-2)
  mask_P<- which(prot_RP_p[]>10 & prot_RP_p[]<=20)
  prot_wlP[mask_P] <-wl_p[mask_P,2] + (wl_p[mask_P,3]-wl_p[mask_P,2])*(prot_RP_p[prot_RP_p[]>10 & prot_RP_p[]<=20] - 10)/(20-10)
  mask_P<- which(prot_RP_p[]>20 & prot_RP_p[]<=50)
  prot_wlP[mask_P] <-wl_p[mask_P,3] + (wl_p[mask_P,4]-wl_p[mask_P,3])*(prot_RP_p[prot_RP_p[]>20 & prot_RP_p[]<=50] - 20)/(50-20)
  mask_P<- which(prot_RP_p[]>50 & prot_RP_p[]<=100)
  prot_wlP[mask_P] <-wl_p[mask_P,4] + (wl_p[mask_P,5]-wl_p[mask_P,4])*(prot_RP_p[prot_RP_p[]>50 & prot_RP_p[]<=100] - 50)/(100-50)
  mask_P<- which(prot_RP_p[]>100 & prot_RP_p[]<=200)
  prot_wlP[mask_P] <-wl_p[mask_P,5] + (wl_p[mask_P,6]-wl_p[mask_P,5])*(prot_RP_p[prot_RP_p[]>100 & prot_RP_p[]<=200] - 100)/(200-100)
  mask_P<- which(prot_RP_p[]>200 & prot_RP_p[]<=500)
  prot_wlP[mask_P] <-wl_p[mask_P,6] + (wl_p[mask_P,7]-wl_p[mask_P,6])*(prot_RP_p[prot_RP_p[]>200 & prot_RP_p[]<=500] - 200)/(500-200)
  mask_P<- which(prot_RP_p[]>500 & prot_RP_p[]<=1000)
  prot_wlP[mask_P] <-wl_p[mask_P,7] + (wl_p[mask_P,8]-wl_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>500 & prot_RP_p[]<=1000] - 500)/(1000-500)
  mask_P<- which(prot_RP_p[]>1000)
  prot_wlP[mask_P] <-wl_p[mask_P,8] + (wl_p[mask_P,8]-wl_p[mask_P,7])*(prot_RP_p[prot_RP_p[]>1000] - 1000)/(1000-500)  
  rm(mask_P)

#   CHECK RESULTS
# fileSuf<- paste0("DI_",adst,"_",flag_WL,"_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
# write.csv(prot_wlP,file = paste0(workDir,"/tables/EAI_Europe/test_06b/prot_wlP_",fileSuf),row.names=TRUE)

#############################
  print("calculate dike heightening and costs ")
  
  wl_inc <- wl_ens[,flag_WL,list_RP_unif[iad]]-prot_wlP[]
  wl_inc[wl_inc<0]<-0  # exclude areas where there is no increase in protection standard
  wl_inc[is.na(wl_inc)]<-0
  wl_inc[wl_inc>10]<-10  # put a limiter to dike increase, to avoid unrealistic values

  ## CHECK wl_inc results
  # wl_inc_ep[,iad]<-wl_inc
  
  UC<-wl_inc*0
## calculate construction and maintenance costs
  for (np in 1:num_ep) {
    if(table_EP_nuts[np,2]>0){
      ind <-which(table_cost[,"NUTS_CODE"]==table_EP_nuts[np,2])
      UC[np]<-table_cost[ind,"UC_dike_h"]
    }
  }
  AnnCost <- wl_inc*UC*table_length[,"length"]/(ComYear-2020)  # not discounted annual cost (total cost / period for construction)
  MaCost <- wl_inc*UC*table_length[,"length"]*MaintCoef   
## the x2 multiplier is because dikes are built on both sides (we assume equal height)  
#  AnnCost <- wl_inc*2*UC*table_length[,"length"]/(ComYear-2020) 
#  MaCost <- wl_inc*2*UC*table_length[,"length"]*MaintCoef  

  ## calculate annual costs - discounting done later    
  for (ny in (2020-2015+1):(ComYear-2015+1)) {
    #    cost_f[,ny] <- AnnCost/((1+DisRate)^(ny-1))  
    cost_f[,ny] <- AnnCost    
  }
  for (ny in (ComYear-2015+2):(RefYear-2015+1)) {
    #    cost_f[,ny] <- MaCost/((1+DisRate)^(ny-1))   
    cost_f[,ny] <- MaCost        
  }  

  #############################
## CHECK OF WL_INC VALUES
  ## read table of XY coordinates for EFAS points
#   table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))
#   val_check<-cbind(table_EP_XY[,2:3],wl_inc)
#   grid_check<- rasterFromXYZ(val_check, res=c(5000,5000), crs=NA, digits=5)
#   writeGDAL(as(grid_check,'SpatialGridDataFrame'), fname=paste0(workDir,"/maps/grid_check_wl_inc_100y_",flag_WL,".tif"),band, drivername='GTiff',type="Float32",mvFlag=-9999,options=c("INTERLEAVE=PIXEL","COMPRESS=LZW"))                                                                                                           

## CHECK OF Q100 VALUES  
#    table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))
#    Q100D=dis_ens[,flag_WL,"100"]-dis_ens[,"base","100"] 
#    val_check<-cbind(table_EP_XY[,2:3],Q100D)
#    grid_check<- rasterFromXYZ(val_check, res=c(5000,5000), crs=NA, digits=5)
#    writeGDAL(as(grid_check,'SpatialGridDataFrame'), fname=paste0(workDir,"/maps/grid_check_Q100y_D_",flag_WL,".tif"),band, drivername='GTiff',type="Float32",mvFlag=-9999,options=c("INTERLEAVE=PIXEL","COMPRESS=LZW"))                                                                                                            
  
  print("cost calculation completed")  
  
  #############################
  
  print("calculate benefits")  
  
  ## open files of EP results for the adaptation scenario  
  fileSuf<- paste0(flag_WL,"_",adst,"_RP",list_RP_unif[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
  ead_ad <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf,sep=""),row.name=1)
  eapa_ad <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf,sep=""),row.name=1)
  ## open files of EP results for the climate-only scenario
  if(flag_MS=="sta"){
    fileSuf2<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
    ead_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf2,sep=""),row.name=1)
    eapa_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf2,sep=""),row.name=1)
  }else if(flag_MS=="tra"){  
    ## CHECK THIS BEFORE USING IT!  
    ead_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/ead_ep_clim/ead_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv",sep=""),row.name=1)
    eapa_na <-read.csv(file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_clim/eapa_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv",sep=""),row.name=1)    
  }  
  
  ind1=which(colnames(ead_ad)=="X2015")
  ind2=which(colnames(ead_ad)=="X2100")
  ben_ep <- ead_na-ead_ad
  ben_ep[ben_ep<0]<-0
  ben_ep <- ben_ep[,ind1:ind2]  
  
  ## calculate also reduction in population affected
  popR_ep <- eapa_na-eapa_ad
  popR_ep[popR_ep<0]<-0
  popR_ep <- popR_ep[,ind1:ind2]

  ## interpolate
  ben_f <- cost_f*0
  popR_f<- cost_f*0
  ben_f[,1]<-ben_ep[,1]
  popR_f[,1]<-popR_ep[,1]
  for (ii in 1:(dim(ben_ep)[2]-1)){
    ben_f[,ii*5+1]<-ben_ep[,ii+1]
    popR_f[,ii*5+1]<-popR_ep[,ii+1]
  }
  for (jj in 1:(dim(ben_ep)[2]-1)){
    ben_f[,jj*5-3]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])/5
    ben_f[,jj*5-2]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*2/5
    ben_f[,jj*5-1]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*3/5
    ben_f[,jj*5-0]<-ben_ep[,jj]+(ben_ep[,jj+1]-ben_ep[,jj])*4/5  
    popR_f[,jj*5-3]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])/5
    popR_f[,jj*5-2]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*2/5
    popR_f[,jj*5-1]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*3/5
    popR_f[,jj*5-0]<-popR_ep[,jj]+(popR_ep[,jj+1]-popR_ep[,jj])*4/5  
  }
 
  ## adjust cost calculation: where there is no benefit, we assume no measure is taken and viceversa
  cost_f[ben_f==0]<-0
  ben_f[cost_f==0]<-0
  
  print("apply discount rates")
  for (ny in 1:(RefYear-2015+1)) {
    for (np in 1:nNU) {
      ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
      DisRate<-table_countries[np,"DisRate"]
      cost_f_dis[ind_nuts,ny] <- cost_f[ind_nuts,ny]/((1+DisRate)^(ny-1))  
      ben_f_dis[ind_nuts,ny] <- ben_f[ind_nuts,ny]/((1+DisRate)^(ny-1))        
    }
  }  
  # NPV_f <- ben_f-cost_f

  ## calculate costs- benefits (discounted) and impact reduction in 2100 (undiscounted)
  cost_tot_ep[,iad]<-rowSums(cost_f_dis)
  cost_ud_ep[,iad]<-rowSums(cost_f)
  ben_tot_ep[,iad]<-rowSums(ben_f_dis)  
  #  popR_tot_ep[,iad]<-rowSums(popR_f) 
  NPV_ep[,iad]<-ben_tot_ep[,iad]-cost_tot_ep[,iad]
  damR_2100_ep[,iad]<-ben_f[,nRYC]  
  popR_2100_ep[,iad]<-popR_f[,nRYC]
  
  print("aggregate costs and benefits at NUTS2 scale")
  for (np in 1:nNU) {
    ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
    modBen[np,iad] <-sum(ben_tot_ep[ind_nuts,iad])
    modCost[np,iad] <-sum(cost_tot_ep[ind_nuts,iad])
    modCost_ud[np,iad] <-sum(cost_ud_ep[ind_nuts,iad])      
    modPopR_2100[np,iad] <-sum(popR_2100_ep[ind_nuts,iad])  
    modDamR_2100[np,iad] <-sum(damR_2100_ep[ind_nuts,iad])         
    modNPV[np,iad] <- sum(NPV_ep[ind_nuts,iad])  
  ## calculate median increase in dike heights at NUTS2 scale (more robust than mean)
    modWLI[np,iad]<-median(wl_inc[ind_nuts]) 
  }    
## not used anymore
#   print("aggregate costs and benefits at NUTS2 scale")
#   for (ny in 1:nRYC){ 
#     for (np in 1:nNU) {
#       ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
#       ben_f_nuts2[np,ny] <- sum(ben_f[ind_nuts,ny]) 
#       cost_f_nuts2[np,ny]<- sum(cost_f[ind_nuts,ny]) 
#       popR_f_nuts2[np,ny]<- sum(popR_f[ind_nuts,ny]) 
#       NPV_f_nuts2[np,ny] <- sum(NPV_f[ind_nuts,ny])       
#     }
#   }
#   modBen[,iad] <-rowSums(ben_f_nuts2)
#   modCost[,iad] <-rowSums(cost_f_nuts2)   
#   modPop[,iad] <-rowSums(popR_f_nuts2)  
#   modNPV[,iad] <- rowSums(NPV_f_nuts2)  

  print(paste0("adaptation scenario ",iad," of ",length(list_RP_unif)," completed"))                    
#### END LOOP ON ADAPTATION SCENARIOS  
}  
print('---')
print('Cost-benefit analysis COMPLETED')

if(flag_MS=="sta"){
  fileSuf<- paste0("DI_",adst,"_",flag_WL,"_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
} else if(flag_MS=="tra"){
  fileSuf<- paste0("DI_",adst,"_transient_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
}

dir.create( paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par))
# dir.create( paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par))

## check EP costs
# write.csv(cost_tot_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_tot_EP_",fileSuf),row.names=TRUE)
# write.csv(damR_2100_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Ben_2100_EP_",fileSuf),row.names=TRUE)
# write.csv(popR_2100_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/PopR_2100_EP_",fileSuf),row.names=TRUE)
# write.csv(wl_inc_ep,file = paste0(workDir,"/tables/EAI_Europe/adaptation_ep_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/wl_inc_EP_",fileSuf),row.names=TRUE)

write.csv(modNPV,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/NPV_NUTS2_",fileSuf),row.names=TRUE)    
write.csv(modCost,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modBen,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Ben_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modPopR_2100,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/PopR_2100_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modDamR_2100,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/DamR_2100_NUTS2_",fileSuf),row.names=TRUE)
write.csv(modCost_ud,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_ud_NUTS2_",fileSuf),row.names=TRUE)
# write.csv(modWLI,file = paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/WLI_NUTS2_",fileSuf),row.names=TRUE)

#####################
print('---')
print('script ended')

