## 
### produce summary tables at country scale for present-day vulnerability scenario 
## aggregate results for the baseline and warming levels
### 

#### SET RUN MODE
run<-"local" # local-cluster
flag_MR<-"clim" # "clim" "2050" "2100" "JRC" "SSP1" "SSP3" "SSP5"
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
flag_WL <- "2C" ## stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean
#######

if(run=='cluster'){
  args <- commandArgs(TRUE)
  scriptDir <- as.character(args[1])     # directory of config file
  mainDir <- as.character(args[2])     # directory of config file   
} else if(run=='local'){
  scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}
workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))


if(run=='cluster'){
  args <- commandArgs(TRUE)
  scriptDir <- as.character(args[1])     # directory of config file
} else if(run=='local'){
#  scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/Damage")   
  scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}
 workDir <-paste0(scriptDir,"/PESETA_IV_data")
# workDir <-scriptDir

print('input data')

##########
## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)

num_ep <-80553 # number of EFAS network cells 

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
# RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
RY_classes<- c(2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)

nRP<-length(RP_classes)
nRY<-length(RY_classes)
startY=min(RY_classes)
nRYT<-max(RY_classes)-min(RY_classes)+1
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
  startY=startY-1
} 

#########
## READ INPUT TABLES
## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))
## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]
modNames<-WL_table[,2]  
mod1=0

## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

#############################
print('---')
print("- aggregate results for baseline and warming levels ")
                                            

###############
## PART 1:
## aggregate results at country scale for baseline and warming levels
## NOTE: this is the correct procedure where we EXTRACT DATA FOR CENTRAL YEARS
## annual values are based on 30-y average from EVA (extreme value analysis) and are
## therefore representative of periods

ead_nuts2_all_cy <- array(0, dim = c(nNU,4,22),dimnames = list(list_nuts2,c("Base","1.5C","2C","3C"),WL_table[,3]))
eapa_nuts2_all_cy <- array(0, dim = c(nNU,4,22),dimnames = list(list_nuts2,c("Base","1.5C","2C","3C"),WL_table[,3]))

for(ii in 1:lMod){
  Pr_mod<-WL_table[ii,2]
  Pr_rcp<-WL_table[ii,3]
  Pr_wu<-WL_table[ii,4]
  table_modRun <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_",flag_MR,"_fp050/ead_nuts2_",flag_MR,"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1)       	
 if(flag_MR=="clim"){
	indb<-paste0("X",WL_table[ii,"Base"])
 	ead_nuts2_all_cy[,1,ii]<- table_modRun[,indb]
  }
  ind1<-paste0("X",WL_table[ii,"X1.5C"])	
  ead_nuts2_all_cy[,2,ii]<- table_modRun[,ind1]
  ind2<-paste0("X",WL_table[ii,"X2C"])
  ead_nuts2_all_cy[,3,ii]<- table_modRun[,ind2]
  if(Pr_rcp=="rcp85"){
	ind3<-paste0("X",WL_table[ii,"X3C"])
  	ead_nuts2_all_cy[,4,ii]<- table_modRun[,ind3]
  }
}
for(ii in 1:lMod){
  Pr_mod<-WL_table[ii,2]
  Pr_rcp<-WL_table[ii,3]
  Pr_wu<-WL_table[ii,4]
  table_modRun <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_AA_",flag_MR,"_fp050/eapa_nuts2_",flag_MR,"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1)       	
 if(flag_MR=="clim"){
	indb<-paste0("X",WL_table[ii,"Base"])
 	eapa_nuts2_all_cy[,1,ii]<- table_modRun[,indb]
  }
  ind1<-paste0("X",WL_table[ii,"X1.5C"])	
  eapa_nuts2_all_cy[,2,ii]<- table_modRun[,ind1]
  ind2<-paste0("X",WL_table[ii,"X2C"])
  eapa_nuts2_all_cy[,3,ii]<- table_modRun[,ind2]
  if(Pr_rcp=="rcp85"){
	ind3<-paste0("X",WL_table[ii,"X3C"])
  	eapa_nuts2_all_cy[,4,ii]<- table_modRun[,ind3]
  }
}

## write results for all the ensemble and aggregate impacts at country scale 
clim_tw=c("base","1.5C","2C","3C")
for (i in 1:length(clim_tw)) {
#  write.csv(ead_nuts2_all_cy[,i,], file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_",clim_tw[i],"_fp050.csv"),row.names=TRUE)
#  write.csv(eapa_nuts2_all_cy[,i,], file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_",clim_tw[i],"_fp050.csv"),row.names=TRUE) 
  ead_countries_all_cy= aggregate(ead_nuts2_all_cy[,i,], by=list(table_countries[,"COUNTRY"]), sum)
  eapa_countries_all_cy= aggregate(eapa_nuts2_all_cy[,i,], by=list(table_countries[,"COUNTRY"]), sum)  
  rownames(ead_countries_all_cy)=ead_countries_all_cy[,"Group.1"]
  rownames(eapa_countries_all_cy)=eapa_countries_all_cy[,"Group.1"]
  write.csv(eapa_countries_all_cy, file = paste0(workDir,"/tables/EAI_Europe/eapa_countries_",clim_tw[i],"_fp050.csv"),row.names=TRUE)
  write.csv(ead_countries_all_cy, file = paste0(workDir,"/tables/EAI_Europe/ead_countries_",clim_tw[i],"_fp050.csv"),row.names=TRUE)  
}



## calculate mean-median values over the ensemble
## NOTE: now not used

# ead_countries_avg_cy<-array(0, dim = c(nCY,4),dimnames = list(list_countries,c("Base","1.5C","2C","3C")))
#  eapa_countries_avg_cy<-array(0, dim = c(nCY,4),dimnames = list(list_countries,c("Base","1.5C","2C","3C")))
# 
#  ead_nuts2_avg_cy <- apply(ead_nuts2_all_cy,c(1,2),mean,na.rm=TRUE)
#  eapa_nuts2_avg_cy <- apply(eapa_nuts2_all_cy,c(1,2),mean,na.rm=TRUE)
# #  ead_nuts2_avg_cy <- apply(ead_nuts2_all_cy,c(1,2),median,na.rm=TRUE)
# #  eapa_nuts2_avg_cy <- apply(eapa_nuts2_all_cy,c(1,2),median,na.rm=TRUE)
#  ead_nuts2_avg_cy[is.na(ead_nuts2_avg_cy)]<-0
#  eapa_nuts2_avg_cy[is.na(eapa_nuts2_avg_cy)]<-0
#  for (nc in 1:nCY) {
#    ind_cnty<-which(table_countries[,1]==list_countries[nc])
#    for (nn in 1:4) {
#      ead_countries_avg_cy[nc,nn]<-sum(ead_nuts2_avg_cy[ind_cnty,nn])
#      eapa_countries_avg_cy[nc,nn]<-sum(eapa_nuts2_avg_cy[ind_cnty,nn])
#    }
#  }
#  write.csv(ead_nuts2_avg_cy, file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_avg_cy_",flag_MR,".csv"),row.names=TRUE)
#  write.csv(ead_countries_avg_cy, file = paste0(workDir,"/tables/EAI_Europe/ead_countries_avg_cy_",flag_MR,".csv"),row.names=TRUE)
#  write.csv(eapa_nuts2_avg_cy, file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_avg_cy_",flag_MR,".csv"),row.names=TRUE)
#  write.csv(eapa_countries_avg_cy, file = paste0(workDir,"/tables/EAI_Europe/eapa_countries_avg_cy_",flag_MR,".csv"),row.names=TRUE)


###############
## Part 2: aggregate data for the period 2071-2100
## to modify: now calculating mean over period 2071-2100 use instead 2085 values as reference!!!

# ead_nuts2_AA_aggr <- array(0, dim = c(nNU,30,11))
# im=1
# for(ii in 1:lMod){
#   Pr_mod<-WL_table[ii,2]
#   Pr_rcp<-WL_table[ii,3]
#   Pr_wu<-WL_table[ii,4]
#   table_modRun <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_nuts2_AA_",flag_MR,"/ead_nuts2_",flag_MR,"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1)       
#   if(Pr_rcp=="rcp85"){
#       t1<-as.matrix(table_modRun[,62:91])
#       ead_nuts2_AA_aggr[,,im]<-t1
#       im=im+1
#   }
# }
## NOTE: calculating mean is not correct, annual values are already a 30-y average
## replace next line with using 2085 values as reference!!!
 # NU_avg_models<-apply(ead_nuts2_AA_aggr,c(1,3),mean)
 # 
 # NU_avg_ensemble<-rowMeans(NU_avg_models,na.rm=TRUE)
 # CY_avg_ensemble<-array(0, dim = c(nCY,1))
 # for (nc in 1:nCY) {
 #  ind_cnty<-which(table_countries[,1]==list_countries[nc])
 #  CY_avg_ensemble[nc]<-sum(NU_avg_ensemble[ind_cnty]) # doesn't work of there are NA values
 # }
 # 
 # write.csv(CY_avg_ensemble, file = paste0(workDir,"/tables/EAI_Europe/CY_avg_ensemble.csv"),row.names=TRUE)
 # write.csv(NU_avg_models, file = paste0(workDir,"/tables/EAI_Europe/NU_avg_models.csv"),row.names=TRUE)


print('Aggregation COMPLETED')



