## 
## aggregate EP results to NUTS2 and contry scale and produce summary tables 
## for year 2100 for adaptation scenarios (i.e. varying protection standards)

#### SET RUN MODE
run<-"cluster" # local-cluster
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean

#######

if(run=='cluster'){
  args <- commandArgs(TRUE)
  mainDir <- as.character(args[1])     # directory of config file   
} else if(run=='local'){
  mainDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}

prot_par<- as.character(args[2]) # level of flood protection 
dam_par <- as.character(args[3]) # level of flood damage
# flag_WL <- "3C" # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
# prot_par<- "050"/"100"/"150"/ #050: 50% of estimated average protection standards
# dam_par<- "q05"/"avg"/"q95"

workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))

##########
print('prepare input data')
## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)

num_ep <-80553 # number of EFAS network cells 
# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
# RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
RY_classes<- c(2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)

nRP<-length(RP_classes)
nRY<-length(RY_classes)
startY=min(RY_classes)
nRYT<-max(RY_classes)-min(RY_classes)+1
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
  startY=startY-1
} 

#########
## READ INPUT TABLES
## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))
## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]
modNames<-WL_table[,2]  

## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

###
## no of design return periods of hazard-reduction measures (dikes or retention areas) 
  nSC<-16
  SC_classes<-c(2,10,15,20,30,50,70,100,150,200,350,500,700,1000,1500,2000)

#############################
print('---')
print("- aggregate results at NUTS2 level for adaptation scenarios ")
#############################
## Calculate aggregate nuts2 and country values for 2100
## no-adaptation scenario
##############
## initialize variables and create output directories

ead_nuts2_AA_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))
eapa_nuts2_AA_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))                                                 
ead_f_nuts2 <- array(0, dim = c(nNU,1))
eapa_f_nuts2 <- array(0, dim = c(nNU,1))
ead_p_nuts2 <- array(0, dim = c(nNU,1))
eapa_p_nuts2 <- array(0, dim = c(nNU,1))
WLS <-c("1.5C","2C","3C")

ead_nuts2_15 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
ead_nuts2_20 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
ead_nuts2_30 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
eapa_nuts2_15 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
eapa_nuts2_20 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
eapa_nuts2_30 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes)) 

ead_nuts2_WL_aggr <- array(0, dim = c(nNU,lMod,4))
eapa_nuts2_WL_aggr <- array(0, dim = c(nNU,lMod,4))

dir.create( paste0(workDir,"/tables/EAI_Europe/results_risk_scenarios_fp",prot_par,"_ed",dam_par))
dir.create( paste0(workDir,"/tables/EAI_Europe/ead_nuts2_WL_2100_adapt_models_fp",prot_par,"_ed",dam_par))
dir.create( paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_WL_2100_adapt_models_fp",prot_par))  
#######
## run calculations

print("---")

##  REACTIVATE WHEN ALL DATA ARE THERE

 for (jj in 1:3){
#  jj=2
    flag_WL<-WLS[jj]
    for(ii in 1:lMod){
      Pr_mod<-WL_table[ii,2]
      Pr_rcp<-WL_table[ii,3]
      Pr_wu<-WL_table[ii,4]
      if(flag_WL == "3C" & Pr_rcp =='rcp45'){  # rcp45 does not produce results for 3C 
        print("no results for 3C-rcp 4.5" )
      }else{
        if(flag_MS=="sta"){
          fileSuf<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
          ead_na <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf),row.name=1)
          eapa_na <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf),row.name=1)
        } else if(flag_MS=="tra"){
          fileSuf<- paste0("_transient_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
          ead_na <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_transient_PIV_fp",prot_par,"/ead_ep_",fileSuf),row.name=1)
          eapa_na <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_transient_PIV_fp",prot_par,"/eapa_ep_",fileSuf),row.name=1)
        }        
        ## aggregate  values for 2100 at nuts2 scale 
        for (np in 1:nNU) {
          #   #    print(paste("NUTS2 ",list_nuts2[np],sep=""))
          ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])       
          ead_f_nuts2[np,1]<-sum(ead_na[ind_nuts,dim(ead_na)[2]]) 
          eapa_f_nuts2[np,1]<-sum(eapa_na[ind_nuts,dim(eapa_na)[2]])  
        }
        ead_nuts2_WL_aggr[,ii,jj+1]<-ead_f_nuts2[,1]
        eapa_nuts2_WL_aggr[,ii,jj+1]<-eapa_f_nuts2[,1]
        ## aggregate  values for present day (1995) at nuts2 scale        
        if(flag_WL == "2C"){  # present day results are the same for all WLs 
          for (np in 1:nNU) {
            ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])       
            ead_p_nuts2[np,1]<-sum(ead_na[ind_nuts,"X1995"]) 
            eapa_p_nuts2[np,1]<-sum(eapa_na[ind_nuts,"X1995"]) 
          }
          ead_nuts2_WL_aggr[,ii,1]<-ead_p_nuts2[,1]
          eapa_nuts2_WL_aggr[,ii,1]<-eapa_p_nuts2[,1]
        }
      }
    }
 }
## calculate ensemble averages (could be improved using apply)
## damage
  ead_nuts2_WL_aggr[ead_nuts2_WL_aggr==0]<-NA
  WL1=ead_nuts2_WL_aggr[,,1]
  WL2=ead_nuts2_WL_aggr[,,2]
  WL3=ead_nuts2_WL_aggr[,,3]
  WL4=ead_nuts2_WL_aggr[,,4]
  ## average on all models
  ead_nuts2_AA_WL[,1] <- rowMeans(WL1,na.rm=TRUE)
  ead_nuts2_AA_WL[,2] <- rowMeans(WL2,na.rm=TRUE)
  ead_nuts2_AA_WL[,3] <- rowMeans(WL3,na.rm=TRUE)
  ead_nuts2_AA_WL[,4] <- rowMeans(WL4,na.rm=TRUE)
  ead_nuts2_AA_WL[is.na(ead_nuts2_AA_WL)]<-0
  ## population
  eapa_nuts2_WL_aggr[eapa_nuts2_WL_aggr==0]<-NA
  WL1=eapa_nuts2_WL_aggr[,,1]
  WL2=eapa_nuts2_WL_aggr[,,2]
  WL3=eapa_nuts2_WL_aggr[,,3]
  WL4=eapa_nuts2_WL_aggr[,,4]
  ## average
  eapa_nuts2_AA_WL[,1] <- rowMeans(WL1,na.rm=TRUE)
  eapa_nuts2_AA_WL[,2] <- rowMeans(WL2,na.rm=TRUE)
  eapa_nuts2_AA_WL[,3] <- rowMeans(WL3,na.rm=TRUE)
  eapa_nuts2_AA_WL[,4] <- rowMeans(WL4,na.rm=TRUE)
  eapa_nuts2_AA_WL[is.na(eapa_nuts2_AA_WL)]<-0

print("aggregate WL impacts at country scale and write results ")

ead_countries_AA_WL <- array(0, dim = c(nCY,4),dimnames = list(list_countries,c("all-base","all-1.5C","all-2C","all-3C")))                                                 
eapa_countries_AA_WL <- array(0, dim = c(nCY,4),dimnames = list(list_countries,c("all-base","all-1.5C","all-2C","all-3C")))                                                 
for(nn in 1:4) {
  for (nc in 1:nCY) {
    ind_cnty<-which(table_countries[,1]==list_countries[nc])
    ead_countries_AA_WL[nc,nn]<-sum(ead_nuts2_AA_WL[ind_cnty,nn]) # doesn't work of there are NA values
    eapa_countries_AA_WL[nc,nn]<-sum(eapa_nuts2_AA_WL[ind_cnty,nn]) # doesn't work of there are NA values
  }
}
write.csv(ead_nuts2_AA_WL, file = paste0(workDir,"/tables/EAI_Europe/results_risk_scenarios_fp",prot_par,"_ed",dam_par,"/ead_nuts2_na_WL_2100.csv"),row.names=TRUE)
write.csv(eapa_nuts2_AA_WL,file = paste0(workDir,"/tables/EAI_Europe/results_risk_scenarios_fp",prot_par,"_ed",dam_par,"/eapa_nuts2_na_WL_2100.csv"),row.names=TRUE)
write.csv(ead_countries_AA_WL, file = paste0(workDir,"/tables/EAI_Europe/results_risk_scenarios_fp",prot_par,"_ed",dam_par,"/ead_countries_na_WL_2100.csv"),row.names=TRUE)
write.csv(eapa_countries_AA_WL,file = paste0(workDir,"/tables/EAI_Europe/results_risk_scenarios_fp",prot_par,"_ed",dam_par,"/eapa_countries_na_WL_2100.csv"),row.names=TRUE)

######
print('---')
## WRITE separate result for each model
for(ii in 1:lMod){
  Pr_mod<-WL_table[ii,2]
  Pr_rcp<-WL_table[ii,3]
  Pr_wu<-WL_table[ii,4]
  if(flag_MS=="sta"){
    fileSuf2<- paste0("nuts2_na_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
  } else if(flag_MS=="tra"){
    fileSuf2<- paste0("nuts2_transient_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
  }
  ead_mod=ead_nuts2_WL_aggr[,ii,]
  eapa_mod=eapa_nuts2_WL_aggr[,ii,]  
  
## add NUTS2 and WLs to rows and columns  
  dimnames(ead_mod) <- list(list_nuts2,c("base",WLS)) 
  dimnames(eapa_mod) <- list(list_nuts2,c("base",WLS)) 
  write.csv(ead_mod, file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_WL_2100_adapt_models_fp",prot_par,"_ed",dam_par,"/ead_",fileSuf2),row.names=TRUE)
  write.csv(eapa_mod, file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_WL_2100_adapt_models_fp",prot_par,"/eapa_",fileSuf2),row.names=TRUE)  
}


##############
## Calculate aggregate nuts2 and country values for 2100
## all adaptation scenarios
##############

## initialize variables and create output directories
ead_nuts2_AA_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))
eapa_nuts2_AA_WL <- array(0, dim = c(nNU,4),dimnames = list(list_nuts2,c("base","1.5C","2C","3C")))                                                 
ead_countries_AA_WL <- array(0, dim = c(nCY,4),dimnames = list(list_countries,c("all-base","all-1.5C","all-2C","all-3C")))                                                 
eapa_countries_AA_WL <- array(0, dim = c(nCY,4),dimnames = list(list_countries,c("all-base","all-1.5C","all-2C","all-3C")))                                                 
ead_f_nuts2 <- array(0, dim = c(nNU,1))
eapa_f_nuts2 <- array(0, dim = c(nNU,1))
WLS <-c("1.5C","2C","3C")

ead_nuts2_15 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
ead_nuts2_20 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
ead_nuts2_30 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
eapa_nuts2_15 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
eapa_nuts2_20 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))                                                 
eapa_nuts2_30 <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes)) 
  
ead_nuts2_mod <- array(0, dim = c(nNU,lMod,4,length(SC_classes)),dimnames = list(list_nuts2,1:lMod,c("base",WLS),SC_classes)) 
eapa_nuts2_mod <- array(0, dim = c(nNU,lMod,4,length(SC_classes)),dimnames = list(list_nuts2,1:lMod,c("base",WLS),SC_classes)) 

dir.create( paste0(workDir,"/tables/EAI_Europe/ead_WL_2100_adapt_ens_fp",prot_par,"_ed",dam_par))
dir.create( paste0(workDir,"/tables/EAI_Europe/eapa_WL_2100_adapt_ens_fp",prot_par))

##############
## run calculations
for(iad in 1:length(SC_classes)){
# for(iad in 1:2){ 
  ead_nuts2_WL_aggr <- array(0, dim = c(nNU,lMod,4))
  eapa_nuts2_WL_aggr <- array(0, dim = c(nNU,lMod,4))
  print("---")
  print(paste0("adaptation scenario ",iad," of ",length(SC_classes)," -  RP ", SC_classes[iad], " years"))
  
  ## REACTIVATE    
 for (jj in 1:3){
#  jj=2
    flag_WL<-WLS[jj] 
    for(ii in 1:lMod){
      Pr_mod<-WL_table[ii,2]
      Pr_rcp<-WL_table[ii,3]
      Pr_wu<-WL_table[ii,4]
      if(flag_WL == "3C" & Pr_rcp =='rcp45'){  # rcp45 does not produce results for 3C 
        print("no results for 3C-rcp 4.5" )
      }else{
         if(flag_MS=="sta"){
          fileSuf<- paste0(flag_WL,"_",adst,"_RP",SC_classes[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
          ead_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",dam_par,"/ead_ep_",fileSuf),row.name=1)
          eapa_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf),row.name=1)
        } else if(flag_MS=="tra"){
          fileSuf<- paste0("_transient_",adst,"_RP",SC_classes[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
          ead_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_transient_PIV_fp",prot_par,"/ead_ep_",fileSuf),row.name=1)
          eapa_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_transient_PIV_fp",prot_par,"/eapa_ep_",fileSuf),row.name=1)
        }
  ## aggregate values for 2100 at nuts2 scale 
        for (np in 1:nNU) {
          #   #    print(paste("NUTS2 ",list_nuts2[np],sep=""))
          ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])       
          ead_f_nuts2[np,1]<-sum(ead_f[ind_nuts,dim(ead_f)[2]]) 
          eapa_f_nuts2[np,1]<-sum(eapa_f[ind_nuts,dim(eapa_f)[2]]) 
        }
        ead_nuts2_WL_aggr[,ii,jj+1]<-ead_f_nuts2[,1]
        eapa_nuts2_WL_aggr[,ii,jj+1]<-eapa_f_nuts2[,1]
      }
    }
  }
## write model results separately  
  ead_nuts2_mod[,,,iad]=ead_nuts2_WL_aggr
  eapa_nuts2_mod[,,,iad]=eapa_nuts2_WL_aggr
## calculate ensemble averages (could be improved using apply)  
## damage
  ead_nuts2_WL_aggr[ead_nuts2_WL_aggr==0]<-NA
  WL1=ead_nuts2_WL_aggr[,,1]
  WL2=ead_nuts2_WL_aggr[,,2]
  WL3=ead_nuts2_WL_aggr[,,3]
  WL4=ead_nuts2_WL_aggr[,,4]
  ## average on all models
  ead_nuts2_AA_WL[,1] <- rowMeans(WL1,na.rm=TRUE)
  ead_nuts2_AA_WL[,2] <- rowMeans(WL2,na.rm=TRUE)
  ead_nuts2_AA_WL[,3] <- rowMeans(WL3,na.rm=TRUE)
  ead_nuts2_AA_WL[,4] <- rowMeans(WL4,na.rm=TRUE)
  ead_nuts2_AA_WL[is.na(ead_nuts2_AA_WL)]<-0
  ## population
  eapa_nuts2_WL_aggr[eapa_nuts2_WL_aggr==0]<-NA
  WL1=eapa_nuts2_WL_aggr[,,1]
  WL2=eapa_nuts2_WL_aggr[,,2]
  WL3=eapa_nuts2_WL_aggr[,,3]
  WL4=eapa_nuts2_WL_aggr[,,4]
  ## average
  eapa_nuts2_AA_WL[,1] <- rowMeans(WL1,na.rm=TRUE)
  eapa_nuts2_AA_WL[,2] <- rowMeans(WL2,na.rm=TRUE)
  eapa_nuts2_AA_WL[,3] <- rowMeans(WL3,na.rm=TRUE)
  eapa_nuts2_AA_WL[,4] <- rowMeans(WL4,na.rm=TRUE)
  eapa_nuts2_AA_WL[is.na(eapa_nuts2_AA_WL)]<-0

  fileSuf<- paste0("nuts2_WL_2100_RP_",SC_classes[iad],"_2100.csv")
  write.csv(ead_nuts2_AA_WL, file = paste0(workDir,"/tables/EAI_Europe/ead_WL_2100_adapt_ens_fp",prot_par,"_ed",dam_par,"/ead_",fileSuf),row.names=TRUE)
  write.csv(eapa_nuts2_AA_WL,file = paste0(workDir,"/tables/EAI_Europe/eapa_WL_2100_adapt_ens_fp",prot_par,"/eapa_",fileSuf),row.names=TRUE)
# write table with all nuts2 aggregated results  
  ead_nuts2_15[,iad] <- ead_nuts2_AA_WL[,2]                                                 
  ead_nuts2_20[,iad] <- ead_nuts2_AA_WL[,3]                                                   
  ead_nuts2_30[,iad] <- ead_nuts2_AA_WL[,4]                                                 
  eapa_nuts2_15[,iad] <- eapa_nuts2_AA_WL[,2]                                                 
  eapa_nuts2_20[,iad] <- eapa_nuts2_AA_WL[,3]                                                  
  eapa_nuts2_30[,iad] <- eapa_nuts2_AA_WL[,4]   
######
  print('---')

 print("aggregate WL impacts at country scale and write results ")
  for(nn in 1:4) {
    for (nc in 1:nCY) {
      #    print(paste("Country ",list_countries[nc],sep=""))
      ind_cnty<-which(table_countries[,1]==list_countries[nc])
      ead_countries_AA_WL[nc,nn]<-sum(ead_nuts2_AA_WL[ind_cnty,nn]) # doesn't work of there are NA values
      eapa_countries_AA_WL[nc,nn]<-sum(eapa_nuts2_AA_WL[ind_cnty,nn]) # doesn't work of there are NA values
    }
  }
  fileSuf<- paste0("countries_WL_2100_RP_",SC_classes[iad],"_2100.csv")
  write.csv(ead_countries_AA_WL,file = paste0(workDir,"/tables/EAI_Europe/ead_WL_2100_adapt_ens_fp",prot_par,"_ed",dam_par,"/ead_",fileSuf),row.names=TRUE)
  write.csv(eapa_countries_AA_WL,file = paste0(workDir,"/tables/EAI_Europe/eapa_WL_2100_adapt_ens_fp",prot_par,"/eapa_",fileSuf),row.names=TRUE)

}

## WRITE separate result for each model
for(ii in 1:lMod){
  Pr_mod<-WL_table[ii,2]
  Pr_rcp<-WL_table[ii,3]
  Pr_wu<-WL_table[ii,4]
  ead_mod<-ead_nuts2_mod[,ii,,] 
  eapa_mod<-eapa_nuts2_mod[,ii,,] 
  
## REACTIVATE  
  # for (jj in 1:3){
  jj=2
    flag_WL<-WLS[jj] 
    if(flag_MS=="sta"){
      fileSuf2<- paste0("nuts2_ad_2100_",flag_WL,"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
    } else if(flag_MS=="tra"){
      fileSuf2<- paste0("nuts2_ad_2100_transient_",flag_WL,"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")  
    }
    
  write.csv(ead_mod[,jj+1,], file = paste0(workDir,"/tables/EAI_Europe/ead_nuts2_WL_2100_adapt_models_fp",prot_par,"_ed",dam_par,"/ead_",fileSuf2),row.names=TRUE)
  write.csv(eapa_mod[,jj+1,], file = paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_WL_2100_adapt_models_fp",prot_par,"/eapa_",fileSuf2),row.names=TRUE)
#  }
}

## write ensemble results
 write.csv(ead_nuts2_15, file = paste0(workDir,"/tables/EAI_Europe/ead_WL_2100_adapt_ens_fp",prot_par,"_ed",dam_par,"/ead_nuts2_ad_1.5C_2100.csv"),row.names=TRUE)
 write.csv(ead_nuts2_20, file = paste0(workDir,"/tables/EAI_Europe/ead_WL_2100_adapt_ens_fp",prot_par,"_ed",dam_par,"/ead_nuts2_ad_2C_2100.csv"),row.names=TRUE)
 write.csv(ead_nuts2_30, file = paste0(workDir,"/tables/EAI_Europe/ead_WL_2100_adapt_ens_fp",prot_par,"_ed",dam_par,"/ead_nuts2_ad_3C_2100.csv"),row.names=TRUE)
 write.csv(eapa_nuts2_15, file = paste0(workDir,"/tables/EAI_Europe/eapa_WL_2100_adapt_ens_fp",prot_par,"/eapa_nuts2_ad_1.5C_2100.csv"),row.names=TRUE)
 write.csv(eapa_nuts2_20, file = paste0(workDir,"/tables/EAI_Europe/eapa_WL_2100_adapt_ens_fp",prot_par,"/eapa_nuts2_ad_2C_2100.csv"),row.names=TRUE)
 write.csv(eapa_nuts2_30, file = paste0(workDir,"/tables/EAI_Europe/eapa_WL_2100_adapt_ens_fp",prot_par,"/eapa_nuts2_ad_3C_2100.csv"),row.names=TRUE)


print('Aggregation COMPLETED')



