## 
## calculate optimal adaptation scenarios

#### SET RUN MODE
run<-"cluster" # local-cluster
# flag_MR<-"clim" # "clim" "2050" "2100" "JRC" "SSP1" "SSP3" "SSP5"
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original dat) or 
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean
flag_AN <-"all" ## choose whether to analyze all models (all) or just ensemble mean-median (ens)
#######

if(run=='cluster'){
  args <- commandArgs(TRUE)
  mainDir <- as.character(args[1])     # directory of config file   
} else if(run=='local'){
  mainDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}

flag_WL <- as.character(args[2]) # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
prot_par<- as.character(args[3]) # level of flood protection 
dam_par <- as.character(args[4]) # level of flood damage
cost_par<- as.character(args[5]) # level of adaptation costs 
flag_AD <- as.character(args[6]) # adaptation strategy considered
# flag_WL <- "3C" # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
# flag_AD <- "FP" #dike heightening "DI", storage areas "ST", flood proofing of buildings "FP", relocation "RE" 
# prot_par<- "050"/"100"/"150"/ #050: 50% of estimated average protection standards
# dam_par<- "q05"/"avg"/"q95"
# cost_par<- "q25"/"avg"/"q75"

workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))

##########
print('prepare input data')
## VARIABLES:  dis: discharge - pop:population - dam: damage - prot: protection level (either as RP or QP)

num_ep <-80553 # number of EFAS network cells 
# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
# RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
RY_classes<- c(2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)

nRP<-length(RP_classes)
nRY<-length(RY_classes)
startY=min(RY_classes)
nRYT<-max(RY_classes)-min(RY_classes)+1
if(nRYT==120) {
  nRYT=nRYT+1  # add another year to interpolate between 1980 and 2100
  startY=startY-1
} 

#########
## READ INPUT TABLES
## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))
## read table of warming levels for each model rum
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]
modNames<-WL_table[,2]  
mod1=0

## format NUTS2-country tables
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

###
if(flag_AD=="FP" | flag_AD=="RE"){
  nSC<-20
  SC_classes<-seq(from = 0.05, to = 1, by = 0.05)
} else if(flag_AD=="DI" | flag_AD=="ST"){
  nSC<-16
  SC_classes<-c(2,10,15,20,30,50,70,100,150,200,350,500,700,1000,1500,2000)
}  

#############################
print('---')
print("-  calculate optimal adaptation scenarios")
#############################

## Adjust number of model runs to consider for WL 3C (only RCP8.5 runs)
if(flag_WL == "3C"){
  lMod<-dim(WL_table)[1]/2  
  modNames<-WL_table[12:22,2]
  mod1<-11
}
                                            
NPV_all_nu <- array(0, dim = c(nNU,nSC,lMod),dimnames = list(list_nuts2,SC_classes,modNames))   
Ben_all_nu <- array(0, dim = c(nNU,nSC,lMod),dimnames = list(list_nuts2,SC_classes,modNames))  
Cost_all_nu <- array(0, dim = c(nNU,nSC,lMod),dimnames = list(list_nuts2,SC_classes,modNames))  
Cost_ud_all_nu <- array(0, dim = c(nNU,nSC,lMod),dimnames = list(list_nuts2,SC_classes,modNames)) 
## PopR not in use, replaced by PopR2100
PopR_all_nu <- array(0, dim = c(nNU,nSC,lMod),dimnames = list(list_nuts2,SC_classes,modNames))  
WLI_all_nu <- array(0, dim = c(nNU,nSC,lMod),dimnames = list(list_nuts2,SC_classes,modNames))   
RP_median_nu<- array(0, dim = c(nNU,nSC,lMod),dimnames = list(list_nuts2,SC_classes,modNames)) 

## read data

for(ii in 1:lMod){
  Pr_mod<-WL_table[ii+mod1,2]
  Pr_rcp<-WL_table[ii+mod1,3]
  Pr_wu<-WL_table[ii+mod1,4]
  if(flag_MS=="sta"){
    fileSuf<- paste0(flag_AD,"_",adst,"_",flag_WL,"_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
  } else if(flag_MS=="tra"){
    fileSuf<- paste0(flag_AD,"_",adst,"_transient_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
  }
  if(flag_WL=="3C" & Pr_rcp=="rcp45"){
    print("model not used for WL 3C")
  } else {
    
    modNPV_nu <- as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/NPV_NUTS2_",fileSuf),row.name=1))    
    modCost_nu <-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_NUTS2_",fileSuf),row.name=1))
    modBen_nu <- as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Ben_NUTS2_",fileSuf),row.name=1))
    modCost_ud_nu <-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/Cost_ud_NUTS2_",fileSuf),row.name=1))
    
 ## read increase in dikes height
#    if(flag_AD=="DI"){
#      modWLI_nu <- as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/adaptation_NUTS2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/WLI_NUTS2_",fileSuf),row.name=1))
#      WLI_all_nu[,,ii]<-modWLI_nu
#    }      
    NPV_all_nu[,,ii]<-modNPV_nu
    Cost_all_nu[,,ii]<-modCost_nu
    Cost_ud_all_nu[,,ii]<-modCost_ud_nu    
    Ben_all_nu[,,ii]<-modBen_nu
   
  ## SUMMARIZE PRESENT VALUES OF DESIGN RETURN PERIODS  
#     if(flag_AD=="DI" | flag_AD=="ST"){
#       fileSuf<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
#       mod_RP <- as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/prot_des_adapt/prot_RP_des_",fileSuf)))    
#       for(iad in 1:length(SC_classes)){
#         for (np in 1:nNU) {
#           ind<-which(table_EP_nuts[,2]==list_nuts2[np])
#           RP_median_nu[np,iad,ii] <- median(mod_RP[ind,iad+1],na.rm = TRUE) 
#         }
#       }
#     }
  }
}

####### 
##NEW:
## CALCULATE MAX NPV FOR EACH MODEL AND THEN SAVE ENSEMBLE SPREAD

## save benefit to cost ratio using values corresponding to the adaptation scenario with max NPV
## (even for cases where NPV<0 and BCR <1)
if(flag_AN=="all"){
  colNames<-c("BC optim","Dam undis 2100","DamR 2100","Pop exp 2100","PopR 2100","Cost undis","RP_future","Ben opt","Cost opt","PopR opt")
  CBA_opt_nu<-array(0, dim = c(nNU,10,lMod),dimnames = list(list_nuts2,colNames,modNames)) 
  CBA_opt_cn<-array(0, dim = c(nCY,10,lMod),dimnames = list(list_countries,colNames,modNames)) 
  
  for(ii in 1:lMod){
    CBA_opt_nu[,"RP_future",]<-apply(NPV_all_nu,c(1,3),which.max)  # adaptation scenario with max NPV
    ind<-as.numeric(CBA_opt_nu[,"RP_future",ii])
    # NPV_nu_max<-apply(NPV_all_nu,c(1,3),max)
    
    Pr_mod<-WL_table[ii+mod1,2]
    Pr_rcp<-WL_table[ii+mod1,3]
    Pr_wu<-WL_table[ii+mod1,4]
    if(flag_MS=="sta"){
      fileSuf<- paste0("nuts2_na_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
    } else if(flag_MS=="tra"){
      fileSuf<- paste0("nuts2_na_2100_transient_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
    }
    dam_na<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/ead_nuts2_WL_2100_adapt_models_fp",prot_par,"_ed",dam_par,"/ead_",fileSuf),row.name=1))
    pop_na<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_nuts2_WL_2100_adapt_models_fp",prot_par,"/eapa_",fileSuf),row.name=1))
   ## make sure there are no NA values    
    dam_na[is.na(dam_na)]<-0
    pop_na[is.na(pop_na)]<-0    
    
    if(flag_WL=="3C" & Pr_rcp=="rcp45"){
      print("model not used for WL 3C")
    } else {   
      if(flag_MS=="sta"){
        fileSuf<- paste0(flag_AD,"_",adst,"_",flag_WL,"_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
      } else if(flag_MS=="tra"){
        fileSuf<- paste0(flag_AD,"_",adst,"_transient_clim_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv") 
      }      
      
      PopR_ud_aggr_nu <- as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/PopR_2100_NUTS2_",fileSuf),row.name=1))
      Ben_ud_aggr_nu <- as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/adaptation_nuts2_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/DamR_2100_NUTS2_",fileSuf),row.name=1))  
## make sure there are no NA values    
#     dam_nuts2[is.na(dam_nuts2)]<-0
#     pop_nuts2[is.na(pop_nuts2)]<-0         
#      Ben_ud_aggr_nu <- dam_na[,paste0("X",flag_WL)]-dam_nuts2
#      PopR_ud_aggr_nu <- pop_na[,paste0("X",flag_WL)]-pop_nuts2
    }
    for (i in 1:nNU){   # take corresponding cost-benefit values
      CBA_opt_nu[i,"Ben opt",ii]<-Ben_all_nu[i,ind[i],ii]
      CBA_opt_nu[i,"Cost opt",ii]<-Cost_all_nu[i,ind[i],ii]
      CBA_opt_nu[i,"PopR opt",ii]<-PopR_all_nu[i,ind[i],ii]
      CBA_opt_nu[i,"Cost undis",ii]<-Cost_ud_all_nu[i,ind[i],ii]     
      CBA_opt_nu[i,"Dam undis 2100",ii]<-dam_na[i,paste0("X",flag_WL)]  
      CBA_opt_nu[i,"DamR 2100",ii]<-Ben_ud_aggr_nu[i,ind[i]]
      CBA_opt_nu[i,"Pop exp 2100",ii]<-pop_na[i,paste0("X",flag_WL)]    
      CBA_opt_nu[i,"PopR 2100",ii]<-PopR_ud_aggr_nu[i,ind[i]]  
    }  
    CBA_opt_nu[,"BC optim",ii]<-CBA_opt_nu[,"Ben opt",ii]/CBA_opt_nu[,"Cost opt",ii]
  #  CBA_opt_nu[,"RP_future",ii]<-SC_classes[ind+sc_lim[1]-1]
  
    ### aggregate at country level and save results
    for (nc in 1:nCY){
      ind_cy<-which(table_countries[,1]==list_countries[nc])
      CBA_opt_cn[nc,"Ben opt",ii]<-sum(CBA_opt_nu[ind_cy,"Ben opt",ii])
      CBA_opt_cn[nc,"Cost opt",ii]<-sum(CBA_opt_nu[ind_cy,"Cost opt",ii])
      CBA_opt_cn[nc,"PopR opt",ii]<-sum(CBA_opt_nu[ind_cy,"PopR opt",ii])
      CBA_opt_cn[nc,"Dam undis 2100",ii]<-sum(CBA_opt_nu[ind_cy,"Dam undis 2100",ii])
      CBA_opt_cn[nc,"DamR 2100",ii]<-sum(CBA_opt_nu[ind_cy,"DamR 2100",ii])  
      CBA_opt_cn[nc,"Pop exp 2100",ii]<-sum(CBA_opt_nu[ind_cy,"Pop exp 2100",ii])   
      CBA_opt_cn[nc,"PopR 2100",ii]<-sum(CBA_opt_nu[ind_cy,"PopR 2100",ii])  
      CBA_opt_cn[nc,"Cost undis",ii]<-sum(CBA_opt_nu[ind_cy,"Cost undis",ii])  
    }
    CBA_opt_cn[,"BC optim",ii]<-CBA_opt_cn[,"Ben opt",ii]/CBA_opt_cn[,"Cost opt",ii]

## NO NEED TO PRINT THESE RESULTS      
#    write.csv(CBA_opt_cn[,,ii],file = paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_optimal_countries_",flag_AD,"_",adst,"_",fileSuf),row.names=TRUE)
#    write.csv(CBA_opt_nu[,,ii],file = paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_optimal_nuts2_",flag_AD,"_",adst,"_",fileSuf),row.names=TRUE)
  }
  
  #################
  ## elaborate ensemble statistics
  colNames2<-c("mean","median","5%","95%","min","max")
  CBA_stat_cn<-array(0, dim = c(nCY,6),dimnames = list(list_countries,colNames2)) 
  CBA_stat_nu<-array(0, dim = c(nNU,6),dimnames = list(list_nuts2,colNames2)) 
    
  ## replace BCR na and Inf values with zeros
  CBA_opt_cn[is.na(CBA_opt_cn)]<-0
  CBA_opt_nu[is.na(CBA_opt_nu)]<-0
  CBA_opt_cn[CBA_opt_cn=="Inf"]<-0
  CBA_opt_nu[CBA_opt_nu=="Inf"]<-0
  
  ## write separate files for each variable of interest
  colNames3<-c("BC optim","Dam undis 2100","DamR 2100","Pop exp 2100","PopR 2100","Cost undis","Ben opt","Cost opt")
  
  for(j in 1:length(colNames3)){
    colvar=colNames3[j]
  
    CBA_stat_cn[,"mean"] <- apply(CBA_opt_cn[,colvar,],c(1),mean,na.rm=TRUE)
    CBA_stat_cn[,"median"] <- apply(CBA_opt_cn[,colvar,],c(1),median,na.rm=TRUE)
    CBA_stat_cn[,"min"] <- apply(CBA_opt_cn[,colvar,],c(1),min,na.rm=TRUE)
    CBA_stat_cn[,"max"] <- apply(CBA_opt_cn[,colvar,],c(1),max,na.rm=TRUE)
    CBA_stat_cn[,"5%"] <- apply(CBA_opt_cn[,colvar,],c(1),quantile,c(.05),na.rm=TRUE)
    CBA_stat_cn[,"95%"] <- apply(CBA_opt_cn[,colvar,],c(1),quantile,c(.95),na.rm=TRUE)
  
    CBA_stat_nu[,"mean"] <- apply(CBA_opt_nu[,colvar,],c(1),mean)
    CBA_stat_nu[,"median"] <- apply(CBA_opt_nu[,colvar,],c(1),median)
    CBA_stat_nu[,"min"] <- apply(CBA_opt_nu[,colvar,],c(1),min)
    CBA_stat_nu[,"max"] <- apply(CBA_opt_nu[,colvar,],c(1),max)
    CBA_stat_nu[,"5%"] <- apply(CBA_opt_nu[,colvar,],c(1),quantile,c(.05),na.rm=TRUE)
    CBA_stat_nu[,"95%"] <- apply(CBA_opt_nu[,colvar,],c(1),quantile,c(.95),na.rm=TRUE)
## print ensemble CBA statistics   
#    write.csv(CBA_stat_cn,file = paste0(workDir,"/tables/EAI_Europe/BCA_statistics_countries_",flag_AD,"_",flag_WL,"_",colNames3[j],".csv"),row.names=TRUE)  
## print all ensemble for each variable   - to be used in boxplots  
    
    dir.create( paste0(workDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par))
    if(colvar=="Dam undis 2100" | colvar=="Pop exp 2100"){
      write.csv(CBA_opt_cn[,colvar,],file = paste0(workDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/CBA_ensemble_countries_",flag_WL,"_",colNames3[j],".csv"),row.names=TRUE)    
      write.csv(CBA_opt_nu[,colvar,],file = paste0(workDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/CBA_ensemble_nuts2_",flag_WL,"_",colNames3[j],".csv"),row.names=TRUE)    
    }else{
      write.csv(CBA_opt_cn[,colvar,],file = paste0(workDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/CBA_ensemble_countries_",flag_AD,"_",flag_WL,"_",colNames3[j],".csv"),row.names=TRUE)    
      write.csv(CBA_opt_nu[,colvar,],file = paste0(workDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/CBA_ensemble_nuts2_",flag_AD,"_",flag_WL,"_",colNames3[j],".csv"),row.names=TRUE)          
    }
  }  
}    
## ELABORATE RESULTS FOR THE ENSEMBLE MEAN-MEDIAN
## NOTE: this part replaces the section activated by flag_AN=ens
## to make it consistent with results for all models

  ## CALCULATE MEDIAN-MEAN ACROSS MODELS
  
  ## using the median means that we give equal weight to each realization, no matter the value 
  ## (more focus on probability and less sensitivity to extreme model values)
if (ensemble=="median"){
    CBA_aggr_nu<-apply(CBA_opt_nu,c(1,2),median)
    CBA_aggr_cn<-apply(CBA_opt_cn,c(1,2),median)    
}else if (ensemble=="mean"){
    CBA_aggr_nu<-apply(CBA_opt_nu,c(1,2),mean)
    CBA_aggr_cn<-apply(CBA_opt_cn,c(1,2),mean)
}  
## Print results
 write.csv(CBA_aggr_cn,file = paste0(workDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/CBA_ens_",ensemble,"_countries_",flag_AD,"_",adst,"_",flag_WL,".csv"),row.names=TRUE)
 write.csv(CBA_aggr_nu,file = paste0(workDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par,"/CBA_ens_",ensemble,"_nuts2_",flag_AD,"_",adst,"_",flag_WL,".csv"),row.names=TRUE)
  
#########
## ELABORATE RESULTS FOR THE ENSEMBLE MEAN-MEDIAN
## NOTE: THIS PART OF THE CODE IS NOT ANYMORE USED and was changed   
## to make it consistent with results for all models
## here the code calculate mean-median over all models first and then finds max NPV across protection increases
  ## now we calculate max NPV for each model and then we average  
  
# }else if(flag_AN=="ens"){
# 
# ## CALCULATE MEDIAN-MEAN ACROSS MODELS FOR EACH ADMIN AND ADAPT SCENARIO
# 
# 
#   if (ensemble=="median"){
#     NPV_aggr_nu<-apply(NPV_all_nu,c(1,2),median)
#   }else if (ensemble=="mean"){
#     NPV_aggr_nu<-apply(NPV_all_nu,c(1,2),mean)
#   }  
#   
#   ## calculate skewness to ealuate differences between mean and median
#   # NPV_sd_nu<-apply(NPV_all_nu,c(1,2),sd)
#   # NPV_skew_nu<-(NPV_mean_nu-NPV_median_nu)/NPV_sd_nu
#   
#   ind_NPV_nu<- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   Ben_aggr_nu <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   Cost_aggr_nu<- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   Ben_ud_aggr_nu<- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   Cost_ud_aggr_nu<- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   PopR_aggr_nu<- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   PopR_ud_aggr_nu<- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   RP_aggr_nu <- array(0, dim = c(nNU,nSC),dimnames = list(list_nuts2,SC_classes))
#   WLI_aggr_nu <- array(0, dim = c(nCY,nSC),dimnames = list(list_countries,SC_classes))
#    
#     ## read undiscounted (ud) damage and population exposed (adapted and not) for year 2100
#     dam_nuts2<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/ead_WL_2100_adapt_ens/ead_nuts2_ad_",flag_WL,"_2100.csv"),row.name=1))
#     pop_nuts2<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_WL_2100_adapt_ens/eapa_nuts2_ad_",flag_WL,"_2100.csv"),row.name=1))  
#  
#     dam_na<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_risk_scenarios/ead_nuts2_na_WL_2100.csv"),row.name=1))
#     pop_na<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_risk_scenarios/eapa_nuts2_na_WL_2100.csv"),row.name=1))
#     dam_na=dam_na*10^6
#     Ben_ud_aggr_nu <- dam_na[,paste0("X",flag_WL)]-dam_nuts2
#     PopR_ud_aggr_nu <- pop_na[,paste0("X",flag_WL)]-pop_nuts2  
#   
#   ## retrieve costs/ ben for the model that gives NPV median/mean value
#   for(i in 1:dim(NPV_aggr_nu)[1]){
#     for(j in 1:dim(NPV_aggr_nu)[2]){
#         ind_NPV_nu[i,j]<-which.min(abs(NPV_aggr_nu[i,j]-NPV_all_nu[i,j,]))
#         Cost_aggr_nu[i,j]<-Cost_all_nu[i,j,ind_NPV_nu[i,j]]
#         Cost_ud_aggr_nu[i,j]<-Cost_ud_all_nu[i,j,ind_NPV_nu[i,j]]      
#         Ben_aggr_nu[i,j]<-Ben_all_nu[i,j,ind_NPV_nu[i,j]]
#         PopR_aggr_nu[i,j]<-PopR_all_nu[i,j,ind_NPV_nu[i,j]]      
#     }
#   }
#   
#   ## CALCULATE MEDIAN future RP values and dike heightening
#   RP_aggr_nu<-apply(RP_median_nu,c(1,2),median)
#   if(flag_AD=="DI"){
#     WLI_aggr_nu<-apply(WLI_all_nu,c(1,2),median)
#   } 
#   
#   NPV_aggr_nu[NPV_aggr_nu==0]<--10^20
#   ## not used because values were not consistent with NPV median
#   # Cost_median_nu<-apply(Cost_all_nu,c(1,2),median)
#   
#   if(flag_AD=="FP"){  
#     sc_lim=c(1,12)   # consider only up to 60% reduction)
#   } else {
#     sc_lim=c(1,nSC)
#   } 
#   NPV_aggr_nu <- NPV_aggr_nu[,sc_lim[1]:sc_lim[2]]
#   Cost_aggr_nu <-Cost_aggr_nu[,sc_lim[1]:sc_lim[2]]
#   Ben_aggr_nu <- Ben_aggr_nu[,sc_lim[1]:sc_lim[2]]
#   Cost_ud_aggr_nu <-Cost_ud_aggr_nu[,sc_lim[1]:sc_lim[2]]
#   Ben_ud_aggr_nu <- Ben_ud_aggr_nu[,sc_lim[1]:sc_lim[2]]
#   PopR_aggr_nu <-PopR_aggr_nu[,sc_lim[1]:sc_lim[2]]
#   PopR_ud_aggr_nu <-PopR_ud_aggr_nu[,sc_lim[1]:sc_lim[2]]
#   
#   ####### 
#   ## save benefit to cost ratio using values corresponding to the adaptation scenario with max NPV
#   ## (even for cases where NPV<0 and BCR <1)
#   
#   ## NUTS2
#   CBA_opt_nu<-array(0, dim = c(nNU,10),dimnames = list(list_nuts2,c("BC optim","Dam undis 2100","DamR 2100","Pop exp 2100","PopR 2100","Cost undis","RP_future","Ben opt","Cost opt","PopR opt"))) 
#   CBA_opt_nu[,"RP_future"]<-apply(NPV_aggr_nu,1,which.max)  # adaptation scenario with max NPV
#   ind<-as.numeric(CBA_opt_nu[,"RP_future"])
#   for (i in 1:nNU){   # take corresponding cost-benefit values
#     CBA_opt_nu[i,"Ben opt"]<-Ben_aggr_nu[i,ind[i]]
#     CBA_opt_nu[i,"Cost opt"]<-Cost_aggr_nu[i,ind[i]]
#     CBA_opt_nu[i,"PopR opt"]<-PopR_aggr_nu[i,ind[i]]
#     CBA_opt_nu[i,"Dam undis 2100"]<-dam_na[i,paste0("X",flag_WL)]  
#     CBA_opt_nu[i,"DamR 2100"]<-Ben_ud_aggr_nu[i,ind[i]]
#     CBA_opt_nu[i,"Pop exp 2100"]<-pop_na[i,paste0("X",flag_WL)]    
#     CBA_opt_nu[i,"PopR 2100"]<-PopR_ud_aggr_nu[i,ind[i]]  
#     CBA_opt_nu[i,"Cost undis"]<-Cost_ud_aggr_nu[i,ind[i]]  
#   }  
#   CBA_opt_nu[,"BC optim"]<-CBA_opt_nu[,"Ben opt"]/CBA_opt_nu[,"Cost opt"]
#   CBA_opt_nu[,"RP_future"]<-SC_classes[ind+sc_lim[1]-1]
#   
#   ## calculate country scale as simple aggregation of NUTS2
#   CBA_opt_cn<-array(0, dim = c(nCY,9),dimnames = list(list_countries,c("BC optim","Dam undis 2100","DamR 2100","Pop exp 2100","PopR 2100","Cost undis","Ben opt","Cost opt","PopR opt"))) 
#   for (nc in 1:nCY){
#     ind_cy<-which(table_countries[,1]==list_countries[nc])
#     CBA_opt_cn[nc,"Ben opt"]<-sum(CBA_opt_nu[ind_cy,"Ben opt"])
#     CBA_opt_cn[nc,"Cost opt"]<-sum(CBA_opt_nu[ind_cy,"Cost opt"])
#     CBA_opt_cn[nc,"PopR opt"]<-sum(CBA_opt_nu[ind_cy,"PopR opt"])
#     CBA_opt_cn[nc,"Dam undis 2100"]<-sum(CBA_opt_nu[ind_cy,"Dam undis 2100"])
#     CBA_opt_cn[nc,"DamR 2100"]<-sum(CBA_opt_nu[ind_cy,"DamR 2100"])  
#     CBA_opt_cn[nc,"Pop exp 2100"]<-sum(CBA_opt_nu[ind_cy,"Pop exp 2100"])   
#     CBA_opt_cn[nc,"PopR 2100"]<-sum(CBA_opt_nu[ind_cy,"PopR 2100"])  
#     CBA_opt_cn[nc,"Cost undis"]<-sum(CBA_opt_nu[ind_cy,"Cost undis"])  
#   }
#   CBA_opt_cn[,"BC optim"]<-CBA_opt_cn[,"Ben opt"]/CBA_opt_cn[,"Cost opt"]
#   
#   ####### 
#   ## Print results
#   # write.csv(NPV_skew_cn,file = paste0(workDir,"/tables/EAI_Europe/NPV_skew_countries_",flag_AD,"_",adst,"_",flag_WL,".csv"),row.names=TRUE)
#   # write.csv(NPV_skew_nu,file = paste0(workDir,"/tables/EAI_Europe/NPV_skew_nuts2_",flag_AD,"_",adst,"_",flag_WL,".csv"),row.names=TRUE)
#   write.csv(CBA_opt_cn,file = paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_optimal_countries_",flag_AD,"_",adst,"_",flag_WL,".csv"),row.names=TRUE)
#   write.csv(CBA_opt_nu,file = paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_optimal_nuts2_",flag_AD,"_",adst,"_",flag_WL,".csv"),row.names=TRUE)
#   
#   # if(flag_AD=="DI" | flag_AD=="ST"){
#   #   write.csv(RP_aggr_nu,file = paste0(workDir,"/tables/EAI_Europe/RP_changes_nuts2_",flag_WL,".csv"),row.names=TRUE)
#   #   if(flag_AD=="DI"){
#   #     write.csv(WLI_aggr_nu,file = paste0(workDir,"/tables/EAI_Europe/WLI_nuts2_",flag_WL,".csv"),row.names=TRUE)
#   #   }
#   # }
# }

print('script COMPLETED')

