
## DRAW GRAPHS for the comparison of adaptation options

## script 07d: (only climate sensitivity included) - BOXPLOT with error bars included
## script 07e: (full model sensitivity included)- BARPLOT and ERROR BARS


#### SET RUN MODE
run<-"local" # local-cluster
# flag_MR<-"clim" # "clim" "2050" "2100" "JRC" "SSP1" "SSP3" "SSP5"
flag_WL <- "3C" ## stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
flag_AN <-"ens" ## choose whether to analyze all models (all) or just ensemble mean-median (ens)
#######

library(ggplot2)  

if(run=='cluster'){
  args <- commandArgs(TRUE)
  scriptDir <- as.character(args[1])     # directory of config file
} else if(run=='local'){
  scriptDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}
workDir <-paste0(scriptDir,"/PESETA_IV_data")

##############
## read data

# varlist<-c("BC optim","Dam undis 2100","DamR 2100","Pop exp 2100","PopR 2100","Cost undis","RP_future","Ben opt","Cost opt","PopR opt")
varlist<-c("BC optim","DamR 2100","PopR 2100","Cost undis")
dam_NA<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_ensemble_countries_RE_",flag_WL,"_Dam undis 2100.csv"),row.name=1))
pop_NA<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_ensemble_countries_RE_",flag_WL,"_Pop exp 2100.csv"),row.name=1))

#  list of EU countries
EUC<-c("Austria","Belgium","Bulgaria","Croatia","Cyprus","Czech_Republic","Denmark","Estonia","Finland","France","Germany","Greece","Hungary","Ireland","Italy","Latvia","Lithuania","Luxembourg","Malta","Netherlands","Poland","Portugal","Romania","Slovakia","Slovenia","Spain","Sweden","United_Kingdom")
EUC_lab<-c("AT","BE","BG","HR","CY","CZ","DK","EE","FI","FR","DE","GR","HU","IE","IT","LV","LT","LU","MT","NE","PL","PO","RO","SK","SI","ES","SE","UK")

for(j in 1:length(varlist)){
 var_DI<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_ensemble_countries_DI_",flag_WL,"_",varlist[j],".csv"),row.name=1))
 var_ST<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_ensemble_countries_ST_",flag_WL,"_",varlist[j],".csv"),row.name=1))
 var_RE<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_ensemble_countries_RE_",flag_WL,"_",varlist[j],".csv"),row.name=1))
 var_FP<-as.matrix(read.csv(paste0(workDir,"/tables/EAI_Europe/results_cba_models/CBA_ensemble_countries_FP_",flag_WL,"_",varlist[j],".csv"),row.name=1)) 
 if(varlist[j]=="DamR 2100"){
   var_FP=var_FP*0.5
   var_DI=(dam_NA-var_DI)/10^6
   var_ST=(dam_NA-var_ST)/10^6   
   var_RE=(dam_NA-var_RE)/10^6
   var_FP=(dam_NA-var_FP)/10^6
   dam_NA= dam_NA/10^6
 }else if(varlist[j]=="PopR 2100"){
   var_FP=var_FP*0
   var_DI=pop_NA-var_DI
   var_ST=pop_NA-var_ST   
   var_RE=pop_NA-var_RE
 }else if(varlist[j]=="Cost undis"){
   var_DI=var_DI/(80*10^6)
   var_ST=var_ST/(80*10^6)   
   var_RE=var_RE/(80*10^6)
   var_FP=var_FP/(80*10^6)
 }

#################
### plot results in boxplot

 CN<-rep(EUC,dim(dam_NA)[2])
 if (varlist[j]=="DamR 2100"){
    ModVal<-as.numeric(as.list(dam_NA[EUC,]))
    AD_label<-rep("NA",length(CN))
    data_NA=data.frame(CN,ModVal,AD_label)
  }
 if (varlist[j]=="PopR 2100"){
    ModVal<-as.numeric(as.list(pop_NA[EUC,]))
    AD_label<-rep("NA",length(CN))
    data_NA=data.frame(CN,ModVal,AD_label)
 }
  ModVal<-as.numeric(as.list(var_ST[EUC,]))
  AD_label<-rep("ST",length(CN))
  data_ST=data.frame(CN,ModVal,AD_label)
  
  ModVal<-as.numeric(as.list(var_DI[EUC,]))
  AD_label<-rep("DI",length(CN))
  data_DI=data.frame(CN,ModVal,AD_label)
  
  ModVal<-as.numeric(as.list(var_FP[EUC,]))
  AD_label<-rep("FP",length(CN))
  data_FP=data.frame(CN,ModVal,AD_label)
  
  ModVal<-as.numeric(as.list(var_RE[EUC,]))
  AD_label<-rep("RE",length(CN))
  data_RE=data.frame(CN,ModVal,AD_label)
 
  if (varlist[j]=="DamR 2100"){
    data_p=rbind(data_NA,data_RE,data_FP,data_DI,data_ST)
  } else if (varlist[j]=="PopR 2100"){
    data_p=rbind(data_NA,data_RE,data_DI,data_ST)
  } else{ 
    data_p=rbind(data_RE,data_FP,data_DI,data_ST)
  }

 # Reorder varieties (group) (mixing low and high treatments for the calculations)
# new_order <- with(data_p, reorder(CN , ModVal, AD_label , na.rm=T))

  ## make the boxplot, asking to use the 2 factors : ModVal (in the good order) and countries :
  qualityFig<-3
  if(varlist[j]=="DamR 2100"){
    col1=c("firebrick1","darkorange1","darkorchid1", "dodgerblue1","forestgreen")
    ylab1 = "avg. annual economic damage [M???]"
#    ylim1<-c(0.1,max(ModVal)) # axis limits are needed when using log scale
    ylim1<-NULL
    png(filename = "E:/JRC_work/adaptation -PESETA IV/2020 NCC paper/figures/dam_05.png", width = (800*qualityFig), height = (400*qualityFig), res=(100*qualityFig))        
  } else if (varlist[j]=="PopR 2100"){
    col1=c("firebrick1","darkorange1", "dodgerblue1","forestgreen")
    ylab1 = "avg. annual population exposed"
    ylim1<-NULL
    png(filename = "E:/JRC_work/adaptation -PESETA IV/2020 NCC paper/figures/pop_05.png", width = (800*qualityFig), height = (400*qualityFig), res=(100*qualityFig))    
  }else if (varlist[j]=="BC optim"){
    col1=c("darkorange1","darkorchid1", "dodgerblue1","forestgreen")
    ylim1<-NULL
    ylab1 = "benefit-to-cost ratio"
    png(filename = "E:/JRC_work/adaptation -PESETA IV/2020 NCC paper/figures/BCR_05.png", width = (800*qualityFig), height = (400*qualityFig), res=(100*qualityFig))    
  }else if (varlist[j]=="Cost undis"){
    col1=c("darkorange1","darkorchid1", "dodgerblue1","forestgreen")
    ylab1 = "avg. annual implementation costs [M???]"
    ylim1<-NULL
    png(filename = "E:/JRC_work/adaptation -PESETA IV/2020 NCC paper/figures/cost_05.png", width = (800*qualityFig), height = (400*qualityFig), res=(100*qualityFig))    
  }
  ## set general parameters for all plots
    par(mar=c(3,4,3,1),cex.axis=0.55,cex.lab=0.8)  
    myplot <- boxplot(ModVal ~ AD_label*CN , data=data_p  , # log="y",
                    boxwex=0.4 , ylab=ylab1,  col=col1, ylim = ylim1,                   
                    boxlty="blank",
                    outline=FALSE,xaxt="n")
  
  ## To add the label of x axis using alternate values
  #  my_names <- sapply(strsplit(myplot$names , '\\.') , function(x) x[[2]] )
#  my_names <- my_names[seq(1 , length(my_names) , 4)]

   if (varlist[j]=="DamR 2100"){
     axis(1, at = seq(2.5 , (28*5+1) , 5), labels = EUC_lab ,tick=FALSE , cex=0.2)
     ## Add the grey vertical lines to separate country bars#
     for(i in seq(0.5 , (28*5+1) , 5)){abline(v=i,lty=1, col="grey")}
     legend("topleft", bg="white",bty="o",legend = c("no adaptation","relocation","dam.red.measures","dykes strengthening", "storage areas"),
            col=col1, pch = 15, pt.cex = 2, cex = 0.8,  horiz = F, inset = c(0))
   } else  if (varlist[j]=="PopR 2100"){
     axis(1, at = seq(2.5 , (28*4+1) , 4), labels = EUC_lab ,tick=FALSE , cex=0.2)
     ## Add the grey vertical lines to separate country bars#
     for(i in seq(0.5 , (28*4+1) , 4)){abline(v=i,lty=1, col="grey")}
     legend("topleft", bg="white",bty="o",legend = c("no adaptation","relocation","dykes strengthening", "storage areas"),
            col=col1, pch = 15, pt.cex = 2, cex = 0.8,  horiz = F, inset = c(0))
   } else  if (varlist[j]=="Cost undis"){
     axis(1, at = seq(2.5 , (28*4+1) , 4), labels = EUC_lab ,tick=FALSE ,cex=0.2)
     ## Add the grey vertical lines to separate country bars#
     for(i in seq(0.5 , (28*4+1) , 4)){abline(v=i,lty=1, col="grey")}
     legend("topleft", bg="white",bty="o",legend = c("relocation","dam.red.measures","dykes strengthening", "storage areas"),
            col=col1, pch = 15, pt.cex = 2, cex = 0.8,  horiz = F, inset = c(0))
   }else  if (varlist[j]=="BC optim"){
     axis(1, at = seq(2.5 , (28*4+1) , 4), labels = EUC_lab ,tick=FALSE , cex=0.2)
     ## Add the grey vertical lines to separate country bars, and horizontal one to define BCR =1#
     for(i in seq(0.5 , (28*4+1) , 4)){abline(v=i,lty=1, col="grey")}
     abline(h=1,lty=1, col="grey")
     legend("topleft", bg="white",bty="o", legend = c("relocation","dam.red.measures","dykes strengthening", "storage areas"),
            col=col1, pch = 15,  pt.cex = 2, cex = 0.8,  horiz = F, inset=c(0),y.intersp=1)     
   }
   dev.off()  # to close print device (png or other)
}



