
## ELABORATE ENSEMBLE STATISTICS AND DRAW GRAPHS for the comparison of adaptation options

## Versioning
## script 07d: (only climate sensitivity included) - BOXPLOT with error bars included
## script 07e: (full model sensitivity included)- BARPLOT and ERROR BARS
##########################

#### SET RUN MODE
run<-"local" # local-cluster
# flag_MR<-"clim" # "clim" "2050" "2100" "JRC" "SSP1" "SSP3" "SSP5"
flag_AN <-"ens" ## choose whether to analyze all models (all) or just ensemble mean-median (ens)
#######

if(run=='cluster'){
  library(ggplot2,lib.loc="/home/dottofr/Rlib")
  args <- commandArgs(TRUE)
  mainDir <- as.character(args[1])     # directory of config file
} else if(run=='local'){
  library(ggplot2) 
  mainDir<-("//ies.jrc.it/H07/nahaUsers/dottofr/flood_impacts")   
}
workDir <-paste0(mainDir,"/PESETA_IV_data/ensemble_results")
dataDir <-paste0(mainDir,"/PESETA_IV_data")

figV <- "02-all-col3"  # version given to all saved figures

##############
## read data

## list of parameters considered in the sensitivity analysis
 adlist<- c("DI","ST","FP","RE") # c("DI","ST","FP","RE")
 WLlist<- c("1.5C","2C","3C")   # c("1.5C","2C","3C")
 PLlist<- c("050","100","150") # c("050","100","150")
 DLlist<- c("q05","q50","q95") # c("q05","q50","q95")
 CLlist<- c("q25","q50","q75") # c("q25","q50","q75") 

varlist<-c("DamR 2100","PopR 2100","Cost undis","Ben opt")
varlist_out<-c("EAD_2100","EAPE_2100","AnnCost","BCR")
## input variables
# DamR 2100: reduction in expected annual damage (EAD) in 2100 with adaptation
# PopR 2100: reduction in expected annual population exposed (EAPE) in 2100 with adaptation
# Ben opt: total discounted benefits (avoided damages) over 2020-2100
# Cost opt: total discounted costs over 2020-2100 for adaptation
# Cost undis: undiscounted total costs over 2020-2100 for adaptation
## output variables
# EAD 2100:undiscounted expected annual damage in 2100 with\without adaptation
# EAD-p 2100: reduction in expected annual damage in 2100 with adaptation (0:no reduction 1:complete reduction)
# EAPE 2100: expected annual pop.exp. in 2100 with\without adaptation
# EAPE-p 2100: reduction in expected annual pop.exp. in 2100 with adaptation (0:no reduction 1:complete reduction)
# AnnCost: undiscounted average annual costs over 2020-2100 for adaptation
# BCR: overall benefit-to-cost ratio

#  list of EU countries
EUC<-c("Austria","Belgium","Bulgaria","Croatia","Cyprus","Czech_Republic","Denmark","Estonia","Finland","France","Germany","Greece","Hungary","Ireland","Italy","Latvia","Lithuania","Luxembourg","Malta","Netherlands","Poland","Portugal","Romania","Slovakia","Slovenia","Spain","Sweden","United_Kingdom")
EUC_lab<-c("AT","BE","BG","HR","CY","CZ","DK","EE","FI","FR","DE","GR","HU","IE","IT","LV","LT","LU","MT","NE","PL","PO","RO","SK","SI","ES","SE","UK")

########################
## PART 1- CALCULATE BASELINE RESULTS FOR NUTS2 AND COUNTRIES
########################

## results for baseline impacts (1995 as central year for all model runs)
WL_table <- read.csv(paste(dataDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]
modNames<-WL_table[,2]  
table_countries <- read.csv(paste(dataDir,"/tables/table_nuts_countries.csv", sep=""))
list_countries<-as.vector(unique(table_countries[,1]))
list_nuts2<-unique(table_countries[,2])
nCY<-length(list_countries)
nNU<-length(list_nuts2)

EU_data_PNA=0 
EU_data_DNA=0    
flag_EU_data=0  # flag to correctly build output dataframe

for (jj in 1:length(PLlist)){  
    for (kk in 1:length(DLlist)){
        flag_WL <- "base"
        prot_par <- PLlist[jj]
        dam_par <- DLlist[kk]
        for(ii in 1:lMod){
            Pr_mod<-WL_table[ii,2]
            Pr_rcp<-WL_table[ii,3]
            Pr_wu<-WL_table[ii,4]
            fileDir= paste0(dataDir,"/tables/EAI_Europe/ead_nuts2_WL_2100_adapt_models_fp",prot_par,"_ed",dam_par)
            dam_NA<-as.matrix(read.csv(paste0(fileDir,"/ead_nuts2_na_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1))
            fileDir= paste0(dataDir,"/tables/EAI_Europe/eapa_nuts2_WL_2100_adapt_models_fp",prot_par)
            pop_NA<-as.matrix(read.csv(paste0(fileDir,"/eapa_nuts2_na_2100_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1))
            dam_NA= dam_NA/10^6      
            if(flag_EU_data==0){
              EU_data_PNA=pop_NA[,"base"]             
              EU_data_DNA=dam_NA[,"base"]
              flag_EU_data=1
            } else {  # append results consecutively              
              EU_data_PNA=cbind(EU_data_PNA,pop_NA[,"base"] )
              EU_data_DNA=cbind(EU_data_DNA,dam_NA[,"base"] )                           
            } 
        }                    
   }
}    
EU_data_PNA[is.na(EU_data_PNA)] <- 0
EU_data_DNA[is.na(EU_data_DNA)] <- 0  
## aggregate impacts at country scale 
EU_data_PNA_cy= aggregate(EU_data_PNA, by=list(table_countries[,"COUNTRY"]), sum)
EU_data_DNA_cy= aggregate(EU_data_DNA, by=list(table_countries[,"COUNTRY"]), sum)
rownames(EU_data_PNA_cy)=EU_data_PNA_cy[,"Group.1"]
rownames(EU_data_DNA_cy)=EU_data_DNA_cy[,"Group.1"]

## print ensemble results for testing
#    write.csv(EU_data_PNA, file = paste0(workDir,"/results_nuts2_PNA.csv"),row.names=TRUE)
    
##   COMPUTE QUANTILES USING ALL DATA FROM SENS.ANALYSIS: models-dam-cost-prot    
#    confInt <- function(x,probs=c(0.1, 0.25, 0.5, 0.75, 0.9) ){quantile}
qt_data_PNA= t(apply(EU_data_PNA, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
qt_data_DNA= t(apply(EU_data_DNA, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))  
qt_data_PNA_cy= t(apply(EU_data_PNA_cy[,2:199], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
qt_data_DNA_cy= t(apply(EU_data_DNA_cy[,2:199], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))   

qt_data_all=cbind(qt_data_PNA,qt_data_DNA)
colnames(qt_data_all)=c("PNA-10%","PNA-25%","PNA-50%","PNA-75%","PNA-90%","PNA-avg","DNA-10%","DNA-25%","DNA-50%","DNA-75%","DNA-90%","DNA-avg")
qt_data_all_cy=cbind(qt_data_PNA_cy,qt_data_DNA_cy)
colnames(qt_data_all_cy)=c("PNA-10%","PNA-25%","PNA-50%","PNA-75%","PNA-90%","PNA-avg","DNA-10%","DNA-25%","DNA-50%","DNA-75%","DNA-90%","DNA-avg")
## save nuts2 tables
write.csv(qt_data_all, file = paste0(workDir,"/results_SA_nuts2_baseline.csv"),row.names=TRUE)  
write.csv(qt_data_all_cy, file = paste0(workDir,"/results_SA_countries_baseline.csv"),row.names=TRUE)  
rm(EU_data_DNA,EU_data_PNA,EU_data_DNA_cy,EU_data_PNA_cy)

## CALCULATE RESULTS FOR NUTS2 -  adaptation under future warming levels
for(j in 1:length(varlist)){
  for (ii in 1:length(WLlist)){   
    EU_data_DI=0
    EU_data_ST=0
    EU_data_FP=0    
    EU_data_RE=0    
    EU_data_PNA=0  
    EU_data_DNA=0    
    EU_data_DI_p=0
    EU_data_ST_p=0
    EU_data_FP_p=0    
    EU_data_RE_p=0    
    flag_EU_data=0  # flag to correctly build output dataframe
    flag_EU_data_p=0
    for (jj in 1:length(PLlist)){  
      for (kk in 1:length(DLlist)){
        for (nn in 1:length(CLlist)){
          flag_WL <- WLlist[ii] 
          prot_par <- PLlist[jj]
          dam_par <- DLlist[kk]
          cost_par <- CLlist[nn] 
          fileDir= paste0(dataDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par)
          ## read variables 
          dam_NA<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_",flag_WL,"_Dam undis 2100.csv"),row.name=1))
          pop_NA<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_",flag_WL,"_Pop exp 2100.csv"),row.name=1))
          var_DI<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_DI_",flag_WL,"_",varlist[j],".csv"),row.name=1))
          var_ST<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_ST_",flag_WL,"_",varlist[j],".csv"),row.name=1))
          var_RE<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_RE_",flag_WL,"_",varlist[j],".csv"),row.name=1))
          var_FP<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_FP_",flag_WL,"_",varlist[j],".csv"),row.name=1))     
          
          if(varlist[j]=="DamR 2100"){
            var_DI=(dam_NA-var_DI)/10^6
            var_ST=(dam_NA-var_ST)/10^6   
            var_RE=(dam_NA-var_RE)/10^6
            var_FP=(dam_NA-var_FP)/10^6
            dam_NA= dam_NA/10^6
            var_DI_p=(dam_NA-var_DI)/dam_NA
            var_ST_p=(dam_NA-var_ST)/dam_NA   
            var_RE_p=(dam_NA-var_RE)/dam_NA
            var_FP_p=(dam_NA-var_FP)/dam_NA
            var_DI_p[is.nan(var_DI_p)] <- 0
            var_ST_p[is.nan(var_ST_p)] <- 0  
            var_RE_p[is.nan(var_RE_p)] <- 0
            var_FP_p[is.nan(var_FP_p)] <- 0        
          }else if(varlist[j]=="PopR 2100"){
            var_FP=pop_NA
            var_DI=pop_NA-var_DI
            var_ST=pop_NA-var_ST   
            var_RE=pop_NA-var_RE
            var_DI_p=(pop_NA-var_DI)/pop_NA
            var_ST_p=(pop_NA-var_ST)/pop_NA   
            var_RE_p=(pop_NA-var_RE)/pop_NA
            var_FP_p= pop_NA*0
            var_DI_p[is.na(var_DI_p)] <- 0
            var_ST_p[is.na(var_ST_p)] <- 0  
            var_RE_p[is.na(var_RE_p)] <- 0
            var_FP_p[is.na(var_FP_p)] <- 0                    
          }else if(varlist[j]=="Cost undis"){
            var_DI=var_DI/(81*10^6)
            var_ST=var_ST/(81*10^6)   
            var_RE=var_RE/(81*10^6)
            var_FP=var_FP/(81*10^6)
          }else if(varlist[j]=="Ben opt"){
            var2="Cost opt"
            var_DI2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_DI_",flag_WL,"_",var2,".csv"),row.name=1))
            var_ST2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_ST_",flag_WL,"_",var2,".csv"),row.name=1))
            var_RE2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_RE_",flag_WL,"_",var2,".csv"),row.name=1))
            var_FP2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_nuts2_FP_",flag_WL,"_",var2,".csv"),row.name=1)) 
            var_DI=var_DI/var_DI2
            var_ST=var_ST/var_ST2  
            var_RE=var_RE/var_RE2
            var_FP=var_FP/var_FP2 
            var_DI[is.na(var_DI)] <- 0
            var_ST[is.na(var_ST)] <- 0  
            var_RE[is.na(var_RE)] <- 0
            var_FP[is.na(var_FP)] <- 0 
            var_DI[is.infinite(var_DI)] <- 0
            var_ST[is.infinite(var_ST)] <- 0  
            var_RE[is.infinite(var_RE)] <- 0
            var_FP[is.infinite(var_FP)] <- 0            
          }         
          if(flag_EU_data==0){
            EU_data_DI=var_DI
            EU_data_RE=var_RE
            EU_data_ST=var_ST              
            EU_data_FP=var_FP
            EU_data_PNA=pop_NA             
            EU_data_DNA=dam_NA 
            flag_EU_data=1
          } else {  # append results consecutively
            EU_data_DI=cbind(EU_data_DI,var_DI)
            EU_data_RE=cbind(EU_data_RE,var_RE)              
            EU_data_ST=cbind(EU_data_ST,var_ST)
            EU_data_FP=cbind(EU_data_FP,var_FP)              
            EU_data_PNA=cbind(EU_data_PNA,pop_NA)
            EU_data_DNA=cbind(EU_data_DNA,dam_NA)                           
          } 
          ## calculate reduced impacts as fraction          
          if(varlist[j]=="DamR 2100" | varlist[j]=="PopR 2100"){ 
            if(flag_EU_data_p==0){
              EU_data_DI_p=var_DI_p
              EU_data_RE_p=var_RE_p
              EU_data_ST_p=var_ST_p              
              EU_data_FP_p=var_FP_p
              flag_EU_data_p=1
            } else {  # append results consecutively
              EU_data_DI_p=cbind(EU_data_DI_p,var_DI_p)
              EU_data_RE_p=cbind(EU_data_RE_p,var_RE_p)              
              EU_data_ST_p=cbind(EU_data_ST_p,var_ST_p)
              EU_data_FP_p=cbind(EU_data_FP_p,var_FP_p)                                         
            }            
          }             
        }
      }
    }    
    ## print ensemble results for testing
#    EU_data_fp2=EU_data_FP[EUC,]
#    write.csv(EU_data_fp2, file = paste0(workDir,"/results_nuts2_FP_cost_ud_all.csv"),row.names=TRUE)
    
    ##   COMPUTE QUANTILES USING ALL DATA FROM SENS.ANALYSIS: models-dam-cost-prot    
    #    confInt <- function(x,probs=c(0.1, 0.25, 0.5, 0.75, 0.9) ){quantile}
    qt_data_DI= t(apply(EU_data_DI, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))  
    qt_data_FP= t(apply(EU_data_FP, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_RE= t(apply(EU_data_RE, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_ST= t(apply(EU_data_ST, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_PNA= t(apply(EU_data_PNA, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_DNA= t(apply(EU_data_DNA, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))  
    
    
    qt_data_all=cbind(qt_data_DI,qt_data_ST,qt_data_RE,qt_data_FP,qt_data_PNA,qt_data_DNA)
    colnames(qt_data_all)=c("DI-10%","DI-25%","DI-50%","DI-75%","DI-90%","DI-avg","ST-10%","ST-25%","ST-50%","ST-75%","ST-90%","ST-avg",
                            "RE-10%","RE-25%","RE-50%","RE-75%","RE-90%","RE-avg","FP-10%","FP-25%","FP-50%","FP-75%","FP-90%","FP-avg",
                            "PNA-10%","PNA-25%","PNA-50%","PNA-75%","PNA-90%","PNA-avg","DNA-10%","DNA-25%","DNA-50%","DNA-75%","DNA-90%","DNA-avg")
    ## save country tables
    write.csv(qt_data_all, file = paste0(workDir,"/results_SA_nuts2_",WLlist[ii],"_",varlist_out[j],".csv"),row.names=TRUE)  
    rm(EU_data_DI,EU_data_RE,EU_data_ST,EU_data_FP,EU_data_DNA,EU_data_PNA)
    ##   COMPUTE QUANTILES for reduced impacts as fraction   
    if(varlist[j]=="DamR 2100" | varlist[j]=="PopR 2100"){    
      qt_data_DI_p= t(apply(EU_data_DI_p, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))  
      qt_data_FP_p= t(apply(EU_data_FP_p, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
      qt_data_RE_p= t(apply(EU_data_RE_p, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
      qt_data_ST_p= t(apply(EU_data_ST_p, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
      qt_data_all_p=cbind(qt_data_DI_p,qt_data_ST_p,qt_data_RE_p,qt_data_FP_p)
      colnames(qt_data_all_p)=c("DI-10%","DI-25%","DI-50%","DI-75%","DI-90%","DI-avg","ST-10%","ST-25%","ST-50%","ST-75%","ST-90%","ST-avg",
                                "RE-10%","RE-25%","RE-50%","RE-75%","RE-90%","RE-avg","FP-10%","FP-25%","FP-50%","FP-75%","FP-90%","FP-avg")      
      if(varlist[j]=="DamR 2100"){  
        write.csv(qt_data_all_p, file = paste0(workDir,"/results_SA_nuts2_",WLlist[ii],"_EAD-r_2100.csv"),row.names=TRUE)          
      } else if(varlist[j]=="PopR 2100"){    
        write.csv(qt_data_all_p, file = paste0(workDir,"/results_SA_nuts2_",WLlist[ii],"_EAPE-r_2100.csv"),row.names=TRUE)  
      }
      rm(EU_data_DI,EU_data_RE,EU_data_ST,EU_data_FP,EU_data_DNA,EU_data_PNA)      
    } 
    
  } #close here loop over WLS 
} #close here loop over varlist    


###########################
## PART 2- CALCULATE RESULTS FOR EU+UK COUNTRIES
###########################

 for(j in 1:length(varlist)){
  for (ii in 1:length(WLlist)){   
    EU_data_DI=0
    EU_data_ST=0
    EU_data_FP=0    
    EU_data_RE=0    
    EU_data_PNA=0  
    EU_data_DNA=0    
    EU_data_DI_p=0
    EU_data_ST_p=0
    EU_data_FP_p=0    
    EU_data_RE_p=0    
    flag_EU_data=0  # flag to correctly build output dataframe
    flag_EU_data_p=0    
    for (jj in 1:length(PLlist)){  
      for (kk in 1:length(DLlist)){
        for (nn in 1:length(CLlist)){
          flag_WL <- WLlist[ii] 
          prot_par <- PLlist[jj]
          dam_par <- DLlist[kk]
          cost_par <- CLlist[nn] 
          fileDir= paste0(dataDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par)
          ## read variables 
          dam_NA<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_",flag_WL,"_Dam undis 2100.csv"),row.name=1))
          pop_NA<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_",flag_WL,"_Pop exp 2100.csv"),row.name=1))
          var_DI<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_DI_",flag_WL,"_",varlist[j],".csv"),row.name=1))
          var_ST<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_ST_",flag_WL,"_",varlist[j],".csv"),row.name=1))
          var_RE<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_RE_",flag_WL,"_",varlist[j],".csv"),row.name=1))
          var_FP<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_FP_",flag_WL,"_",varlist[j],".csv"),row.name=1))     
          
          if(varlist[j]=="DamR 2100"){
            var_DI=(dam_NA-var_DI)/10^6
            var_ST=(dam_NA-var_ST)/10^6   
            var_RE=(dam_NA-var_RE)/10^6
            var_FP=(dam_NA-var_FP)/10^6
            dam_NA= dam_NA/10^6
            var_DI_p=(dam_NA-var_DI)/dam_NA
            var_ST_p=(dam_NA-var_ST)/dam_NA   
            var_RE_p=(dam_NA-var_RE)/dam_NA
            var_FP_p=(dam_NA-var_FP)/dam_NA
            var_DI_p[is.nan(var_DI_p)] <- 0
            var_ST_p[is.nan(var_ST_p)] <- 0  
            var_RE_p[is.nan(var_RE_p)] <- 0
            var_FP_p[is.nan(var_FP_p)] <- 0        
          }else if(varlist[j]=="PopR 2100"){
            var_FP=pop_NA
            var_DI=pop_NA-var_DI
            var_ST=pop_NA-var_ST   
            var_RE=pop_NA-var_RE
            var_DI_p=(pop_NA-var_DI)/pop_NA
            var_ST_p=(pop_NA-var_ST)/pop_NA   
            var_RE_p=(pop_NA-var_RE)/pop_NA
            var_FP_p= pop_NA*0
            var_DI_p[is.na(var_DI_p)] <- 0
            var_ST_p[is.na(var_ST_p)] <- 0  
            var_RE_p[is.na(var_RE_p)] <- 0
            var_FP_p[is.na(var_FP_p)] <- 0                    
          }else if(varlist[j]=="Cost undis"){
            var_DI=var_DI/(81*10^6)
            var_ST=var_ST/(81*10^6)   
            var_RE=var_RE/(81*10^6)
            var_FP=var_FP/(81*10^6)
          }else if(varlist[j]=="Ben opt"){
            var2="Cost opt"
            var_DI2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_DI_",flag_WL,"_",var2,".csv"),row.name=1))
            var_ST2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_ST_",flag_WL,"_",var2,".csv"),row.name=1))
            var_RE2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_RE_",flag_WL,"_",var2,".csv"),row.name=1))
            var_FP2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_FP_",flag_WL,"_",var2,".csv"),row.name=1)) 
            var_DI=var_DI/var_DI2
            var_ST=var_ST/var_ST2  
            var_RE=var_RE/var_RE2
            var_FP=var_FP/var_FP2 
            var_DI[is.na(var_DI)] <- 0
            var_ST[is.na(var_ST)] <- 0  
            var_RE[is.na(var_RE)] <- 0
            var_FP[is.na(var_FP)] <- 0 
            var_DI[is.infinite(var_DI)] <- 0
            var_ST[is.infinite(var_ST)] <- 0  
            var_RE[is.infinite(var_RE)] <- 0
            var_FP[is.infinite(var_FP)] <- 0            
          }         
          if(flag_EU_data==0){
            EU_data_DI=var_DI
            EU_data_RE=var_RE
            EU_data_ST=var_ST              
            EU_data_FP=var_FP
            EU_data_PNA=pop_NA
            EU_data_DNA=dam_NA
            flag_EU_data=1
          } else {  # append results consecutively
            EU_data_DI=cbind(EU_data_DI,var_DI)
            EU_data_RE=cbind(EU_data_RE,var_RE)              
            EU_data_ST=cbind(EU_data_ST,var_ST)
            EU_data_FP=cbind(EU_data_FP,var_FP)              
            EU_data_PNA=cbind(EU_data_PNA,pop_NA)
            EU_data_DNA=cbind(EU_data_DNA,dam_NA)                           
          } 
## calculate reduced impacts as fraction          
          if(varlist[j]=="DamR 2100" | varlist[j]=="PopR 2100"){ 
            if(flag_EU_data_p==0){
              EU_data_DI_p=var_DI_p
              EU_data_RE_p=var_RE_p
              EU_data_ST_p=var_ST_p              
              EU_data_FP_p=var_FP_p
              flag_EU_data_p=1
            } else {  # append results consecutively
              EU_data_DI_p=cbind(EU_data_DI_p,var_DI_p)
              EU_data_RE_p=cbind(EU_data_RE_p,var_RE_p)              
              EU_data_ST_p=cbind(EU_data_ST_p,var_ST_p)
              EU_data_FP_p=cbind(EU_data_FP_p,var_FP_p)                                         
            }            
          }             
        }
      }
    }    
## print ensemble results for testing
    EU_data_fp2=EU_data_FP[EUC,]
    write.csv(EU_data_fp2, file = paste0(workDir,"/results_EUcountries_FP_cost_ud_all.csv"),row.names=TRUE)
    
##   COMPUTE QUANTILES USING ALL DATA FROM SENS.ANALYSIS: models-dam-cost-prot    
#    confInt <- function(x,probs=c(0.1, 0.25, 0.5, 0.75, 0.9) ){quantile}
    qt_data_DI= t(apply(EU_data_DI[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))  
    qt_data_FP= t(apply(EU_data_FP[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_RE= t(apply(EU_data_RE[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_ST= t(apply(EU_data_ST[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_PNA= t(apply(EU_data_PNA[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    qt_data_DNA= t(apply(EU_data_DNA[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))  

    
    qt_data_all=cbind(qt_data_DI,qt_data_ST,qt_data_RE,qt_data_FP,qt_data_PNA,qt_data_DNA)
    colnames(qt_data_all)=c("DI-10%","DI-25%","DI-50%","DI-75%","DI-90%","DI-avg","ST-10%","ST-25%","ST-50%","ST-75%","ST-90%","ST-avg",
                            "RE-10%","RE-25%","RE-50%","RE-75%","RE-90%","RE-avg","FP-10%","FP-25%","FP-50%","FP-75%","FP-90%","FP-avg",
                            "PNA-10%","PNA-25%","PNA-50%","PNA-75%","PNA-90%","PNA-avg","DNA-10%","DNA-25%","DNA-50%","DNA-75%","DNA-90%","DNA-avg")
    ## save country tables
    write.csv(qt_data_all, file = paste0(workDir,"/results_SA_EU28countries_",WLlist[ii],"_",varlist_out[j],".csv"),row.names=TRUE)  
    rm(EU_data_DI,EU_data_RE,EU_data_ST,EU_data_FP,EU_data_DNA,EU_data_PNA)
##   COMPUTE QUANTILES for reduced impacts as fraction   
    if(varlist[j]=="DamR 2100" | varlist[j]=="PopR 2100"){    
      qt_data_DI_p= t(apply(EU_data_DI_p[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))  
      qt_data_FP_p= t(apply(EU_data_FP_p[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
      qt_data_RE_p= t(apply(EU_data_RE_p[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
      qt_data_ST_p= t(apply(EU_data_ST_p[EUC,], 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
      qt_data_all_p=cbind(qt_data_DI_p,qt_data_ST_p,qt_data_RE_p,qt_data_FP_p)
      colnames(qt_data_all_p)=c("DI-10%","DI-25%","DI-50%","DI-75%","DI-90%","DI-avg","ST-10%","ST-25%","ST-50%","ST-75%","ST-90%","ST-avg",
                                "RE-10%","RE-25%","RE-50%","RE-75%","RE-90%","RE-avg","FP-10%","FP-25%","FP-50%","FP-75%","FP-90%","FP-avg")      
      if(varlist[j]=="DamR 2100"){  
        write.csv(qt_data_all_p, file = paste0(workDir,"/results_SA_EU28countries_",WLlist[ii],"_EAD-r_2100.csv"),row.names=TRUE)          
      } else if(varlist[j]=="PopR 2100"){    
        write.csv(qt_data_all_p, file = paste0(workDir,"/results_SA_EU28countries_",WLlist[ii],"_EAPE-r_2100.csv"),row.names=TRUE)  
      }
      rm(EU_data_DI,EU_data_RE,EU_data_ST,EU_data_FP,EU_data_DNA,EU_data_PNA)      
    } 
    
  } #close here loop over WLS 
 } #close here loop over varlist    

###################################################
## PART 3- CALCULATE RESULTS AND GRAPHS FOR AGGREGATED EU+UK AREA
###################################################

for(j in 1:length(varlist)){
  for (ii in 1:length(WLlist)){   
    EU_data_all=0
    EU_data_all2=0
    flag_EU_data=0  # flag to correctly build output dataframe  
    flag_EU_data2=0     
    for (jj in 1:length(PLlist)){  
      for (kk in 1:length(DLlist)){
          for (nn in 1:length(CLlist)){
            flag_WL <- WLlist[ii] 
            prot_par <- PLlist[jj]
            dam_par <- DLlist[kk]
            cost_par <- CLlist[nn] 
            fileDir= paste0(dataDir,"/tables/EAI_Europe/results_cba_models_fp",prot_par,"_ed",dam_par,"_ac",cost_par)
## read variables 
            dam_NA<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_",flag_WL,"_Dam undis 2100.csv"),row.name=1))
            pop_NA<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_",flag_WL,"_Pop exp 2100.csv"),row.name=1))
            var_DI<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_DI_",flag_WL,"_",varlist[j],".csv"),row.name=1))
            var_ST<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_ST_",flag_WL,"_",varlist[j],".csv"),row.name=1))
            var_RE<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_RE_",flag_WL,"_",varlist[j],".csv"),row.name=1))
            var_FP<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_FP_",flag_WL,"_",varlist[j],".csv"),row.name=1))     
 
            if(varlist[j]=="DamR 2100"){
                var_DI2=(dam_NA-var_DI)/10^6
                var_ST2=(dam_NA-var_ST)/10^6   
                var_RE2=(dam_NA-var_RE)/10^6
                var_FP2=(dam_NA-var_FP)/10^6
                dam_NA= dam_NA/10^6              
            }else if(varlist[j]=="PopR 2100"){
              var_FP2=pop_NA
              var_DI2=pop_NA-var_DI
              var_ST2=pop_NA-var_ST   
              var_RE2=pop_NA-var_RE
            }else if(varlist[j]=="Cost undis"){
              var_DI=var_DI/(81*10^6)
              var_ST=var_ST/(81*10^6)   
              var_RE=var_RE/(81*10^6)
              var_FP=var_FP/(81*10^6)
            }else if(varlist[j]=="Ben opt"){
              var2="Cost opt"
              var_DI2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_DI_",flag_WL,"_",var2,".csv"),row.name=1))
              var_ST2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_ST_",flag_WL,"_",var2,".csv"),row.name=1))
              var_RE2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_RE_",flag_WL,"_",var2,".csv"),row.name=1))
              var_FP2<-as.matrix(read.csv(paste0(fileDir,"/CBA_ensemble_countries_FP_",flag_WL,"_",var2,".csv"),row.name=1)) 
            }      
             
## calculate EU28 totals 
            if(varlist[j]=="DamR 2100"){
              EU_data=array(0, dim=c(5,dim(var_DI)[2]))
              EU_data[1,]=apply(var_DI2[EUC,], 2, sum)
              EU_data[2,]=apply(var_ST2[EUC,], 2, sum)
              EU_data[3,]=apply(var_FP2[EUC,], 2, sum)
              EU_data[4,]=apply(var_RE2[EUC,], 2, sum)
              EU_data[5,]=apply(dam_NA[EUC,], 2, sum)
              EU_data2=array(0, dim=c(4,dim(var_DI)[2]))
              EU_data2[1,]=apply(var_DI[EUC,], 2, sum)/(10^6*apply(dam_NA[EUC,], 2, sum) )
              EU_data2[2,]=apply(var_ST[EUC,], 2, sum)/(10^6*apply(dam_NA[EUC,], 2, sum) )
              EU_data2[3,]=apply(var_FP[EUC,], 2, sum)/(10^6*apply(dam_NA[EUC,], 2, sum) )
              EU_data2[4,]=apply(var_RE[EUC,], 2, sum)/(10^6*apply(dam_NA[EUC,], 2, sum) )               
              adlist<-c("4-raising dykes","5-retention areas","3-flood proofing","2-relocation","1-no adaptation")              
              adlist2<-c("3-raising dykes","4-retention areas","2-flood proofing","1-relocation")
            }else if(varlist[j]=="PopR 2100"){
               EU_data=array(0, dim=c(4,dim(var_DI)[2]))
               EU_data[1,]=apply(var_DI2[EUC,], 2, sum)
               EU_data[2,]=apply(var_ST2[EUC,], 2, sum)
               EU_data[3,]=apply(var_RE2[EUC,], 2, sum)
               EU_data[4,]=apply(pop_NA[EUC,], 2, sum)
               EU_data2=array(0, dim=c(4,dim(var_DI)[2]))
               EU_data2[1,]=apply(var_DI[EUC,], 2, sum)/apply(pop_NA[EUC,], 2, sum)
               EU_data2[2,]=apply(var_ST[EUC,], 2, sum)/apply(pop_NA[EUC,], 2, sum)
               EU_data2[3,]=apply(var_RE[EUC,], 2, sum)/apply(pop_NA[EUC,], 2, sum) 
               EU_data2[4,]=apply(pop_NA[EUC,], 2, sum)*0               
               adlist<-c("3-raising dykes","4-retention areas","2-relocation","1-no adaptation")
               adlist2<-c("3-raising dykes","4-retention areas","2-relocation","1-flood proofing")                              
            }else if(varlist[j]=="Cost undis"){
               EU_data=array(0, dim=c(4,dim(var_DI)[2]))
               EU_data[1,]=apply(var_DI[EUC,], 2, sum)
               EU_data[2,]=apply(var_ST[EUC,], 2, sum)
               EU_data[3,]=apply(var_FP[EUC,], 2, sum)
               EU_data[4,]=apply(var_RE[EUC,], 2, sum) 
               adlist<-c("1-raising dykes","2-retention areas","3-flood proofing","4-relocation")     
            }else if(varlist[j]=="Ben opt"){
## calculate BCR using discounted benefits and costs
               EU_data=array(0, dim=c(4,dim(var_DI)[2]))
               EU_data[1,]=apply(var_DI[EUC,], 2, sum)/ apply(var_DI2[EUC,], 2, sum)
               EU_data[2,]=apply(var_ST[EUC,], 2, sum)/ apply(var_ST2[EUC,], 2, sum)
               EU_data[3,]=apply(var_FP[EUC,], 2, sum)/ apply(var_FP2[EUC,], 2, sum)
               EU_data[4,]=apply(var_RE[EUC,], 2, sum)/ apply(var_RE2[EUC,], 2, sum)
#               EU_data[is.na(EU_data)] <- 0
#               EU_data[is.infinite(EU_data)] <- 0                  
               adlist<-c("1-raising dykes","2-retention areas","3-flood proofing","4-relocation")   
            } 
            if(flag_EU_data==0){
              EU_data_all=EU_data
              flag_EU_data=1              
            } else {  # append results consecutively
              EU_data_all=cbind(EU_data_all,EU_data)
            }
            if(varlist[j]=="DamR 2100" | varlist[j]=="PopR 2100"){            
              if(flag_EU_data2==0){
                EU_data_all2=EU_data2
                flag_EU_data2=1 
              } else {  # append results consecutively
                EU_data_all2=cbind(EU_data_all2,EU_data2)
              } 
            }              
        }
      }
    }    
##   COMPUTE QUANTILES USING ALL DATA FROM SENS.ANALYSIS: models-dam-cost-prot    
##  (we use quantiles over standard deviation to avoid issues with outliers-zero)
    # sd_data=apply(var_DI[EUC,], 1, sd)
    qt_data= t(apply(EU_data_all, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
    rownames(qt_data)=adlist[]    
    AD_label<-rep(adlist,3)
    if(flag_WL=="1.5C"){
      WL_label<-rep("1.5?C",length(adlist))      
      data_15C=data.frame(WL_label,qt_data[,c("25%","50%","75%")],adlist)
      data_15C_csv=data.frame(WL_label,qt_data,adlist)      
    }else if(flag_WL=="2C"){
      WL_label<-rep("2?C",length(adlist))
      data_20C=data.frame(WL_label,qt_data[,c("25%","50%","75%")],adlist)
      data_20C_csv=data.frame(WL_label,qt_data,adlist)      
    }else if(flag_WL=="3C"){
      WL_label<-rep("3?C",length(adlist))       
      data_30C=data.frame(WL_label,qt_data[,c("25%","50%","75%")],adlist)  
      data_30C_csv=data.frame(WL_label,qt_data,adlist)      
    }  
## produce additional table of % impact reduction    
    if(varlist[j]=="DamR 2100" | varlist[j]=="PopR 2100"){ 
      qt_data2= t(apply(EU_data_all2, 1, function(x) c(quantile(x,probs=c(0.1,0.25,0.5,0.75,0.9)),mean(x)) ))    
      rownames(qt_data2)=adlist2[]    
      if(flag_WL=="1.5C"){
        WL_label<-rep("1.5?C",length(adlist2))      
        data_15C_csv2=data.frame(WL_label,qt_data2,adlist2)      
      }else if(flag_WL=="2C"){
        WL_label<-rep("2?C",length(adlist2))
        data_20C_csv2=data.frame(WL_label,qt_data2,adlist2)      
      }else if(flag_WL=="3C"){
        WL_label<-rep("3?C",length(adlist2))       
        data_30C_csv2=data.frame(WL_label,qt_data2,adlist2)      
      }     
    }    
    
#    write.csv(EU_data_all, file = paste0(workDir,"/results_EU28_cost_ud_all.csv"),row.names=TRUE)
    
  } #close here loop over WLS   
  ## create data structure for new graphs

  data_p=rbind(data_15C,data_20C,data_30C)
  data_p_csv=rbind(data_15C_csv,data_20C_csv,data_30C_csv)  
  ## save EU28 tables
  write.csv(data_p_csv, file = paste0(workDir,"/results_EU28_",varlist_out[j],".csv"),row.names=TRUE)  
  if(varlist[j]=="DamR 2100"){  
    data_p_csv2=rbind(data_15C_csv2,data_20C_csv2,data_30C_csv2)
    write.csv(data_p_csv2, file = paste0(workDir,"/results_EU28_EAD-r 2100.csv"),row.names=TRUE)    
  } else if(varlist[j]=="PopR 2100"){  
    data_p_csv2=rbind(data_15C_csv2,data_20C_csv2,data_30C_csv2)
    write.csv(data_p_csv2, file = paste0(workDir,"/results_EU28_EAPE-r 2100.csv"),row.names=TRUE)    
  } 
  
  
## DRAW BARPLOT and ERROR BARS
  
## color blind friendly palette  
##         black        orange       skyblue   bluishgreen        yellow 
##     "#000000"     "#E69F00"     "#56B4E9"     "#009E73"     "#F0E442" 
##          blue    vermillion reddishpurple          gray 
##     "#0072B2"     "#D55E00"     "#CC79A7"     "#999999"
# colorBlindGrey8 <- c("#999999","#E69F00","#56B4E9","#009E73","#F0E442","#0072B2","#D55E00","#CC79A7")
# scales::show_col(colorBlindGrey8)
  
## check out https://colorbrewer2.org/ for additional palettes (e.g. monochomatic)  
  ##         black        orange       skyblue   bluishgreen        yellow 
  ##     "#000000"     "#E69F00"     "#56B4E9"     "#009E73"     "#F0E442" 
  ##          blue    vermillion reddishpurple          gray 
  ##     "#0072B2"     "#D55E00"     "#CC79A7"     "#999999"
  # colorBlindGrey8 <- c("#999999","#E69F00","#56B4E9","#009E73","#F0E442","#0072B2","#D55E00","#CC79A7")
  # scales::show_col(colorBlindGrey8)  
  
  ## Barplot with error bars using function geom_errorbar() 
  qualityFig<-3
  if(varlist[j]=="DamR 2100"){
    figName=paste(workDir,"/damRed_EU28_v",figV,sep="")
    ylabName="expected annual damage [M \u20ac]"
    leg_list<-c("no adaptation","relocation","flood proofing","raising dykes","retention areas")
#    col_list<-c("firebrick3","darkorange3","gold3", "lightskyblue1","darkgreen")  # standard palette
#    col_list<-c("#D55E00", "#56B4E9", "#009E73", "#F0E442", "#0072B2")      # color blind friendly palette 
    col_list<-c("#045a8d", "#f1eef6", "#bdc9e1", "#74a9cf", "#2b8cbe")  # blue palette
  }else if(varlist[j]=="PopR 2100"){
    figName=paste(workDir,"/PopRed_EU28_v",figV,sep="")
    ylabName="expected annual population exposed"    
    leg_list<-c("no adaptation","relocation","raising dykes","retention areas")    
#    col_list<-c("firebrick3","darkorange3", "lightskyblue1","darkgreen") 
#    col_list<-c("#D55E00", "#56B4E9", "#F0E442", "#0072B2") 
    col_list<-c("#045a8d", "#f1eef6", "#74a9cf", "#2b8cbe")  # blue palette       
  }  else if(varlist[j]=="Ben opt"){
    figName=paste(workDir,"/BCR_EU28_v",figV,sep="")
    ylabName="benefit to cost ratio"  
    leg_list<-c("raising dykes","retention areas","flood proofing","relocation")   
#    col_list<-c( "lightskyblue1","darkgreen","gold3","darkorange3")  
#    col_list<-c("#F0E442", "#0072B2", "#009E73", "#56B4E9")    
    col_list<-c( "#74a9cf", "#2b8cbe", "#bdc9e1","#f1eef6")  # blue palette  
  } else if(varlist[j]=="Cost undis"){
    figName=paste(workDir,"/Cost_EU28_v",figV,sep="")
    ylabName="Avg annual undiscounted costs [M \u20ac]"  
    leg_list<-c("raising dykes","retention areas","flood proofing","relocation")
#    col_list<-c( "lightskyblue1","darkgreen","gold3","darkorange3") 
#    col_list<-c("#F0E442", "#0072B2", "#009E73", "#56B4E9")     
    col_list<-c( "#74a9cf", "#2b8cbe", "#bdc9e1","#f1eef6")  # blue palette     
  }
  
#  png(filename = paste(figName,".png",sep=""), width = (800*qualityFig), height = (400*qualityFig), res=(100*qualityFig))        
  
  pdf(filename = paste(figName,".pdf",sep=""), width = (800*qualityFig), height = (400*qualityFig), res=(100*qualityFig))        
  
  # Default bar plot
  p<- ggplot(data_p, aes(x=WL_label, y=X50., fill=AD_label)) + 
    geom_bar(stat="identity", color="black", 
             position=position_dodge()) +
    geom_errorbar(aes(ymin=X25., ymax=X75.), width=.2,
                  position=position_dodge(.9)) +
    ylab(ylabName) + xlab(NULL) + 
#    theme_bw() # white background and gray grid lines
    theme_minimal( base_size = 14)+
    theme(legend.title = element_blank())+
#    scale_fill_discrete(labels=leg_list) +
    scale_fill_manual(values = col_list )

  #    theme(axis.text = element_text( size=15),
  #                  legend.text = element_text(size = 8),
  #                 legend.title = element_text(face = "bold"))
    print(p)
   dev.off()  # to close print device (png or other)  
   
}    

